/**
 * OhDataPackDoubleArray.java
 *
 * Created: 2004/01/30
 *
 * @author shiiba
 * @version $Revision$
 *
 * @(#) OhDataPackDoubleArray; class to express datapacks of type double arrays.
 */

package ohymosj;

import java.util.*;
import java.io.*;


/**
 * OhDataPackDoubleArray.
 *
 *
 * <japanese>
 * <p>
 * NX OhDataPackDoubleArray ́Adouble[] ^̃f[^l
 * f[^pbN^`NXłB<p>
 * </japanese>
 *
 * 
 * @version $Revision$ $Date$
 */
public class OhDataPackDoubleArray extends OhDataPack
  implements Serializable, Cloneable
{

  // @begin{field} ---------------------------------------------

  /**
   * datapack of type double[]
   *
   */
  private final double[] _dataValue;

  /**
   * size of a data array
   */
  private final int _size;
  
  // @end{field} -----------------------------------------------

  // @begin{constructors} --------------------------------------

  /**
   * Creates a new <code>OhDataPackDoubleArray</code> instance.
   *
   */
  public OhDataPackDoubleArray() { _size = 0; _dataValue = null; }

  /**
   * Creates a new <code>OhDataPackDoubleArray</code> instance.
   *
   * @param aDataTime an <code>OhTimePointImmutable</code> value
   * @param aValue a <code>double[]</code> value
   */
  public OhDataPackDoubleArray(
    OhTimePointImmutable aDataTime,
    double[] aValue) {
    _dataTime.copy(aDataTime);
    _dataValue = new double[aValue.length];
    System.arraycopy(aValue, 0, _dataValue, 0, aValue.length);
    _size = aValue.length;
  }
  
  /**
   * Creates a new <code>OhDataPackDoubleArray</code> instance.
   *
   * <japanese>
   * <p>
   * aDataStr Ɋi[Ăf[^pbN񂩂f[^pbN
   * 𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackDoubleArray(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;

    double[] dataValue = null;
    int size = 0;

    try {    
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataTime.setFrac( Double.parseDouble(br.readWord()) );
      size = Integer.parseInt(br.readWord());
      dataValue = new double[size];
      for (int i = 0; i < size; i++) {
        dataValue[i] = Double.parseDouble(br.readWord());
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }

    _size = size;
    _dataValue = new double[_size];
    for (int i = 0; i < size; i++) {
      _dataValue[i] = dataValue[i];
    }
    
  }
    
  // @end{constructors} ----------------------------------------

  // @begin{implementation of abstract methods} ----------------

  /**
   * Returns the String "doubleArray"
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "doubleArray";
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B<p>
   *
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂
   * Ĉsɏo܂Bs̍ŌɉstȂ
   * ŉB
   *
   * <pre>
   * [EXAMPLE]
   *
   * 2003 38600 0.3 2 1.1 2.2
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() + " " + _dataTime.getTsec() +
             " " + _dataTime.getFrac());
    os.print(" " + _dataValue.length);
    for (int i = 0; i < _dataValue.length; i++) {
      os.print(" " + _dataValue[i]);
    }
  }
  
  /**
   * <japanese>
   * <p>
   * f[^pbNptH[}bgŃt@Cɏo
   * B<p>
   *
   * ptH[}bgƂ́A_year, _tsec, _dataValue ̂
   * ijʂł̂ЂƂsďo
   * ܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * (2003)
   * (38600)
   * (3.1)
   * (2)
   * (1.1)
   * (2.2)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataTime.getFrac() + ")" + NEWLINE);
    os.print("(" + _dataValue.length + ")" + NEWLINE);
    for (int i = 0; i < _dataValue.length; i++) {
      os.print("(" + _dataValue[i] + ")" + NEWLINE);
    }
  }

  // @end{implementation of abstract methods} ------------------

  // @begin{methods} -------------------------------------------

  public int getSize() {
    return _size;
  }

  /**
   * Returns a clone of the data value of type double[].
   * <p>
   *
   * <japanese>
   * <p>
   * f[^l (double ̔z)  clone Ԃ܂B
   * </japanese>
   *
   * @return a <code>double[]</code> value
   */
  public double[] getDataValue() {
    double[] t = (double[]) _dataValue.clone();
    return t;
  }

  // @end{methods} ---------------------------------------------

}// OhDataPackDoubleArray
