/**
 * @(#) Class to express datapacks of type float.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * OhDataPackFloat;
 *
 * OhDataPackFloat is a class of datapacks of type float. <p>
 *
 * OhDataPackFloat is derived from <a href="OhDataPack.html">OhDataPack</a>. <p>
 *
 * A datapack of type float is a pair of data of data time and
 * data of type float.
 *
 * <japanese>
 * <p>
 * <a href="OhDataPack.html">OhDataPack</a> pč쐬Afloat ^f[^pbNN
 * XłB
 *
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.5 $ $Date: 2004/01/18 14:18:54 $
 */
public class OhDataPackFloat extends OhDataPack
{
  // @begin{field} ---------------------------------------------

  /**
   * <japanese>
   * <p>
   * float ^̃f[^i[郁oϐłB
   * </japanese>
   */
  float _dataValue;

  // @end{field} -----------------------------------------------
  

  // @begin{constructors} --------------------------------------

  /**
   * Creates a new default <code>OhDataPackFloat</code> instance.
   *
   */
  public OhDataPackFloat() {}
  
  /**
   * Creates a new <code>OhDataPackFloat</code> instance.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float</code> value
   */
  public OhDataPackFloat(OhTimePoint aDataTime, float aDataValue) {
    _dataTime  = aDataTime;    //; Field of the super class OhDataPack.
    _dataValue = aDataValue;
  }

  /**
   * Creates a new <code>OhDataPackFloat</code> instance.
   *
   * <japanese>
   * <p>
   * _dataStr Ɋi[Ăf[^pbN񂩂f[^pbN
   * ̒lǂݎA_dataTime, _dataValue Ɋi[ꂽIuWF
   * Ng𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackFloat(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;

    try {
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataValue = Float.parseFloat(br.readWord());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }
    
  // @end{constructors} ------------------------------------------------  

  // @begin{methods} ---------------------------------------------------  

  // 0. copy method.

  /**
   * Copy method.
   *
   * @param aDpd an <code>OhDataPackFloat</code> value
   */
  public void copy(OhDataPackFloat aDpd) {
    _dataTime  = aDpd._dataTime;
    _dataValue = aDpd._dataValue;
  }
  
  
  /**
   * Returns the data value.
   *
   * @return a <code>float</code> value
   */
  public float getDataValue() {
    return _dataValue;
  }

  /**
   * Returns the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float[]</code> value
   */
  public void getData(OhTimePoint aDataTime, float[] aDataValue) {
    aDataTime.copy(_dataTime);
    aDataValue[0] = _dataValue;
  }
  
  /**
   * Sets the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float</code> value
   */
  public void setData(OhTimePoint aDataTime, float aDataValue)
    {
      _dataTime  = aDataTime;    //; "setDataTime(aDataTime);"
      _dataValue = aDataValue;
    }

  /**
   * Returns the data type "float".
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "float";
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B<p>
   *
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂
   * Ĉsɏo܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * 2003 38600 3.1
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() + " " + _dataTime.getTsec() +
      " " + _dataValue);
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNptH[}bgŃt@Cɏo
   * B<p>
   *
   * ptH[}bgƂ́A_year, _tsec, _dataValue ̂
   * ijʂł̂ЂƂsďo
   * ܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * (2003)
   * (38600)
   * (3.1)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataValue + ")" + NEWLINE);
  }

  // @end{methods} ---------------------------------------------
  
} // OhDataPackDoule
