/**
 * @(#) Class to express datapacks of type int.
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * OhDataPackInt is a class of datapacks of type int. <p>
 *
 * OhDataPackInt is derived from <a href="OhDataPack.html">OhDataPack</a>. <p>
 *
 * A datapack of type int is a pair of data of data time and
 * data of type int.
 *
 * <japanese>
 * <p>
 * <a href="OhDataPack.html">OhDataPack</a> pč쐬Aint ^f[^pbNN
 * XłB
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.7 $ $Date: 2004/01/18 14:18:54 $
 */
public class OhDataPackInt extends OhDataPack
  implements Serializable, Cloneable, OhDataPackIntImmutable
{
  // @begin{field} ---------------------------------------------

  /**
   * Field variable 'dataValue'.
   *
   * <japanese>
   * <p>
   * int ^̃f[^i[郁oϐłB
   * </japanese>
   */
  private int _dataValue;

  // @end{field} -----------------------------------------------
  

  // @begin{constructors} --------------------------------------

  /**
   * Creates a new default <code>OhDataPackInt</code> instance.<p>
   *
   * <japanese>
   * ftHgRXgN^Bf[^́Anew OhTimePoint() 
   * ɐݒ肳Af[^ĺA0 ɐݒ肳܂B
   * </japanese>
   *
   */
  public OhDataPackInt() {
    super();
    _dataValue = 0;
  }
  
  /**
   * Creates a new <code>OhDataPackInt</code> instance.
   *
   * @param aDataTime an <code>OhTimePointImmutable</code> value
   * @param aDataValue a <code>int</code> value
   */
  public OhDataPackInt(OhTimePointImmutable aDataTime, int aDataValue) {
    _dataTime.copy(aDataTime);
      // Field of the super class OhDataPack.
    _dataValue = aDataValue;
  }

  /**
   * Creates a new <code>OhDataPackInt</code> instance.
   *
   * <japanese>
   * <p>
   * _dataStr Ɋi[Ăf[^pbN񂩂f[^pbN
   * ̒lǂݎA_dataTime, _dataValue Ɋi[ꂽIuWF
   * Ng𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackInt(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;

    try {
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataTime.setFrac( Double.parseDouble(br.readWord()) );
      _dataValue = Integer.parseInt(br.readWord());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }
    
  // @end{constructors} ------------------------------------------------  

  // @begin{methods} ---------------------------------------------------  

  // 0. copy method.

  /**
   * Copy method.
   *
   * @param dpi an <code>OhDataPackInt</code> value
   */
  public final void copy(OhDataPackIntImmutable dpi) {
    _dataTime.copy(dpi.getDataTime());
    _dataValue = dpi.getDataValue();
  }
  
  /**
   * Returns a deep copy of the current OhDataPackInt object<p>
   *
   * <japanese>
   * ݂ OhDataPackInt IuWFNg̃N[ ([Rs[) 
   * Ԃ܂B
   * </japanese>
   *
   * @return an <code>Object</code> value
   */
  public Object clone() {
    OhDataPackInt dp = new OhDataPackInt(_dataTime, _dataValue);
    return dp;
  }

  /**
   * Returns the data value.
   *
   * @return a <code>int</code> value
   */
  public int getDataValue() {
    return _dataValue;
  }

  /**
   * Sets the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>int</code> value
   */
  public void setData(OhTimePoint aDataTime, int aDataValue)
    {
      _dataTime  = aDataTime;    //; "setDataTime(aDataTime);"
      _dataValue = aDataValue;
    }

  /**
   * Returns the data type "int".
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "int";
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B<p>
   *
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂
   * Ĉsɏo܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * 2003 38600 3
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() +
      " " + _dataTime.getTsec() +
      " " + _dataTime.getFrac() +
      " " + _dataValue);
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNptH[}bgŃt@Cɏo܂B<p>
   *
   * ptH[}bgƂ́A_year, _tsec, _dataValue ̂
   * ijʂł̂ЂƂsďo
   * ܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * (2003)
   * (38600)
   * (3)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataTime.getFrac() + ")" + NEWLINE);
    os.print("(" + _dataValue + ")" + NEWLINE);
  }

  // @end{methods} ---------------------------------------------
  
} // OhDataPackDoule
