// @(#) class OhDataPackIntTest

package ohymosj;

import junit.framework.TestCase;
import java.io.*;

public class OhDataPackIntTest extends TestCase
{
  public OhDataPackIntTest(String name) {
    super(name);
  }
  
  public void setUp() {
  }
  
  public void tearDown() {
  }

  // @begin{constructors check} ----------------------------------

  public void testOhDataPackIntVoid() {
    OhDataPackInt a = new OhDataPackInt();
    assertEquals("testOhDaaPackIntVoidA",
      true, a.getDataTime().equals(new OhTimePoint()));
    assertEquals("testOhDaaPackIntVoidB",
      0., a.getDataValue(), 0.0001);
  }

  public void testOhDataPackIntTimePointInt() {
    OhTimePoint tp = new OhTimePoint(2003, 10, 0.1);
    OhDataPackInt a = new OhDataPackInt(tp, 2);
    assertEquals("testOhDataPackIntTimePointIntA",
      true, a.getDataTime().equals(tp));
    assertEquals("testOhDataPackIntTimePointIntB", 2, a.getDataValue());
  }

  public void testOhDataPackIntString() {
    String str = "2003 10 0.1 2";
    OhDataPackInt a = new OhDataPackInt(str);
    OhTimePointImmutable tp = a.getDataTime();
    assertEquals("testOhDataPackIntTimePointString",
      2003, tp.getYear());
    assertEquals("testOhDataPackIntTimePointString",
      10, tp.getTsec());
    assertEquals("testOhDataPackIntTimePointString",
      0.1, tp.getFrac(), 0.0001);
    assertEquals("testOhDataPackIntTimePointString",
      2, a.getDataValue());
  }
  
  // @end{constructors check} ------------------------------------

  // @begin{methods check} ---------------------------------------

  public void testCopy() {
    OhDataPackInt dpa = new OhDataPackInt("2003 10 0.1 2");
    OhDataPackInt dpb = new OhDataPackInt();
    dpb.copy(dpa);
    assertEquals("testCopy", 2003, dpb.getDataTime().getYear());
    assertEquals("testCopy", 10, dpb.getDataTime().getTsec());
    assertEquals("testCopy", 0.1, dpb.getDataTime().getFrac(), 0.0001);
    assertEquals("testCopy", 2, dpb.getDataValue());
  }

  /**
   * test of
   *   public Object clone()
   *
   */
  public void test_clone() {
    OhDataPackInt dp = new OhDataPackInt(
      new OhTimePoint(2003, 10, 0.0), 2);
    OhDataPackInt dpc = (OhDataPackInt) dp.clone();
    assertEquals("test_clone", 2, dpc.getDataValue());
    assertEquals("test_clone", true,
      dpc.getDataTime().equals(new OhTimePoint(2003, 10, 0.0)));
  }
  
  /**
   * test of
   *  public double getDataValue()
   */
  public void test_getDataValue() {
    OhDataPackInt dp = new OhDataPackInt(
      new OhTimePoint(2003, 10, 0.0), 2);
    assertEquals("test_getDataValue", 2, dp.getDataValue());
  }
  
  /**
   * test
   * public void setData(OhTimePointImmutable aDataTime, double aDataValue)
   *
   */
  public void test_setData() {
    OhDataPackInt dpd = new OhDataPackInt();
    OhTimePoint tp = new OhTimePoint(2003, 10, 0.0); 
    dpd.setData(tp, 2);
    assertEquals("test_setData", true, dpd.getDataTime().equals(tp));
    assertEquals("test_setData", 2, dpd.getDataValue());
  }
  
  /**
   * test of
   *   public String dataType() 
   * 
   */
  public void test_dataType() {
    OhDataPackInt dpd = new OhDataPackInt();
    assertEquals("test_dataType", (Object)"int", dpd.dataType());
  }

  public void test_dpack2Ofile() {
    OhDataPackInt dpa = new OhDataPackInt("2003 10 0.1 2");
    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    dpa.dpack2Ofile(out);
    String result = buffer.toString();
    assertEquals("test_dpack2Ofile", (Object)"2003 10 0.1 2", result);
  }
  
  /**
   * test of
   *   public void dpack2File(PrintWriter os)
   *
   */
  public void testDpack2File() {
    String newline = System.getProperty("line.separator");

    OhDataPackInt dpa = new OhDataPackInt("2003 10 0.1 2");

    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    dpa.dpack2File(out);
    String result = buffer.toString();
    String expected = "(2003)" + newline +
      "(10)" + newline +
      "(0.1)" + newline +
      "(2)" + newline;
    assertEquals("testDpack2File", expected, result);
  }
  
  // @end{methods check} -----------------------------------------
}

