/**
 * @(#) Class to express datapacks of type long.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * OhDataPackLong;
 *
 * OhDataPackLong is a class of datapacks of type long. <p>
 *
 * OhDataPackLong is derived from <a href="OhDataPack.html">OhDataPack</a>. <p>
 *
 * A datapack of type long is a pair of data of data time and
 * data of type long.
 *
 * <japanese>
 * <p>
 * <a href="OhDataPack.html">OhDataPack</a> pč쐬Along ^f[^pbNN
 * XłB
 *
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.5 $ $Date: 2004/01/18 14:18:54 $
 */
public class OhDataPackLong extends OhDataPack
{
  // @begin{field} ---------------------------------------------

  /**
   * <japanese>
   * <p>
   * long ^̃f[^i[郁oϐłB
   * </japanese>
   */
  long _dataValue;

  // @end{field} -----------------------------------------------
  

  // @begin{constructors} --------------------------------------

  /**
   * Creates a new default <code>OhDataPackLong</code> instance.
   *
   */
  public OhDataPackLong() {}
  
  /**
   * Creates a new <code>OhDataPackLong</code> instance.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>long</code> value
   */
  public OhDataPackLong(OhTimePoint aDataTime, long aDataValue) {
    _dataTime  = aDataTime;    //; Field of the super class OhDataPack.
    _dataValue = aDataValue;
  }

  /**
   * Creates a new <code>OhDataPackLong</code> instance.
   *
   * <japanese>
   * <p>
   * _dataStr Ɋi[Ăf[^pbN񂩂f[^pbN̒lǂݎA
   * _dataTime, _dataValue Ɋi[ꂽIuWFNg𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackLong(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;

    try {    
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataValue = Long.parseLong(br.readWord());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }
    
  // @end{constructors} ------------------------------------------------  

  // @begin{methods} ---------------------------------------------------  

  // 0. copy method.

  /**
   * Copy method.
   *
   * @param aDpd an <code>OhDataPackLong</code> value
   */
  public void copy(OhDataPackLong aDpd) {
    _dataTime  = aDpd._dataTime;
    _dataValue = aDpd._dataValue;
  }
  
  
  /**
   * Returns the data value.
   *
   * @return a <code>long</code> value
   */
  public long getDataValue() {
    return _dataValue;
  }

  /**
   * Returns the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>long[]</code> value
   */
  public void getData(OhTimePoint aDataTime, long[] aDataValue) {
    aDataTime.copy(_dataTime);
    aDataValue[0] = _dataValue;
  }
  
  /**
   * Sets the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>long</code> value
   */
  public void setData(OhTimePoint aDataTime, long aDataValue)
    {
      _dataTime  = aDataTime;    //; "setDataTime(aDataTime);"
      _dataValue = aDataValue;
    }

  /**
   * Returns the data type "long".
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "long";
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B<p>
   *
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂
   * Ĉsɏo܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * 2003 38600 4907
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() + " " + _dataTime.getTsec() +
      " " + _dataValue);
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNptH[}bgŃt@Cɏo܂B<p>
   *
   * ptH[}bgƂ́A_year, _tsec, _dataValue ̂ꂼijʂł̂
   * ЂƂsďo܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * (2003)
   * (38600)
   * (4907)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataValue + ")" + NEWLINE);
  }

  // @end{methods} ---------------------------------------------
  
} // OhDataPackDoule
