/**
 * @(#) OhInputFileFloat; Derived from OhInputFile.
 *
 * <a href="OhObject.html">OhObject</a>
 * <-- <a href="OhPort.html">OhPort</a>
 * <-- <a href="OhSenderPort.html">OhSenderPort</a>
 * <-- <a href="OhSendPort.html">OhSendPort</a>
 * <-- <a href="OhInputPort.html">OhInputPort</a>
 * <-- <a href="OhInputFile.html">OhInputFile</a>
 */

package ohymosj;

import java.util.*;
import java.io.*;


/**
 * The OhInputFileFloat class is a class of input port which
 * reads in datapacks of type float from an input file and sends
 * them. The OhInputFileFloat class extends (Ref"OhInputFile").<p>
 *
 * OhInputFileFloat is a class of input port
 * which reads in datapacks of type float from an input file and sends them. <p>
 *
 * OhInputFileFloat can be connected with
 * <a href="OhReceiveFloat.html">OhReceiveFloat</a>,
 * <a href="OhRelayFloat.html">OhRelayFloat</a>, and 
 * <a href="OhOutputFileFloat.html">OhOutputFileFloat</a>. <p>
 *
 * OhInputFileFloat reads an input file using "simple data
 * format for input and output." <p>
 *
 * "Simple data format for input and output" is a format in which
 *
 * <pre>
 *  .....
 *    all the data items for a datapack should be written in one
 *    line, and data items should be separated with a number of
 *    spaces or tabs.
 *
 *    Example: 2003 120 0.3 1.2 2.3 4.5
 * ....
 * </pre>
 *
 * If you need different format, please make a derived class of
 * OhInputFileFloat and redefine "makeDataPack()".
 *
 * <japanese>
 * <p>
 * t@C float ^̃f[^pbNǂݎ͒[q <p>
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.1 $ $Date: 2004/01/17 13:03:35 $
 */
public class OhInputFileFloat extends OhInputFile
{
  // @begin{initialization} ------------------------------------

  /**
   * Initializ the object. This method assigns the argument
   * aFileName to the instance variable
   * <a href="OhObject.html#field_summary">OhObject._className</a>.<p>
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void init(String aFileName) throws OhError {
    super.init(aFileName);
  }

  // @end{initialization} --------------------------------------

  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>OhInputFileFloat</code> instance.
   *
   * @exception OhError if an error occurs
   */
  public OhInputFileFloat() throws OhError {
    super();
    _className = "OhInputFileFloat";
  }


  /**
   * Creates a new <code>OhInputFileFloat</code> instance.
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhInputFileFloat(String aFileName) throws OhError {
    super(aFileName);
    _className = "OhInputFileFloat";
  }

  // @end {constructors} ---------------------------------------

  
  // @begin {methods} ------------------------------------------

  // 1. connection function.

  /**
   * Returns the data pack type which expresses the type of data
   * to be received.  This returns "float".
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "float";
  }


  // 2. data input function

  /**
   * Reads string datapacks of type float, creates datapacks of
   * type float, and returns it.
   *
   * <japanese>
   * <p>
   *  aLineData Ɋi[Ăf[^pbN񂩂f[^
   * pbN쐬AObject ^ɃLXgĖ߂܂B<p>
   *
   * ygpz<p>
   * Object dataPack = makeDataPack("2003 12 0.3 3.5");<p>
   *
   * ̗ႾƁAf[^ OhTimePoint(2003, 12, 0.3)AɃf[^
   * l 3.5 f[^pbNǂݎ܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   * @return an <code>OhDataPack</code> value
   */
  public OhDataPack makeDataPack(String aDataStr) {
    OhDataPackFloat dp = new OhDataPackFloat(aDataStr);
    return dp;
  }


  /**
   * Sends datapacks of type float which is stored in the form
   * of string.
   *
   * <japanese>
   * <p>
   *  aStr Ɋi[Ăf[^pbN񂩂f[^pb
   * N쐬A͒[qɑM܂B<p>
   *
   * \bh̒` <a href="OhSendPort.html#method_summary">OhSendPort#dataPackStock()</a> gp
   * Ă܂B<p>
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public void sendByString(String aDataStr) {
    OhDataPackFloat dp = new OhDataPackFloat(aDataStr);
    dataPackStock(dp);
  }

  // @end {methods} --------------------------------------------

} // OhInputFileFloat
