/**
 * @(#) OhInterface; base class for OHyMoSJ interface.
 */

package ohymosj;

import java.util.*;
import java.io.*;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.text.BadLocationException;

/**
 * OhInterface;
 *
 * @author satoh, shiiba
 * @version $Revision: 1.15 $ $Date: 2004/01/18 15:13:42 $
 */
public class OhInterface {

  /**
   * Gets reply.
   *
   * <japanese>
   * <br>
   * Window ʂďƂ肷Iꍇ́A
   * ̃̕bZ[W͗p_CAO{bN
   * X <a href="OhWin.html">OhWin</a> \܂B<p>
   *
   * getReply \bh́A͂āuv{^
   * A̕Ԃ܂B󔒁E󂫕F܂B
   * uv܂͉É~{^ꍇ́Anull Ԃ
   * ܂B
   *
   * </japanese>
   *
   *
   * @param aMessage a <code>String</code> value
   * @param aDefaultReply a <code>String</code> value
   * @return a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public static String getReply(
    String aMessage, String aDefaultReply) throws OhError {

    Object obj = null;
    String rep = null;
    
    if (OhOption.getDefault().getUseWinP()) {
      
      // @begin{windows} ---------------------------------------
    
      ImageIcon myIcon = new ImageIcon("ohymos.png");
      
      obj = JOptionPane.showInputDialog(
        OhWin.getDefault(),           //; Parent frame.
        aMessage,                     //; Message.
        "getReply",                   //; Title.
        JOptionPane.PLAIN_MESSAGE,    //; Message type.
        myIcon,                       //; HY icon.
        null,
        aDefaultReply                 //; Default reply.
      );

      if (obj == null)
        throw new OhError("Quit.");

      rep = obj.toString();
      
      // @end{windows} -----------------------------------------
    }    
    else {
      System.out.print(aMessage);

      InputStreamReader isr = new InputStreamReader(System.in);
      BufferedReader  br  = new BufferedReader(isr);

      try {
        rep = br.readLine();
      }
      catch (IOException e) {
        throw new OhError("Quit.");
      }
    }

    if (rep.equals("^C"))
      throw new OhError("Quit.");
    
    return rep;
  }


  /**
   * Pause.
   *
   * <japanese>
   * <br>
   * Window ʂďƂ肷Iꍇ́A
   * ̃̕bZ[W <a href="OhWin.html">OhWin</a> \܂B
   * ̃EBhÉAuOKvƕ\ꂽ{^
   * ܂BuOKv{^ƁAɖ߂܂B
   * </japanese>
   *
   * @param aMessage a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public static void pause(String aMessage) {

    if (OhOption.getDefault().getUseWinP()) {
      // @begin{windows} ---------------------------------------
      
      ImageIcon myIcon  = new ImageIcon("ohymos.png");
      JLabel    myLabel = new JLabel(myIcon);  // Makes KH logo.

      Object[] bts = {myLabel, "OK"};
      // buttons placed on the pane
      
      int ret = JOptionPane.showOptionDialog(
        OhWin.getDefault(),        // Parent frame.
        aMessage,                  // Message.
        "OhPause",                 // Title.
        JOptionPane.DEFAULT_OPTION,
        JOptionPane.PLAIN_MESSAGE,
        null,
        bts, bts[1]);
      

      if (ret == 1 || ret == -1) {    // "OK" button was clicked
        return;
      }
      // @end{windows} -----------------------------------------
    }
    else {
      System.out.println(aMessage);
      System.out.println("Hit return key: ");

      InputStreamReader isr = new InputStreamReader(System.in);
      BufferedReader  br  = new BufferedReader(isr);

      try {
        String s = br.readLine();
      }
      catch (IOException e) {
        ;
        // throw new OhError("Quit.");
      }
    }
  }

} // OhInterface
