/**
 * @(#) OhOption; Declaration of OHyMoSJ option variables.
 */

package ohymosj;

import ohymosj.*;
import java.io.*;

/**
 * The OhOption class treats specification of general options
 * for simulation.
 *
 * <japanese>
 * <p>
 * OHyMoSJ ɂV~[VŜ̃IvV <p>
 *
 * OHyMoSJ ɂV~[VŜɊւIvV̎w
 * 舵܂BIvV́ÃNX̃oϐ̒lŎw
 * ܂B̓Iɂ́A<p>
 *
 * <ul>
 *  <li> <a href="TimeControl.html">TimeControl</a> ɊւIvV
 *  <li> GUI ̎gpɊւIvV
 * </ul>
 *
 * 舵Ă܂B<p>
 *
 * ftHg̃IvV́A
 *
 * <pre>
 *  ETimeControl _tc = TimeControl.GO_BEYOND_TERMINAL_TIME;
 *    ԖڕW𒴂ČvZB
 *  Eboolean _useWinP = false;
 *    GUI gpȂB
 * </pre>
 *
 * ̂悤Ɏw肳Ă܂B<p>
 *
 * ܂ÃNX singlton NXłB
 * OhOption  singlton 擾ɂ́A
 *
 * <pre>
 *  OhOption.getDefault().<method>;
 * </pre>
 *
 * Ƃ܂B<p>
 *
 * OhTotalSystem.setOhymosjOption() ̗͂łāAvO
 * s[UC^NeBuɃIvVw肷郁\b
 * hłB
 *
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.11 $ $Date: 2004/01/19 07:12:15 $
 */
public class OhOption
{
  /**
   * The singlton of the OHyMoSJ option.
   *
   * <japanese>
   * <p>
   * OHyMoSJ  singlton.
   * </japanese>
   *
   */
  private static OhOption instance = null;

  /**
   * Option about use of windows.
   *
   * <japanese>
   * <p>
   * GUI ̎gpɊւIvV <p>
   *
   * ftHǵA
   *
   * <pre>
   *  _useWinP = false;
   * </pre>
   *
   * Ȃ킿AGUI gpȂ悤ɐݒ肳Ă܂B
   * 
   * </japanese>
   *
   */
  protected boolean _useWinP = false;

  /**
   * Option about <a href="OhTimeControl.html">OhTimeControl</a>.
   *
   * <japanese>
   * <p>
   * <a href="OhTimeControl.html">OhTimeControl</a> ɊւIvV
   *
   * ftHǵA
   *
   * <pre>
   *  _tc = OhTimeControl.GO_BEYOND_TERMINAL_TIME;
   * </pre>
   *
   * Ȃ킿AԖڕW𒴂ČvZ悤ɐݒ肳Ă
   * B
   * </japanese>
   *
   */
  protected TimeControl _tc = TimeControl.GO_BEYOND_TERMINAL_TIME;

  /**
   * Creates a new default <code>OhOption</code> instance.
   *
   */
  protected OhOption () {
    super();
  }

  /**
   * Returns the OhOption object singleton.
   *
   * <japanese>
   * <p>
   * OhOption  singlton 擾܂BNX\bhłB
   * </japanese>
   *
   * @return an <code>OhOption</code> value
   */
  public static OhOption getDefault() {
    if (instance == null) {
      instance = new OhOption();
    }
    return instance;
  }

  /**
   * Specify whether GUI is used or not.
   *
   * <japanese>
   * <p>
   * GUI ̎gpɊւIvV̐ݒ <p>
   *
   * ftHgł GUI gpȂ悤ɐݒ肳Ă̂ŁA
   *  aUseWinP  true w肵Ă
   * GUI gp[hɂȂ܂B
   * </japanese>
   *
   * @param aUseWinP a <code>boolean</code> value
   */
  public void setUseWinP(boolean aUseWinP) {
    _useWinP = aUseWinP;
  }
  
  /**
   * Returns the status of option specification about GUI.
   *
   * <japanese>
   * <p>
   * GUI ̎gpɊւIvVԂB <p>
   *
   * GUI gpȂꍇ falseAGUI gpꍇ true Ԃ܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   */
  public boolean getUseWinP() {
    return _useWinP;
  }
  
  /**
   * Specify the option about treatment of time.
   *
   * <japanese>
   * <p>
   * <a href="OhTimeControl.html">OhTimeControl</a> ɊւIvV̐ݒ <p>
   *
   * ftHgł͒ԖڕW𒴂ČvZ悤ɐݒ肳
   * ̂ŁA aTc 
   * <a href="OhTimeControl.html#field_summary">OhTimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME</a> 
   * w肵ĂƒԖڕW𒴂ɌvZ悤ɐݒ肳܂B
   * </japanese>
   *
   * @param aTc a <code>TimeControl</code> value
   */
  public void setTimeControlOption(TimeControl aTc) {
    _tc = aTc;
  }
  
  /**
   * Returns the specification about treatment of time.
   *
   * <japanese>
   * <p>
   * <a href="TimeControl.html">TimeControl</a> ɊւIvVԂB <p>
   *
   * ԖڕW𒴂ČvZꍇ
   * <a href="TimeControl.html#field_summary">TimeControl.GO_BEYOND_TERMINAL_TIME</a>A
   * ԖڕW𒴂ɌvZꍇ
   * <a href="OhTimeControl.html#field_summary">OhTimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME</a> 
   * Ԃ܂B
   * </japanese>
   *
   *
   * @return a <code>TimeControl</code> value
   */
  public TimeControl getTimeControlOption() {
    return _tc;
  }

} // OhOption
