/**
 * @(#) OhOutputFileDouble
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * The OhOutputFileDouble class is a class of output ports which
 * receive datapacks of type double and print them in a
 * file. <p>
 *
 * OhOutputFileDouble receives double data.
 * OhOutputFileDouble can be connected with
 * <a href="OhSendDouble.html">OhSendDouble</a>, 
 * <a href="OhRelayDouble.html">OhRelayDouble</a>, and 
 * <a href="OhInputFileDouble.html">OhInputFileDouble</a>.
 *
 * <japanese>
 * <p>
 * double ^̃f[^pbNMăt@CɋL^o͒[q <p>
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.15 $ $Date: 2004/01/17 08:26:42 $
 */
public class OhOutputFileDouble extends OhOutputFile
{
  // @begin{initialization} ------------------------------------

  // @end{initialization} --------------------------------------
  

  // @begin{constructors} --------------------------------------

  /**
   * Creates a new default <code>OhOutputFileDouble</code> instance.
   *
   */
  public OhOutputFileDouble() {
    super();
    _className = "OhOutputFileDouble";
  }
  
    
  /**
   * Creates a new <code>OhOutputFileDouble</code> instance.
   *
   * @param aFileName a <code>String</code> value
   * @param aLineLength an <code>int</code> value
   */
  public OhOutputFileDouble(String aFileName, int aLineLength)
    {
    super(aFileName, aLineLength);
    _className = "OhOutputFileDouble";
  }

  /**
   * Creates a new <code>OhOutputFileDouble</code> instance.
   *
   * @param aFileName a <code>String</code> value
   */
  public OhOutputFileDouble(String aFileName) {
    super(aFileName, 128);
    _className = "OhOutputFileDouble";
  }

  // @end{constructors} ----------------------------------------

  
  // @begin{methods} -------------------------------------------

  // 1. Connection function.
  
  /**
   * Returns the datapack type, "double", which expresses the
   * type of data to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "double";
  }


  // 2. The write-in function of output files.

  /**
   * Nothing is treated.
   *
   * @param aStr a <code>String</code> value
   * @param aDataPack an <code>Object</code> value
   */
  public void makeStringData(String aStr, Object aDataPack) {}


  /**
   * Though this method is defined in the base class
   * <a href="OhOutputFile.html">OhOutputFile</a>
   * it is redifined in
   * <a href="OhOutputFileDouble.html">OhOutputFileDouble</a>
   *
   * <japanese>
   * <p>
   * o̓f[^̏ <p>
   *
   * ܂Ao̓t@C _objectName ǉ[hŃI[vA
   * I[vłȂꍇ return ܂B<p>
   * 
   * ɁAMf[^pbNz _dataPackArray ̐擪Ɋi[
   * Ăf[^pbN珇Ԃ
   * <a href="OhDataPackDouble.html#method_summary">OhDataPackDouble#dpack2Ofile(PrintWriter)</a> p
   * o̓t@CɋL܂B<p>  <p>
   *
   * Mf[^pbN͂̃\bh̒Ŏ擾܂Bf 
   * OhOutputFile ͎M[q̔hfłāAf[^pbN
   * XgbN@\͂܂B
   * </japanese>
   *
   * @exception IOException if an error occurs
   */
  public void outputData() throws IOException {
    
    OhArrayList   dpArray      = new OhArrayList();
    OhArrayList[] dpArrayArray = {dpArray};         //; For work.

    int   numOfDps      = 0;
    int[] numOfDpsArray = {numOfDps};  //; For work.

    getDataPackArray(dpArrayArray, numOfDpsArray);

    dpArray  = dpArrayArray[0];
    numOfDps = numOfDpsArray[0];

    
    // _dataPackArray is not updated.
    //
    if (((OhDataPack)dpArray.get(numOfDps - 1)).
      getDataTime().compareTo(_lastDataTime) <= 0)
      return;    


    /* first time output */

    if (_lastDataTime == OhTimePoint.OhTimePointMin)
      writeHeader();

    ////////////////////////////////////////////////
    // Additional writing.
    //
    // <japanese>
    // ݗpo̓t@C̒ǉI[vB
    // </japanese>
    //
    BufferedWriter bw = null;
    
    try {
      bw = new BufferedWriter(new FileWriter(_objectName, true));
    }
    catch(IOException e) {
      throw e;
    }


    PrintWriter pw = new PrintWriter(bw);  

    if (((OhDataPack)dpArray.get(0)).getDataTime().
      compareTo(OhTimePoint.OhTimePointMin) == 0) {
      ((OhDataPack)dpArray.get(0)).dpack2Ofile(pw);
      pw.print(NEWLINE);
    }


    int k = 0;

    // Skip the data which have been output.
    //
    for (k = 0; k < numOfDps; k++) {
      if (((OhDataPack)dpArray.get(k)).getDataTime().
        compareTo(_lastDataTime) > 0)
        break;
    }

    
    /* output the data */

    for (; k < numOfDps; k++) {
      ((OhDataPack)dpArray.get(k)).dpack2Ofile(pw);
      pw.print(NEWLINE);
    }

    pw.close();

    _lastDataTime = ((OhDataPack)dpArray.get(numOfDps - 1)).
      getDataTime();

    reduceDataPackStock();
  }

  public String basicInfo() {
    return
      "classname: " + _className + NEWLINE +
      "not an abstact class" + NEWLINE +
      "datatype it records: " + dataType();
  }

  // @end{methods} ---------------------------------------------

} // OhOutputFileDouble
