/**
 * @(#) OhOutputPort; base class for creating output ports.
 *
 * <a href="OhObject.html">OhObject</a>
 * <-- <a href="OhPort.html">OhPort</a>
 * <-- <a href="OhReceivePort.html">OhReceivePort</a>
 * <-- .
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * The OhOutputPort class is a base class for creating output
 * ports. This is an abstract class.<p>
 *
 * <japanese>
 * <p>
 * {^o͒[qf <p>
 * 
 * Šn̏o͒[q쐬邽߂̊NXłB <p>
 * ۃNXłB
 * <a href="OhReceivePort.html">OhReceivePort</a> NXhA
 * <a href="OhOutputFile.html">OhOutputFile</a> NXh܂B
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.12 $ $Date: 2004/01/16 03:54:54 $
 */
public abstract class OhOutputPort extends OhReceivePort
{
  // @begin {abstract methods} -------------------------------------------

  // 1. Connection function.

  /**
   * Returns a string which expresses the type of datapacks
   * received by the current object. This is a abstract
   * method. This function must be defined in derived classes.
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();


  // 2. The function which outputs receiving data.

  /**
   * Gets datapacks from the sender and output it.
   * This is an abstract method declaration.
   * It must be implemented in subclasses which extend
   * this <a href="OhOutputPort.html">OhOutputPort</a> class.
   *
   * <japanese>
   * <p>
   * Mf[^̏o <p>
   *
   * eo͒[qŎMf[^pbN쐬A
   * f[^o͂֐`ĉB
   * </japanese>
   *
   * @exception IOException if an error occurs
   */
  public abstract void outputData() throws IOException;

  // @end {abstract methods} ---------------------------------------------

  
  // @begin{initialization} ----------------------------------------------

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public void init(String aPortName, int aPortNum)
    throws OhError {
    super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------


  // @begin {constructors} -----------------------------------------------
  
  /**
   * Creates a new default <code>OhOutputPort</code> instance.
   *
   */
  public OhOutputPort() {
    super();
    _className = "OhOutputPort";
  }

  /**
   * Creates a new <code>OhOutputPort</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public OhOutputPort(String aPortName, int aPortNum) throws OhError {
    super();
    _className = "OhInputPort";
    init(aPortName, aPortNum);
  }

  // @end {constructors} -------------------------------------------------

} // OhOutputPort
