/**
 * @(#) OhReceiveDouble
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * The OhReceiveDouble class is a base class to express a port
 * which receives data of type double.
 *
 * <japanese>
 * <p>
 * double ^̃f[^pbNMM[q̃NX <p>
 *
 * <a href="OhReceivePort.html">OhReceivePort</a> pč쐬M[q̃NX
 * B
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.22 $ $Date: 2004/01/15 09:48:10 $
 */
public class OhReceiveDouble extends OhReceivePort
  implements OhConst
{
  // @begin{initialization} ------------------------------------
  
  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void init(String aPortName, int aPortNum)
    throws OhError {
    super.init(aPortName, aPortNum);
  }

  // @end{initialization} --------------------------------------

  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new <code>OhReceiveDouble</code> instance.
   *
   */
  public OhReceiveDouble() {
    super();
    _className = "OhReceiveDouble";
  }
  
  /**
   * Creates a new <code>OhReceiveDouble</code> port instance by
   * specifying the port name and the port number.
   *
   * <japanese>
   * |[gƃ|[gԍw肵āAOhReceiveDouble ^̎M[
   * q쐬܂B
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNumber an <code>int</code> value
   */
  public OhReceiveDouble(String aPortName, int aPortNumber) {
    super(aPortName, aPortNumber);
    _className = "OhReceiveDouble";
  }

  // @end {constructors} ---------------------------------------

  // @begin {methods} ------------------------------------------
  
 /**
   * Returns the data pack type which expresses the type of data
   * to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "double";
  }
  
  /**
   * Returns the data value at time point <b>aT</b>.
   * This data connection type is brokenline or step or pulse.
   *
   * <japanese>
   *  <b>aT</b> ̃f[^lԂ܂Bf[^Ԍ^
   * <a href="DataConnectionType.html">DataConnectionType</a> ́A
   * ܐAXebvApX̂ꂩƂ܂B
   * </japanese>
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>double</code> value
   */
  public double getData(OhTimePoint aT, Vector aDctVec)
    throws OhError {
    double dataValue = 0D;
    
    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 0) {
      throw new OhError(
        "Error in Get_Data" + NEWLINE + oPrintString(" >"));
    }
    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackDouble)dataPackArray.get(0)).
      getDataTime() );
    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackDouble)dataPackArray.
               get(numberOfDataPacks - 1)).getDataTime() );

    /*
      System.out.println("ta= " + ta);
      System.out.println("tb= " + tb);
      System.out.println("aT= " + aT);
    */
    
    if (aT.compareTo(ta) <= 0) {    // aT <= ta ?
      return
        ((OhDataPackDouble)dataPackArray.get(0)).getDataValue();
    }

    if (aT.compareTo(tb) >= 0) {    // aT >= tb ?
      return
        ((OhDataPackDouble)dataPackArray.
          get(numberOfDataPacks-1)).getDataValue();
    }

    DataConnectionType dataConnectionType =
      (DataConnectionType)aDctVec.elementAt(0);

    if(dataConnectionType == DataConnectionType.BROKENLINE)
    {
      // System.out.println("BROKENLINE");
      
      if (aT.compareTo(ta) <= 0) {
        dataValue =
          ((OhDataPackDouble)dataPackArray.get(0)).getDataValue();
      }
      else if (aT.compareTo(tb) >= 0) {
        dataValue =
          ((OhDataPackDouble)dataPackArray.
            get(numberOfDataPacks - 1)).getDataValue();
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;
        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackDouble)dataPackArray.get(ip1)).
          getDataTime() );
        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackDouble)dataPackArray.get(ip2)).
          getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;
          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).
            getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);  //; t1 = tp;  is NG.
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);  //; t2 = tp;  is NG.
          }
          else {
            return
              ((OhDataPackDouble)dataPackArray.get(ip)).
              getDataValue();
          }
        }

        double y1 = ((OhDataPackDouble)dataPackArray.get(ip1)).
          getDataValue();
        
        if (ip1 == ip2) {
          return y1;
        }

        double y2 =
          ((OhDataPackDouble)dataPackArray.get(ip2)).
          getDataValue();

        OhTimePoint aTBackUp = new OhTimePoint();
        aTBackUp.copy(aT);
        dataValue =
          ((
            OhTimePoint.sub(aTBackUp, t1).toDouble()*y2 +
            OhTimePoint.sub(t2, aTBackUp).toDouble()*y1
            ))
          / OhTimePoint.sub(t2, t1).toDouble();
      }
    }
    else if (dataConnectionType == DataConnectionType.STEP)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = ((OhDataPackDouble)dataPackArray.get(0)).
          getDataValue();
      }
      else if (aT.compareTo(tb) >= 0) {
        dataValue =
          ((OhDataPackDouble)dataPackArray.
            get(numberOfDataPacks - 1)).getDataValue();
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackDouble)dataPackArray.get(ip1)).
          getDataTime() );
        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackDouble)dataPackArray.get(ip2)).
          getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;
          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).
            getDataTime() );
          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);  //; t1 = tp;  is NG.
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);  //; t2 = tp;  is NG.
          }
          else {
            return
              ((OhDataPackDouble)dataPackArray.get(ip)).
              getDataValue();
          }
        }
        dataValue =
          ((OhDataPackDouble)dataPackArray.get(ip2)).
          getDataValue();
      }
    }
    else if (dataConnectionType == DataConnectionType.PULSE)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = 0D;
      }
      else if (aT.compareTo(tb) <= 0) {
        dataValue = 0D;
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackDouble)dataPackArray.get(ip1)).
          getDataTime() );
        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackDouble)dataPackArray.get(ip2)).
          getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;
          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).
            getDataTime() );
          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);  //; t1 = tp;  is NG.
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);  //; t2 = tp;  is NG.
          }
          else {
            return ((OhDataPackDouble)dataPackArray.get(ip)).
              getDataValue();
          }
        }
        if (aT.compareTo(t1) == 0) {
          dataValue =
            ((OhDataPackDouble)dataPackArray.get(ip1)).
            getDataValue();
        }
        else if (aT.compareTo(t2) == 0) {
          dataValue =
            ((OhDataPackDouble)dataPackArray.get(ip2)).
            getDataValue();
        }
        else {
          dataValue = 0D;
        }
      }
    }
    else {
      throw new OhError("Illegal data_connection_type."
        + NEWLINE + oPrintString("> "));
    }
    
    return dataValue;
  }
  
  /**
   * Returns the data value at OhTimePoint <b>aT</b>.
   * This data connection type is specified by using the that of
   * the sender port.
   *
   * <japanese>
   * OhTimePoint <b>aT</b> ̃f[^lԂ܂Bf[^Ԍ^́A
   * ̎M[qɃf[^񋟂Ă鑗M[q̃f[^Ԍ^
   * gp܂B<p>
   *
   * <b>aT</b>A^Ăł
   * f[^pbÑf[^O̎łƂ́A^
   * ĂłÂf[^pbÑf[^lԂ܂B
   * <b>aT</b>A^ĂłV
   * f[^pbÑf[^̎łƂ́A^
   * ĂłVf[^pbÑf[^lԂ܂B
   * </japanese>
   *
   * @param aT an <code>OhTimePoint</code> value
   * @return a <code>double</code> value
   */
  public double getData(OhTimePoint aT) throws OhError {
    return getData(aT, _sender.getDctVec());
  }
  
  /**
   * Set the data value at time t to reference aDataValue[0].
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double[]</code> value
   * @param aDctVec a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public void setData(OhTimePoint aT, double[] aDataValue,
    Vector aDctVec) throws OhError {
    aDataValue[0] = getData(aT, aDctVec);
  }

  
  /**
   * Set the data value at time t to reference aDataValue[0].
   * This data connection type is brokenline.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double[]</code> value
   * @exception OhError if an error occurs
   */
  public void setData(OhTimePoint aT, double[] aDataValue)
    throws OhError {  //; (*4-2)
    aDataValue[0] = getData(aT);
  }
  
  /**
   * Returns the integrated value from time aT1 till time aT2.
   * aT1 should be less than or equal to t2. <p>
   *
   * When data connection type is Pulse, the value summed from
   * aT1 till aT2 will be returned. The pulse at time aT1 is not
   * contained in the sum, and the pulse at time aT2 is
   * contained in the sum.
   *
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>double</code> value
   */
  public double getDataSum(OhTimePoint aT1, OhTimePoint aT2,
    Vector aDctVec) throws OhError {
    double sum = 0D;

    if(aT1.compareTo(aT2) == 0) {
      return sum;
    }

    if(aT1.compareTo(aT2) > 0) {
      OhTimePoint aT3 = aT1;
      aT1 = aT2;
      aT2 = aT3;
    }

    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 1) {
      throw new OhError(
        "Error in Get_Data" + NEWLINE + oPrintString(" >"));
    }


    OhTimePointImmutable ta =
      ((OhDataPackDouble)dataPackArray.get(0)).getDataTime();
    OhTimePointImmutable tb =
      ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).
      getDataTime();

    if (aT1.compareTo(ta) < 0 || aT2.compareTo(tb) > 0) {
      // ta <= aT1 and aT2 <= tb, right?
      throw new IllegalArgumentException("Illegal aT1 or aT2.");
    }

    DataConnectionType dataConnectionType =
      (DataConnectionType)aDctVec.elementAt(0);

    if(dataConnectionType == DataConnectionType.BROKENLINE) {
      double y  = 0D;
      double ya = 0D;
      double yb = 0D;
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePointImmutable tp =
          ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip;
          ipa = ip - 1;
          tb  = tp;

          ta =
            ((OhDataPackDouble)dataPackArray.get(ipa)).
            getDataTime();
          break;
        }
      }


      ya = ((OhDataPackDouble)dataPackArray.get(ipa)).
        getDataValue();
      yb = ((OhDataPackDouble)dataPackArray.get(ipb)).
        getDataValue();

      if(ipa == ipb) {
        y = ya;
      }
      else {
        y = ( OhTimePoint.sub(aT1, ta).toDouble()*yb +
              OhTimePoint.sub(tb, aT1).toDouble()*ya
            ) / OhTimePoint.sub(tb, ta).toDouble();
      }

      OhTimePointImmutable t  = aT1;
      ip = ipb;
      OhTimePointImmutable ts = tb;
      double ys = yb;

      while(ts.compareTo(aT2) < 0) {
        sum += (y + ys) * OhTimePoint.sub(ts, t).toDouble() / 2;
        t = ts;
        y = ys;
        ip++;

        ts = ((OhDataPackDouble)dataPackArray.get(ip)).
          getDataTime();
        ys = ((OhDataPackDouble)dataPackArray.get(ip)).
          getDataValue();
      }

      ys = ( y*OhTimePoint.sub(ts, aT2).toDouble() +
             ys*OhTimePoint.sub(aT2, t).toDouble()
           ) / OhTimePoint.sub(ts, t).toDouble();
      sum += (y+ys) * OhTimePoint.sub(aT2, t).toDouble() / 2;      
    }
    else if(dataConnectionType == DataConnectionType.STEP) {
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePointImmutable tp =
          ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip;
          ipa = ip - 1;
          tb = tp;
          break;
        }
      }

      OhTimePointImmutable tn = tb;
      ip = ipb;

      double yn = ((OhDataPackDouble)dataPackArray.get(ip)).
        getDataValue();
      OhTimePointImmutable ts = aT1;

      while(tn.compareTo(aT2) < 0) {
        sum += yn * OhTimePoint.sub(tn, ts).toDouble();
        ts = tn;
        ip++;

        tn =
          ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();
        yn =
          ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
      }

      if(aT2.compareTo(tn) <= 0) {
        sum += yn * (OhTimePoint.sub(aT2, ts).toDouble());
      }
    }
    else if(dataConnectionType == DataConnectionType.PULSE) {
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePointImmutable tp =
          ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip + 1;
          break;
        }
      }

      OhTimePointImmutable tn;
      double yn = 0D;

      for(; ip < numberOfDataPacks; ip++) {
        tn = ((OhDataPackDouble)dataPackArray.get(ip)).
          getDataTime();
        yn = ((OhDataPackDouble)dataPackArray.get(ip)).
          getDataValue();
        
        if(tn.compareTo(aT2) > 0)
          break;

        sum += yn;
      }
    }
    else {
      throw
        new IllegalArgumentException(
          "Illegal data_connection_type.");
    }
    
    return sum;
  }
  
  /**
   * Gets sum data whose data connection type is brokenline.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @return a <code>double</code> value
   */
  public double getDataSum(OhTimePoint aT1, OhTimePoint aT2)
    throws OhError {
    double sum = 0D;

    if(aT1.compareTo(aT2) == 0) {
      return sum;
    }

    if(aT1.compareTo(aT2) > 0) {
      OhTimePoint aT3 = aT1;
      aT1 = aT2;
      aT2 = aT3;
    }


    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 1) {
      throw new OhError(
        "Error in Get_Data" + NEWLINE + oPrintString(" >"));
    }

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackDouble)dataPackArray.get(0)).
      getDataTime() );
    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackDouble)dataPackArray.
               get(numberOfDataPacks - 1)).getDataTime() );

    if (aT1.compareTo(ta) < 0 || aT2.compareTo(tb) > 0) {
      // ta <= aT1 and aT2 <= tb, right?
      throw new IllegalArgumentException("Illegal aT1 or aT2.");
    }

    double y  = 0D;
    double ya = 0D;
    double yb = 0D;
    int ipa = 0;
    int ipb = numberOfDataPacks - 1;
    
    while(ipb - ipa > 1) {
      ip = (ipa + ipb)/2;

      OhTimePoint tp = new OhTimePoint();
      tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).
        getDataTime() );
      
      if(tp.compareTo(aT1) < 0) {
        ipa = ip;
        ta.copy(tp);  //; ta = tp;  is NG.
      }
      else if(tp.compareTo(aT1) > 0) {
        ipb = ip;
        tb.copy(tp);  //; tb = tp;  is NG.
      }
      else {
        ipb = ip;
        ipa = ip - 1;
        tb.copy(tp);  //; tb = tp;  is NG.
        ta.copy( ((OhDataPackDouble)dataPackArray.get(ipa)).
          getDataTime() );
        break;
      }
    }

    ya = ((OhDataPackDouble)dataPackArray.get(ipa)).getDataValue();
    yb = ((OhDataPackDouble)dataPackArray.get(ipb)).getDataValue();

    if(ipa == ipb) {
      y = ya;
    }
    else {
      y = ( OhTimePoint.sub(aT1, ta).toDouble()*yb +
            OhTimePoint.sub(tb, aT1).toDouble()*ya
          ) / OhTimePoint.sub(tb, ta).toDouble();
    }

    OhTimePoint t = new OhTimePoint();
    t.copy(aT1);

    ip = ipb;

    OhTimePoint ts = new OhTimePoint();
    ts.copy(tb);

    double ys = yb;
    
    while(ts.compareTo(aT2) < 0) {
      sum += (y + ys) * OhTimePoint.sub(ts, t).toDouble() / 2;
      t.copy(ts);  //; t = ts;  is NG.
      y = ys;
      ip++;

      ts.copy( ((OhDataPackDouble)dataPackArray.get(ip)).
        getDataTime() );
      ys = ((OhDataPackDouble)dataPackArray.get(ip)).
        getDataValue();
    }
    
    ys =
      (
        y*(OhTimePoint.sub(ts, aT2).toDouble()) +
        ys*(OhTimePoint.sub(aT2, t).toDouble())
      ) / (OhTimePoint.sub(ts, t).toDouble());
    sum += (y+ys) * (OhTimePoint.sub(aT2, t).toDouble()) / 2;      

//; System.out.println(sum);
    return sum;
  }
  
  /**
   * Gets pulse data.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetPulseData(OhTimePoint aT,
    double[] aDataValue) {    //(*2)

    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);
    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if (numberOfDataPacks <= 0)
      return false;

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackDouble)dataPackArray.get(0)).
      getDataTime() );
    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackDouble)dataPackArray.
               get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) < 0 || aT.compareTo(tb) > 0)
      return false;

    OhTimePoint tn = new OhTimePoint();
    OhTimePoint tp = new OhTimePoint();
    double yn = 0D;
    double yp = 0D;

    tn.copy(ta);  //; tn = ta;  is NG.
    yn = ((OhDataPackDouble)dataPackArray.get(0)).getDataValue();

    if (aT.compareTo(tn) == 0) {
      aDataValue[0] = yn;
      return true;
    }

    boolean hit = false;

    for (ip = 1; ip < numberOfDataPacks; ip++) {
      tp.copy(tn);  //; tp = tn;  is NG.
      yp = yn;

      tn.copy( ((OhDataPackDouble)dataPackArray.get(ip)).
        getDataTime() );
      yn = ((OhDataPackDouble)dataPackArray.get(ip)).
        getDataValue();
      
      if (tn.compareTo(aT) > 0) {
        hit = false;
        break;
      }
      else if (tn.compareTo(aT) == 0){
        aDataValue[0] = yn;
        hit = true;
        break;
      }
    }
    return hit;
  }
  
  /**
   * Gets mean data.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>double</code> value
   * @exception OhError if an error occurs
   */
  public double getMeanData(OhTimePoint aT1, OhTimePoint aT2,
    Vector aDctVec) throws OhError {
    if(aT2.compareTo(aT1) == 0) {
      return getData(aT2, aDctVec);
    }
    else {
      return getDataSum(aT1, aT2, aDctVec) /
        OhTimePoint.sub(aT2, aT1).toDouble();
    }
  }
  
  /**
   * Gets mean data whose data connection type is brokenline.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @return a <code>double</code> value
   */
  public double getMeanData(OhTimePoint aT1, OhTimePoint aT2)
    throws OhError {    //(*6)
    if(aT2.compareTo(aT1) == 0) {
      return getData(aT2);
    }
    else {
      return
        getDataSum(aT1, aT2) /
        (OhTimePoint.sub(aT2, aT1).toDouble());
    }
  }

  // @end {methods} --------------------------------------------

} // OhReceiveDouble
