// @(#) OhReceivePortTest.java

package ohymosj;

import junit.framework.*;
import java.io.*;
import java.util.regex.*;
import java.util.Vector;

public class OhReceivePortTest extends TestCase
{
  class OhReceivePortMoc extends OhReceivePort {
    // @begin{abstract_methods} --------------------------------
    public OhReceivePortMoc(String aPortName, int aPortNumber) {
      super(aPortName, aPortNumber);
    }
    public String dataType() {
      return "double"; 
    }
    public boolean areYouReady() {
      return true;
    }
    public void setDataPackArray() {}
    public boolean canYouGetData(
      OhTimePoint aTFrom, OhTimePoint aTTo) {
      return true;
    }
    public void getDataPackArray(
      OhArrayList[] aDpArray, int[] aNumOfDp) {}
    public OhTimePointImmutable getOldestDataTime() {
      return new OhTimePoint();
    }
    public OhTimePointImmutable getNewestDataTime() {
      return new OhTimePoint();
    }
    public int getNumberOfDataPacks() {
      return 0;
    }
    public boolean canYouGetDataPack(
      Object[] aFormer, OhTimePoint _dataTime,
      Object[] aLater, char aMode)
      {
        return true;
      }
    public OhTimePointImmutable getNecessaryTimeTo() {
      return new OhTimePoint();
    }
    public OhTimePointImmutable getNecessaryTimeFrom() {
      return new OhTimePoint();
    }
    // @end{abstract_methods} ----------------------------------
  }
  
  class OhSenderPortMoc extends OhSenderPort {
    String _message;
    public OhSenderPortMoc(String aPortName, int aPortNumber) {
      super(aPortName, aPortNumber);
    }
    public String dataType() { return "double"; }
    public Vector getDctVec() { return null; }
    public void setDataPackArray() {}
    public boolean canYouGetData(OhTimePointImmutable aTFrom,
      OhTimePointImmutable aTTo) {
      return true;  
    }
    public void getDataPackArray(OhArrayList[] aDpArray,
      int[] aNumOfDp) {}
    public OhArrayList getDataPackArray() { return null; }
    public boolean getDataPack(
      Object[] aFormer, OhTimePoint _dataTime,
      Object[] aLater, char aMode) { return true;
    }
    public void calReq() throws OhCannotGetDataError, OhError {
      _message = "calReq was called";
    }
     public void reduceDataPackStock() {
      _message = "reduceDataPackStock was called";
    }
    public String getMessage() { return _message; }
    public int getNumberOfDataPacks() { return 0; }
    public OhTimePointImmutable getNewestDataTime() {
      return new OhTimePoint(); }
    public OhTimePointImmutable getOldestDataTime() {
      return new OhTimePoint(); }
  }

  public OhReceivePortTest(String name) {
    super(name);
  }
  
  public void setUp() {
  }
  
  public void tearDown() {
  }

  public void test_Foo() {}
  
  // @begin{constructors check} ----------------------------------
  public void test_constructors() {
    OhReceivePort rp = new OhReceivePortMoc("rp", 0);
    assertEquals("test_constructors", "OhReceivePort",
      rp.getClassName());
    assertEquals("test_constructors", "rp", rp.getObjectName());
    assertEquals("test_constructors", 0, rp.getObjectNumber());
  }
  // @end{constructors check} ------------------------------------

  // @begin{methods check} ---------------------------------------
  public void test_setSender() {
    OhReceivePort rp = new OhReceivePortMoc("rp", 0);
    OhSenderPort sp = new OhSenderPortMoc("sp", 0);
    try {
      rp.setSender(sp);
    }
    catch (OhError e) {
      System.out.println(e.what());
    }
  }

  public void test_areYouReady() {
  }

  public void test_setDataPackArray() {
  }
  
  public void test_getNumberOfDataPacks() {
  }

  public void test_canYouGetData() {}

  public void test_getDataPackArray() {}
  public void test_getNecessaryTimeFrom() {}
  public void test_getNecessaryTimeTo() {}
  
  public void test_calReq() {
    OhReceivePort rp = new OhReceivePortMoc("rp", 0);
    OhSenderPort sp = new OhSenderPortMoc("sp", 0);
    try {
      rp.setSender(sp);
      rp.calReq();
    }
    catch (OhError e) {
      System.out.println(e.what());
    }
    catch (OhCannotGetDataError e) {
      System.out.println(e.what());
    }
    assertEquals("test_calReq",
      (Object)"calReq was called", ((OhSenderPortMoc) sp).getMessage());
  }
  public void test_reduceDataPackStock() {
    OhReceivePort rp = new OhReceivePortMoc("rp", 0);
    OhSenderPort sp = new OhSenderPortMoc("sp", 0);
    try {
      rp.setSender(sp);
    }
    catch (OhError e) {
      System.out.println(e.what());
    }
    rp.reduceDataPackStock();
    assertEquals("test_reduceDataPackStock",
      (Object)"reduceDataPackStock was called", ((OhSenderPortMoc) sp).getMessage());
  }
  
  public void test_printConnection() {
    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    OhReceivePort rp = new OhReceivePortMoc("rp", 0);
    rp.printConnection(out, "> ");
    String result = buffer.toString();
    boolean match = Pattern.matches(
      "(?s).*Not connected.*", result);
    if (!match) {
      fail("The result of printConnection() does not match with one expteced.");
    }

    OhSenderPort sp = new OhSenderPortMoc("sp", 0);
    try {
      rp.setSender(sp);
    }
    catch (OhError e) {
      System.out.println(e.what());
    }
    StringWriter buffera = new StringWriter();
    PrintWriter outa = new PrintWriter(buffer);
    rp.printConnection(outa, "> ");
    String resulta = buffer.toString();
    boolean matcha = Pattern.matches(
      "(?s).*Not connected.*", result);
    if (!matcha) {
      fail("The result of printConnection() does not match with one expteced.");
    }
  }
    
  // @end{methods check} -----------------------------------------
}
