// @(#) OhRelayPortTest.java

package ohymosj;

import junit.framework.*;
import java.util.Vector;

public class OhRelayPortTest extends TestCase
{
  public OhRelayPortTest(String name) {
    super(name);
  }
  
  public void setUp() {
  }
  
  public void tearDown() {
  }

  public class OhRelayPortMoc extends OhRelayPort {
    String _message;
    public String dataType() { return "double";
    }
    public OhRelayPortMoc() {
      super();
    }
    public OhRelayPortMoc(String aPortName, int aPortNum) {
      super(aPortName, aPortNum);
    }
  }

  // @begin{constructors check} ----------------------------------

  public void test_constructors() {
    OhRelayPort a = new OhRelayPortMoc();
    assertEquals("1", "OhRelayPort", a.getClassName());

    OhRelayPort b = new OhRelayPortMoc("rlp", 1);
    assertEquals("2", "OhRelayPort", b.getClassName());
    assertEquals("3", "rlp", b.getObjectName());
    assertEquals("4", 1, b.getObjectNumber());
  }
  
  // @end{constructors check} ------------------------------------

  // @begin{methods check} ---------------------------------------

  public void test_getDctVec() throws OhError {
    OhSendDouble sp = new OhSendDouble();
    sp.setDataConnectionType(DataConnectionType.STEP);
    OhRelayPort rlp = new OhRelayPortMoc();
    sp.registerReceiver(rlp);
    rlp.setSender(sp);
    sp.doSwitch();
    Vector dctvec = rlp.getDctVec();
    assertEquals("1", 1, dctvec.size());
    assertEquals("2", DataConnectionType.STEP, dctvec.get(0));
  }

  public void test_setSender() {
    // test of setSender() is done in test_getDctVec()
  }

  public void test_areYouReady() throws OhError {
    OhSendDouble sp = new OhSendDouble();
    sp.setDataConnectionType(DataConnectionType.STEP);
    OhRelayPort rlp = new OhRelayPortMoc();
    sp.registerReceiver(rlp);
    rlp.setSender(sp);
    assertEquals("1", false, rlp.areYouReady());
    sp.doSwitch();
    assertEquals("2", false, rlp.areYouReady());
    rlp.doSwitch();
    assertEquals("3", true, rlp.areYouReady());
  }

  public void test_setDataPackArray() throws OhError {
    OhSendDouble sp = new OhSendDouble();
    OhRelayPort rlp = new OhRelayPortMoc();
    sp.registerReceiver(rlp);
    rlp.setSender(sp);
    sp.doSwitch();
    rlp.doSwitch();
    sp.sendByString("2003 10 0.0 1.0");
    sp.sendByString("2003 20 0.0 2.0");
    sp.sendByString("2003 30 0.0 3.0");
    // rlp.setDataPackArray();
    assertEquals("1", 3, sp.getNumberOfDataPacks());
  }

  public void getOldestDataTime() throws OhError {
    OhSendDouble sp = new OhSendDouble();
    OhRelayPort rlp = new OhRelayPortMoc();
    sp.registerReceiver(rlp);
    rlp.setSender(sp);
    sp.doSwitch();
    rlp.doSwitch();
    // rlp.setDataPackArray();
    assertEquals("1", OhTimePoint.OhTimePointMin,
      rlp.getOldestDataTime());
    sp.sendByString("2003 10 0.0 1.0");
    sp.sendByString("2003 20 0.0 2.0");
    sp.sendByString("2003 30 0.0 3.0");
    // rlp.setDataPackArray();
    assertEquals("2", new OhTimePoint(2003, 10, 0.0),
      rlp.getOldestDataTime());
  }
    
  public void getNewestDataTime() throws OhError {
    OhSendDouble sp = new OhSendDouble();
    OhRelayPort rlp = new OhRelayPortMoc();
    sp.registerReceiver(rlp);
    rlp.setSender(sp);
    sp.doSwitch();
    rlp.doSwitch();
    // rlp.setDataPackArray();
    assertEquals("1", OhTimePoint.OhTimePointMax,
      rlp.getNewestDataTime());
    sp.sendByString("2003 10 0.0 1.0");
    sp.sendByString("2003 20 0.0 2.0");
    sp.sendByString("2003 30 0.0 3.0");
    // rlp.setDataPackArray();
    assertEquals("2", new OhTimePoint(2003, 30, 0.0),
      rlp.getOldestDataTime());
  }

  public void test_getNumberOfDataPacks() throws OhError {
    OhSendDouble sp = new OhSendDouble();
    OhRelayPort rlp = new OhRelayPortMoc();
    sp.registerReceiver(rlp);
    rlp.setSender(sp);
    sp.doSwitch();
    rlp.doSwitch();
    sp.sendByString("2003 10 0.0 1.0");
    sp.sendByString("2003 20 0.0 2.0");
    sp.sendByString("2003 30 0.0 3.0");
    assertEquals("1", 3, rlp.getNumberOfDataPacks());
  }

  public void test_canYouGetData() throws OhError {
    OhSendDouble sp = new OhSendDouble();
    OhRelayPort rlp = new OhRelayPortMoc();
    sp.registerReceiver(rlp);
    rlp.setSender(sp);
    sp.doSwitch();
    rlp.doSwitch();
    sp.sendByString("2003 10 0.0 1.0");
    sp.sendByString("2003 20 0.0 2.0");
    sp.sendByString("2003 30 0.0 3.0");
    assertEquals("1", true,
      rlp.canYouGetData(
        new OhTimePoint(2003,15,0.0),
        new OhTimePoint(2003,25,0.0)));
    assertEquals("2", false,
      rlp.canYouGetData(
        new OhTimePoint(2003,15,0.0),
        new OhTimePoint(2003,35,0.0)));
    assertEquals("3", false,
      rlp.canYouGetData(
        new OhTimePoint(2003,35,0.0),
        new OhTimePoint(2003,45,0.0)));
  }

  public void test_calReq() {
    //; $B>JN,(B
  }
  

  public void test_getDataPackArray() throws OhError {
    OhSendDouble sp = new OhSendDouble();
    OhRelayPort rlp = new OhRelayPortMoc();
    sp.registerReceiver(rlp);
    rlp.setSender(sp);
    sp.doSwitch();
    rlp.doSwitch();
    sp.sendByString("2003 10 0.0 1.0");
    sp.sendByString("2003 20 0.0 2.0");
    sp.sendByString("2003 30 0.0 3.0");
    assertEquals("1", 3, rlp.getNumberOfDataPacks());
    OhArrayList dps = rlp.getDataPackArray();
    assertEquals("2", 2.0,
      ((OhDataPackDouble) dps.get(1)).getDataValue(), 0.001);
    assertEquals("3", 3.0,
      ((OhDataPackDouble) dps.get(2)).getDataValue(), 0.001);

  }

  public void test_getDataPack() {
    //; $B>JN,(B
  }

  public void test_reduceDatapackStock() {
    //; $B>JN,(B
  }

  public void test_printConnection() {
    //; $B>JN,(B
  }

  
  // @end{methods check} -----------------------------------------
}

