/**
 * @(#) OhSendDouble; a port which sends datapacks of type double.
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * The OhSendDouble class is a base class to express ports which
 * send double data.
 *
 * <japanese>
 * <p>
 * double ^̃f[^pbN𑗐M鑗M[q̃NX <p>
 *
 * <a href="OhReceivePort.html">OhReceivePort</a> pč쐬M[q̃NX
 * B
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.23 $ $Date: 2004/01/17 01:58:34 $
 */
public class OhSendDouble extends OhSendPort
{
  // @begin{initialization} ------------------------------------

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public void init(
    String aPortName,
    int aPortNum)
    throws OhError {
    super.init(aPortName, aPortNum);
  }

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>boolean</code> value
   */
  public void init(
    String aPortName,
    int aPortNum,
    Vector aDctVec)
    throws OhError {
    super.init(aPortName, aPortNum, aDctVec);
  }

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDct a <code>DataConnectionType</code> value
   * @exception OhError if an error occurs
   */
  public void init(
    String aPortName,
    int aPortNum,
    DataConnectionType aDct)
    throws OhError {
    super.init(aPortName, aPortNum);
    _dctVec = new Vector();
    _dctVec.addElement(aDct);
  }

  // @end{initialization} --------------------------------------

  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>OhSendDouble</code> instance.
   *
   * @exception OhError if an error occurs
   */
  public OhSendDouble() {
    super();
    _className = "OhSendDouble";
  }

  /**
   * Creates a new <code>OhSendDouble</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSendDouble(String aPortName, int aPortNum) {
    super(aPortName, aPortNum);
    _className = "OhSendDouble";
  }
  
  /**
   * Creates a new <code>OhSendDouble</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public OhSendDouble(String aPortName, int aPortNum, Vector aDctVec) {
    super(aPortName, aPortNum, aDctVec);
    _className = "OhSendDouble";
  }

  /**
   * Creates a new <code>OhSendDouble</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDct a <code>DataConnectionTyper</code> value
   */
  public OhSendDouble(String aPortName, int aPortNum, DataConnectionType aDct) {
    super(aPortName, aPortNum);
    _dctVec = new Vector();
    _dctVec.addElement(aDct);
    _className = "OhSendDouble";
  }
  
  // @end {constructors} ---------------------------------------


  // @begin {methods} ------------------------------------------

  // 1. function for sending data.
  
  /**
   * Returns the data pack type which expresses the type of data to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "double";
  }


  /**
   * Send data whose data time and data value are <b>aT</b> and
   * <b>aValue</b>, respectively.
   *
   * <japanese>
   * <p>
   * ɂƂ aT, aValue OhDataPackDouble ^f[^pbN
   * 쐬A
   * <a href="OhSendPort.html#method_summary">OhSendPort#dataPackStock(OhDataPack)</a>
   * pđM[qɑM܂B
   * </japanese>
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aValue a <code>double</code> value
   */
  public void sendData(OhTimePointImmutable aT, double aValue) {
    dataPackStock(new OhDataPackDouble(aT, aValue));
  }

  /**
   * Send data of type double whose value is stored in the
   * String specified.<p>
   *
   * example: sendByString("2003 10 0.0 1.5");
   *
   * <japanese>
   * <p>
   * ɂƂ aDataStr ŕ\ꂽf[^pbN
   * OhDataPackDouble ^f[^pbN쐬A
   * OhSendPort.dataPackStock(Object) p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public void sendByString(String aDataStr) {
    dataPackStock(new OhDataPackDouble(aDataStr));
  }

  /**
   * Sets the data connection type. You can specify the data
   * connection type directly and do not need give a data
   * connection type vector.<p>
   *
   * <japanese>
   * f[^Ԍ^w肵܂Bf[^Ԍ^zw肷Kv
   * ܂BŎw肵f[^Ԍ^IuWFNgz
   * \bhŐċL܂B<p>
   *
   * f[^Ԍ^̓RXgN^init\bhłwł܂B
   * </japanese>
   *
   * @param aDct a <code>DataConnectionType</code> value
   */
  public void setDataConnectionType(DataConnectionType aDct) {
    _dctVec = new Vector();
    _dctVec.addElement(aDct);
  }
  
  /**
   * Returns the basic info about this port class such as
   * classname and datatype it carries. The string returned
   * contains multiple lines, but the last line does not contain
   * line-end character.
   *
   * <japanese>
   * <p>
   * ̃|[g̃NXƃf[^^Ɋւ𕶎ɂĕ
   * ܂BsłAŌ̍sɂ͍sR[h
   * B
   * </japanese>
   *
   * @return a <code>String</code> value
   */
  public String basicInfo() {
    return
      "classname: " + _className + NEWLINE +
      "datatype it carries: " + dataType();
  }
  
  // @end {methods} --------------------------------------------

} // OhSendDouble
