/**
 * @(#) OhSendLong
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * The OhSendLong class is the base class to express a port
 * which send datapacks of type long.
 *
 * <japanese>
 * <p>
 * long ^̃f[^pbN𑗐M鑗M[q̃NX <p>
 *
 * <a href="OhReceivePort.html">OhReceivePort</a> pč쐬M[q̃NX
 * B
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.3 $ $Date: 2004/01/17 01:58:34 $
 */
public class OhSendLong extends OhSendPort
{
  // @begin{initialization} ------------------------------------

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public void init(String aPortName, int aPortNum)
    throws OhError {
    super.init(aPortName, aPortNum);
  }

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>boolean</code> value
   */
  public void init(String aPortName, int aPortNum,
    Vector aDctVec) throws OhError {
    super.init(aPortName, aPortNum, aDctVec);
  }

  // @end{initialization} --------------------------------------

  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>OhSendLong</code> instance.
   *
   * @exception OhError if an error occurs
   */
  public OhSendLong() {  //; 2003.11.18
    super();
    _className = "OhSendLong";
  }


  /**
   * Creates a new <code>OhSendLong</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSendLong(String aPortName, int aPortNum) {
    super(aPortName, aPortNum);
    _className = "OhSendLong";
  }

  
  /**
   * Creates a new <code>OhSendLong</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public OhSendLong(String aPortName, int aPortNum, Vector aDctVec) {
    super(aPortName, aPortNum, aDctVec);
    _className = "OhSendLong";
  }

  // @end {constructors} ---------------------------------------


  // @begin {methods} ------------------------------------------

  // 1. function for sending data.
  
  /**
   * Returns the data pack type which expresses the type of data
   * to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "long";
  }


  /**
   * Send data whose data time is <b>aT</b> and data value is
   * <b>aValue</b>.
   *
   * <japanese>
   * <p>
   * ɂƂ aT, aValue 
   * <a href="OhDataPackLong.html">OhDataPackLong</a> ^f[^pbN쐬A
   * <a href="OhSendPort.html#method_summary">OhSendPort#dataPackStock(Object)</a> p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aValue a <code>long</code> value
   */
  public void sendData(OhTimePoint aT, long aValue) {
    OhDataPackLong dpd = new OhDataPackLong();
    OhTimePoint tp = new OhTimePoint();  //; For work.
    tp.copy(aT);
    dpd.setData(tp, aValue);
    dataPackStock(dpd);
  }

  /**
   * Send datapacks of type long whose value is stored in
   * <b>aDataStr</b>.
   *
   * <japanese>
   * <p>
   * ɂƂ aDataStr ŕ\ꂽf[^pbN
   * <a href="OhDataPackLong.html">OhDataPackLong</a> ^f[^pbN쐬A
   * <a href="OhSendPort.html#method_summary">OhSendPort#dataPackStock(Object)</a> p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public void sendByString(String aDataStr) {           //; (*2)
    OhDataPackLong dp = new OhDataPackLong(aDataStr);
    dataPackStock(dp);
  }
    
  // @end {methods} --------------------------------------------

} // OhSendLong
