/**
 * @(#) OhSendPort; class to express a send port.
 *
 * <a href="OhObject.html">OhObject</a>
 * <-- <a href="OhPort.html">OhPort</a>
 * <-- <a href="OhSenderPort.html">OhSenderPort</a>
 * <-- <a href="OhSendPort.html">OhSendPort</a>
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * The OhSendPort class is an abstract class for creating send
 * ports. It extends <a href="OhSenderPort.html">OhSenderPort</a>.<p>
 *
 * Data members _dataPackArray and _numberOfDataPacks and some
 * methods to store datapacks are defined.
 *
 * <japanese>
 * <p>
 * {^M[qf̃NX <p>
 *
 * {^M[qf OhSendPort `Ă܂B{^M
 * [qf͒ۃNXłB<p>
 *
 * {^M[qf OhSendPort ́AŎĂf[^
 * M[qłAf[^𑗐M[qOhSenderPort ̈
 * Bʂ̒[qf[^𑗐M[q͒p[q
 * OhRelayPort łB<p>
 *
 * NX OhSendPort ł́A
 *
 * <ul>
 *  <li> Mf[^pbN : _dataPackArray, _numberOfDataPacks
 * </ul>
 *
 * LA
 *
 * <ul>
 *  <li> f[^pbN^ёM@\
 * </ul>
 *
 * `܂B<p>
 *
 * OhSendPort ̑M@\́ANXOhSenderPort Œ`ς݂
 * BOhSendPort ɂ́ȂM[qڑ邱Ƃł܂B
 * <p>
 *
 *  OhSendPort NXof[^̃f[^Ԍ^xNg
 * oϐƂċL܂Bw肵ȂƁAf[^1
 * łƉ肵āA1̃f[^̕Ԍ^́ABROKENLINE
 * (ܐ)łƂ܂BOhSentPort IuWFNgof[^
 * ̃f[^Ԍ^xNg̃ftHg̎wƈقȂƂ́A
 * RXgN^̈Ŏw肷邩A
 * <a href="OhSendPort.html#method_summary">setDctVec(Vector)</a> \bhgpăf[^
 * Ԍ^xNgw肵ĂB
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.63 $ $Date: 2004/01/13 05:40:45 $
 */
public abstract class OhSendPort extends OhSenderPort
{
  // @begin {instance variable or static variable} -------------

  /**
   * The OhArrayList object for storing datapacks.
   *
   * <japanese>
   * <p>
   * f[^pbNMpz <p>
   *
   * _dataPackArray wzɂ́AOhDataPack ^̎^f[^pb
   * Ni[܂B
   * </japanese>
   *
   */
  protected OhArrayList _dataPackArray;

  /**
   * The number of elements stored in "_dataPackArray".
   *
   * <japanese>
   * <p>
   * _dataPackArray ̗vf
   * </japanese>
   *
   */
  protected int _numberOfDataPacks;
  
  /**
   * Boolean variable which is used to control the protect mode
   * of past dataPacks.
   *
   */
  protected boolean _protectPastDataFlag;
  
  /**
   * Data connection type vector of data provided this
   * OhSendPort.
   *
   * <japanese>
   * <p>
   * ̑M[qof[^̃f[^Ԍ^xNg <p>
   *
   *  OhSendPort NXof[^̃f[^Ԍ^xNg
   * oϐƂċL܂B
   * </japanese>
   *
   */
  protected Vector _dctVec;

  // @end {instance variable or static variable} ---------------
  

  // @begin {abstract methods} ---------------------------------

  // 1. Connection function.
  
  /**
   * Returns the string which shows the type of the datapack
   * sent by the current port.
   *
   * <japanese>
   * <p>
   * f[^̌^ԂB<p>
   *
   * NX OhSenderPort ̒ۃ\bhłB
   * ̃NX OhSendPort łۃ\bhƂĂ܂B
   * </japanese>
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();

  
  // 2. Recording and sending function of datapacks.
  
  /**
   * A data pack is created and recorded from the data character
   * sequence stored in "aDstr".
   *
   * <japanese>
   * <p>
   * f[^pbN̍쐬ю^ <p>
   *
   * aDstr (arg) Ɋi[ꂽf[^񂩂f[^pbN쐬
   * āÃ|[gɎ^܂B<p>
   *
   * () ̃\bhĂ΂OɁÃ|[g̃\bh 
   * doSwitch() ĂяoĂKv܂B
   * </japanese>
   *
   * @param aDstr a <code>String</code> value
   */
  public abstract void sendByString(String aDstr);

  // @end {abstract methods} -----------------------------------


  // @begin{initialization} ------------------------------------

  /**
   * Initialization.
   *
   * <japanese>
   * |[gA|[gԍݒ肵܂B
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public void init(String aPortName, int aPortNum) throws OhError {
    _dctVec = new Vector();
    _dctVec.addElement(DataConnectionType.BROKENLINE);
    super.init(aPortName, aPortNum);
  }

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   */
  public void init(
    String aPortName,
    int aPortNum,
    Vector aDctVec)
    throws OhError {
    _dctVec = aDctVec;
    super.init(aPortName, aPortNum);
  }

  // @end{initialization} --------------------------------------

  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>OhSendPort</code> instance.
   *
   */
  public OhSendPort() {
    super();
    _dataPackArray       = null;
    _numberOfDataPacks   = -1;
    _protectPastDataFlag = false;

    _className     = "OhSendPort";
    _commentString = null;
  }

  /**
   * Creates a new <code>OhSendPort</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public OhSendPort(String aPortName, int aPortNum) {
    super();
    _dataPackArray       = null;
    _numberOfDataPacks   = -1;
    _protectPastDataFlag = false;

    _className     = "OhSendPort";
    _commentString = null;
    
    _dctVec = new Vector();
    _dctVec.addElement(DataConnectionType.BROKENLINE);

    _objectName = aPortName;
    _objectNumber = aPortNum;
  }

  /**
   * Creates a new <code>OhSendPort</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   */
  public OhSendPort(String aPortName, int aPortNum, Vector aDctVec) {
    super();
    _dataPackArray       = null;
    _numberOfDataPacks   = -1;
    _protectPastDataFlag = false;

    _dctVec = aDctVec;
    
    _className     = "OhSendPort";
    _commentString = null;
    
    _objectName = aPortName;
    _objectNumber = aPortNum;
  }
  
  // @end {constructors} ---------------------------------------
  
  // @begin {methods} ------------------------------------------

  // Connection method.
  
  /**
   * Set the data connection type vector.
   *
   * @param aDctVec a <code>Vector</code> value
   */
  public void setDctVec(Vector aDctVec) {
    _dctVec = aDctVec;
  }


  /**
   * This method is decleared as abstract method in the
   * super class <b>OhSenderPort</b>. Implemented in this
   * class. <p>
   *
   * Returns the data connection vector.<p>
   *
   * <japanese>
   * ̒[q̃f[^Ԍ^xNg擾܂B̒[q̃f[
   * ^Ԍ^xNǵA<a href="OhSendPort.html">OhSendPort</a> ̃RXgN^A
   * \bh <a href="OhSendPort.html#method_summary">init()</a>A\bh
   * <a href="OhSendPort.html#method_summary">setDctVec()</a> ȂǂŐݒ肳܂B
   * </japanese>
   *
   * @return a <code>Vector</code> value
   */
  public Vector getDctVec() {
    return _dctVec;
  }

  /**
   * Preparation of "_dataPackArray".
   * That is, "_numberOfDataPacks" is set as 0 from -1.
   *
   * <japanese>
   * <p>
   * [q̐ڑƂ̏Iуf[^MJn̂߂̏
   * Ɓi_receiverArray ̍쐬 _dataPackArray ̏ <p>
   *
   * ̃\bh͈ȉs܂B
   *
   * <ol>
   *  <li> MM[qz _dataPackArray 쐬B
   *  <li> MM[q̐ _numberOfReceivers ɐݒ肷B
   *  <li> _dataPackArray ̃mۂB
   *  <li> _numberOfDataPacks ̒l -1  0 ɕύXB
   * <ol>
   *
   * ܂AOhSenderPort.doSwitch() s܂B̓Iɂ́A
   * _receiverArray 쐬܂B<p>
   *
   * ɁA_dataPackArray  new 邱Ƃɂ胁mۂA
   * _numberOfDataPacks ̒l -1  0 ɕϊ܂B
   * <p>
   *
   * rōƂɎsƁAOhError IuWFNg throw ܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void doSwitch() throws OhError {
    super.doSwitch();
    if ((_numberOfDataPacks >= 0) || (_dataPackArray != null)) {
      throw new OhError(
        "Already exists \"_dataPackArray\"." +
        oPrintString(" > ")
      );
    }
    _dataPackArray = new OhArrayList();
    _numberOfDataPacks = 0;
  }

  /**
   * Transer the calculation request to the owner. Note that the
   * owner of a send port is an OhElement object.
   *
   * <japanese>
   * <p>
   * M[q̏L҂ɌvZv`܂BM[q̏L҂́Av
   * ffł邱ƂɒӁB
   * </japanese>
   *
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public void calReq() throws OhCannotGetDataError, OhError {
    ((OhElement)_owner).calReq();
  }

  // 2. Recording and sending function of datapacks.

  /**
   * Stocks "_bufferList" if the data time of "aDataPack" is
   * newer than the data time of the newest datapack stored in
   * "_bufferList. Otherwise the "_bufferList" is not included
   * into "_bufferList". [in] "aDataPack": a datapack to be
   * stocked. It must be casted as <code>OhObject</code>.
   *
   * <japanese>
   * <p>
   * f[^pbN̎^ <p>
   *
   * ŐV̎^f[^pbN aDataPack (arg) ̃f[^L^
   * rAaDataPack ̕V aDataPack  _bufferList 
   * Ɏ^܂B<p>
   *
   * ̏ꍇ aDataPack ̕Âꍇ aDataPack ^
   * ܂B<p>
   *
   * ႦΕsAfsvȏꍇȂǂ͔hNXōĒ`ĉ
   * B
   *
   * </japanese>
   *
   * @param aDataPack an <code>OhDataPack</code> value
   */
  public void dataPackStock(OhDataPack aDataPack) {
    if ((_dataPackArray.size() > 0) &&
      (((OhDataPack)aDataPack).getDataTime().
        compareTo(((OhDataPack)_dataPackArray.getLastElement()).
          getDataTime()) <= 0))
      return;

    _dataPackArray.add(aDataPack);
    _numberOfDataPacks = _dataPackArray.size();
  }
  
  /**
   * Makes data pack array.
   *
   * <japanese>
   * <p>
   * f|^pbNz̍쐬 <p>
   *
   * NX
   * OhSenderPort
   * ̒ۃ\bhłB<p>
   *
   * ۂɂ́A_dataPackArray ̗vf _numberOfDataPacks ɐ
   * 肷邾łB
   * </japanese>
   *
   */
  public void setDataPackArray() {
    _numberOfDataPacks = _dataPackArray.size();
  }

  /**
   * Returns the oldest data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePointImmutable getOldestDataTime() {
    if (_numberOfDataPacks <= 0)
      return OhTimePoint.OhTimePointMin;

    return (((OhDataPack)_dataPackArray.get(0)).getDataTime());
  }

  
  /**
   * Returns the newest data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePointImmutable getNewestDataTime() {
    if (_numberOfDataPacks <= 0)
      return OhTimePoint.OhTimePointMax;

    return
      (((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).
        getDataTime());
  }


  /**
   * Returns the number of data packs which stored in the sender.
   *
   * @return an <code>int</code> value
   */
  public int getNumberOfDataPacks() {
    return _numberOfDataPacks;
  }

  /**
   * Checks the data time recorded.
   *
   * <japanese>
   * <p>
   * f[^^Ԃ̊mF <p>
   *
   * NX
   * OhSenderPort
   * ̒ۃ\bhłB
   * ̃NX OhSendPort Ŏ܂B<p>
   *
   * aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN^Ă
   * ꍇ true ԂA^ĂȂꍇ false Ԃ܂B
   * <p>
   *
   * ܂A_dataPackArray Ɏ^Ăꍇ true Ԃ܂B
   * ^ĂȂꍇ́AX _bufferList Ɏ^Ă邩
   * ׁA_bufferList Ɏ^Ă _dataPackArray XV
   *  true Ԃ܂B^ĂȂꍇ false Ԃ܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaTFrom -> aTFrom ȑOf[^pbN^Ă邱ƂmFB
   *  EaTTo ---> aTTo ȍ~܂Ńf[^pbN^Ă邱ƂmFB
   * [RETURN]
   *  Etrue ---> aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN^ĂꍇB
   *  Efalse --> aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN^ĂȂꍇB
   * </pre>
   * </japanese>
   *
   * @param aTFrom an <code>OhTimePoint</code> value
   * @param aTTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetData(OhTimePointImmutable aTFrom,
    OhTimePointImmutable aTTo) {
    if (_numberOfDataPacks <= 0)
      return false;

    if ((((OhDataPack)_dataPackArray.
           get(_numberOfDataPacks - 1)).getDataTime().
          compareTo(aTTo) < 0) ||
        (((OhDataPack)_dataPackArray.get(0)).getDataTime().
          compareTo(aTFrom) > 0)) {
      return false;
    }
    return true;
  }
      

  /**
   * Gets the datapack array.
   *
   * <japanese>
   * <p>
   * f[^pbNz̑Mi擾j <p>
   *
   * NX
   * OhSenderPort
   * ̒ۃ\bhłB<p>
   *
   * Ƀoϐ _dataPackArray, _numberOfDataPacks i[
   * ܂B
   *
   * </japanese>
   *
   * @param aDpArray an <code>OhArrayList[]</code> value
   * @param aNumOfDp an <code>int[]</code> value
   */
  public void getDataPackArray(
    OhArrayList[] aDpArray, int[] aNumOfDp) {
    aNumOfDp[0] = _numberOfDataPacks;
    aDpArray[0] = _dataPackArray;
  }

  /**
   * Gets the datapack array.
   *
   * @return an <code>OhArrayList</code> value
   */
  public OhArrayList getDataPackArray() {
    return _dataPackArray;
  }

  /**
   * Gets datapacks by specifying a time point
   *
   * <japanese>
   * <p>
   * w肵ăf[^pbN擾܂B<p>
   *
   * ̃\bbh͊NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\b
   * hłB OhSendPort NXŎ܂B<p>
   *
   * aFormer ɂ́Af[^L^ aDataTime ȑO aDataTime 
   * ɍł߂^f[^pbN擾Ai[܂B<p>
   *
   * aLater ɂ́Af[^L^ aDataTime ȍ~ aDataTime 
   * ł߂^f[^pbN擾Ai[܂B<p>
   *
   * f[^L^x aDataTime łf[^pbN^
   * Ăꍇ́AaFormer, aLater Ƃi[܂B<p>
   *
   * 𖞂f[^pbN擾łȂꍇAaFormer, aLater 
   *  null ɐݒ肳܂B<p>
   *
   * aMode ͎擾[hI邽߂̃IvVŁAL̒ʂłB
   *
   * <ul>
   *  <li> 'b' ----> aFormer, aLater ̗K擾B擾
   *       łȂꍇ͗Ƃ null ɐݒ肷B
   *  <li> 'f' ----> aFormer K擾BaLater 擾ł
   *       Ă aFormer ̂ݎ擾B
   *  <li> 'l' ----> aLater K擾BaFormer 擾ł
   *       Ă aLater ̂ݎ擾B
   *  <li> ̑ -> aFormer, aLater ̏ȂƂ擾B
   * </ul>
   *
   * aMode ̃ftHgl 'b' łBd`̂ŁA 
   * aLater nȂꍇ aMode  'f' Ƃ aFormer ݂̂
   * 擾A aFormer nȂꍇ aMode  'l' Ƃ 
   * aLater ݂̂擾܂B
   *
   * </japanese>
   *
   * @param aFormer an <code>Object[]</code> value
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @param aMode a <code>char</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(
    Object[] aFormer, OhTimePoint aDataTime,
    Object[] aLater, char aMode) {  //; (*6-1)
    if (_numberOfDataPacks <= 0) {
      aFormer[0] = null;
      aLater[0]  = null;
      return false;
    }

    if ((((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).
          getDataTime()).compareTo(aDataTime) < 0) {
      aLater[0] = null;

      if ((aMode == 'b') || (aMode == 'l')) {
        aFormer[0] = null;
        return false;
      }

      aFormer[0] = _dataPackArray.get(_numberOfDataPacks - 1);
      return true;
    }
    else if ((((OhDataPack)_dataPackArray.
                get(_numberOfDataPacks - 1)).getDataTime()).
                compareTo(aDataTime) == 0) {
      aFormer[0] = aLater[0] = _dataPackArray.get(_numberOfDataPacks - 1);
      return true;
    }

    /*
    for (int k = _numberOfDataPacks - 2; k >= 0; k--)
      if ((((OhDataPack)_dataPackArray.get(k)).getDataTime()).
        compareTo(aDataTime) <= 0) {
        aFormer[0] = _dataPackArray.get(k);
        aLater[0]  = (((OhDataPack)aFormer[0]).getDataTime().
          compareTo(aDataTime) == 0)
          ? _dataPackArray.get(k + 1)
          : aFormer[0];
        return true;
      }
    */

    for (int k = _numberOfDataPacks - 2; k >= 0; k--)
      if ((((OhDataPack)_dataPackArray.get(k)).getDataTime()).
        compareTo(aDataTime) <= 0) {
        aFormer[0] = _dataPackArray.get(k);
        aLater[0]  = (((OhDataPack)aFormer[0]).getDataTime().
          compareTo(aDataTime) == 0)
          ? aFormer[0]
          : _dataPackArray.get(k + 1);
        return true;
      }
    aFormer[0] = null;
    aLater[0] = _dataPackArray.get(0);
    if ((aMode == 'b') || (aMode == 'f')) {
      aLater[0] = null;
      return false;
    }
    return true;
  }


  /**
   * Gets datapacks by specifying a time point
   *
   * <japanese>
   * <p>
   * wɂf[^pbN̑Mi擾j <p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> 
   * ̒ۃ\bhłB<p>
   *
   * aFormer ɂ́Af[^L^ aDataTime ȑO aDataTime 
   * ɍł߂^f[^pbN擾Ai[܂B<p>
   *
   * aLater ɂ́Af[^L^ aDataTime ȍ~ aDataTime 
   * ł߂^f[^pbN擾Ai[܂B<p>
   *
   * f[^L^x aDataTime łf[^pbN^
   * Ăꍇ́AaFormer, aLater Ƃi[܂B<p>
   *
   * aFormer, aLater ̗K擾܂B擾łȂꍇ
   * Ƃ null ɐݒ肵܂B
   *
   * </japanese>
   *
   * @param aFormer an <code>Object[]</code> value
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(
    Object[] aFormer, OhTimePoint aDataTime, Object[] aLater) {
    if (_numberOfDataPacks <= 0) {
      aFormer[0] = null;
      aLater[0]  = null;
      return false;
    }

    if ((((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).
          getDataTime()).compareTo(aDataTime) < 0) {
      aLater[0] = null;
      aFormer[0] = null;
      
      return false;
    }
    else if ((((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).
               getDataTime()).compareTo(aDataTime) == 0) {
      aFormer[0] = aLater[0] = _dataPackArray.get(_numberOfDataPacks - 1);
      return true;
    }

    for (int k = _numberOfDataPacks - 2; k >= 0; k--)
      if ((((OhDataPack)_dataPackArray.get(k)).getDataTime()).
        compareTo(aDataTime) <= 0) {
        aFormer[0] = _dataPackArray.get(k);
        aLater[0]  = (((OhDataPack)aFormer[0]).getDataTime().
          compareTo(aDataTime) == 0)
          ? aFormer[0]
          : _dataPackArray.get(k + 1);
        return true;
      }

    aFormer[0] = null;
    aLater[0] = null;
    
    return false;
  }


  /**
   * Deletes unnecessary datapacks.  
   * <a href="OhArrayList.html#method_summary">OhArrayList#removeHeadRange()</a> is used to implement
   * this method.
   *
   * <japanese>
   * <p>
   * svf[^pbN̏ <p>
   *
   * NX
   * OhSenderPort
   * ̒ۃ\bhłB<p>
   *
   * OhSendPort.getNecessaryTimeFrom()
   * O̎^f[^pbN _bufferList A
   * _dataPackArray, _numberOfDataPacks XV܂B
   * <p>
   *
   * ̃\bhƂɁA<a href="OhArrayList.html#method_summary">OhArrayList#removeHeadRange()</a>
   * gpĂ܂B
   * </japanese>
   *
   */
  public void reduceDataPackStock() {    //; (*9)
    if (_protectPastDataFlag == true)
      return;

    if (_numberOfDataPacks <= 0)
      return;

    OhTimePointImmutable reserve = getNecessaryTimeFrom();
    // System.out.println(reserve);

    int k = 0;
    for (k = _numberOfDataPacks - 1; k >= 0; k--) {
      // System.out.println("" + k + ": " +
      //  ((OhDataPack)_dataPackArray.get(k)).getDataTime());
      if ((((OhDataPack)_dataPackArray.get(k)).
            getDataTime()).compareTo(reserve) <= 0)
        break;
    }

    // System.out.println("k = " + k);
    if (k <= 0)
      return;    // No data to be deleted.

    // _dataPackArray.removeHeadRange(k + 1);
    _dataPackArray.removeHeadRange(k);
    _numberOfDataPacks = _dataPackArray.size();
  }


  /**
   * Delete past datapack before "aTFrom",
   * and renew "_dataPackArray" and "_numberOfDataPacks".
   *
   * <japanese>
   * <p>
   * wɂsvf[^pbN̏ <p>
   *
   * aTFrom (arg) O̎^f[^pbN _bufferList 
   * A_dataPackArray, _numberOfDataPacks XV܂B
   *
   * </japanese>
   *
   * @param aTFrom an <code>OhTimePoint</code> value
   */
  public void forgetDataPackStock(OhTimePointImmutable aTFrom) {    //; (*10)
    if (_protectPastDataFlag == true)
      return;

    if (_numberOfDataPacks <= 0)
      return;

    int k = 0;
    for (k = _numberOfDataPacks - 1; k >= 0; k--)
      if ((((OhDataPack)_dataPackArray.get(k)).
            getDataTime()).compareTo(aTFrom) <= 0)
        break;

    if (k <= 0)
      return;    // No data to be deleted.

    // _dataPackArray.removeHeadRange(k + 1);
    _dataPackArray.removeHeadRange(k);
    _numberOfDataPacks = _dataPackArray.size();
  }


  /**
   * Deletion of a recorded datapacks.
   *
   * <japanese>
   * <p>
   * S^f[^pbN̏ <p>
   *
   * _bufferList ̑S^f[^pbNA_dataPackArray ܂B
   * ̌A_dataPackArray  null ɁA_numberOfDataPacks  -1 ɐݒ肳܂B
   * </japanese>
   *
   */
  public void clearDataPack() {    //; (*11)
    _dataPackArray     = null;
    _numberOfDataPacks = -1;
  }

  /**
   * Sets "_protectPastDataFlag".
   *
   * @param aFlag a <code>boolean</code> value
   * @return a <code>boolean</code> value
   */
  public void setProtectPastDataFlag(boolean aFlag) {
    _protectPastDataFlag = aFlag;
  }

  // 3. Function to save the recorded datapacks at a file.

  /**
   * Saves the recorded datapacks at the file, "aPw".
   *
   * <japanese>
   * <p>
   * f[^pbN̏ <p>
   *
   * ^Ăf[^pbNo̓Xg[ aPw Ŏw肳t@Cɏ݂܂B<p>
   *
   * f[^ׂ͂ ( ... ) ̂悤Ɋʂň݂͂܂B
   * ŏ̃f[^́A^Ăf[^pbŇłB
   * f[^pbŇȂA
   * ̂ƂɃf[^pbN𕶎ɕϊAʂň͂񂾂݂̂܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveSendPortStock(PrintWriter aPw) {
    _numberOfDataPacks = _dataPackArray.size(); //; Added at 2003.12.06

    aPw.print("(" + _numberOfDataPacks + ") # _numberOfDataPacks" + NEWLINE);

    if (_numberOfDataPacks <= 0)
      return;

    for (int k = 0; k < _numberOfDataPacks; k++) {
      aPw.print(NEWLINE);
      ((OhDataPack)_dataPackArray.get(k)).dpack2File(aPw);
    }
  }
  
  // @end {methods} --------------------------------------------

} // OhSendPort
