package ohymosj;

/**
 * Interface to access an OhTime object which does not change
 * the state of the object.
 * <p>
 *
 * <japanese>
 * <a href="OhTime.html">OhTime</a> IuWFNgɃANZXC^[tF[XB
 * ԂύXȂ\bh܂Ƃ߂Ă܂B
 * </japanese>
 *
 * @author SHIIBA,Michiharu
 * @version 1.0
 */
public interface OhTimeImmutable {
  /**
   * Returns the integer part of time which is expressed in
   * seconds.
   *
   * @return a <code>long</code> value
   */
  public long getSec();
  
  /**
   * Returns the fractional part of time which is expressed in
   * seconds.
   *
   * @return a <code>double</code> value
   */
  public double getFrac();
  
  /**
   * _frac + _sec.
   *
   * @return a <code>double</code> value
   */
  public double toDouble();
  
  /**
   * Returns a clone of this object
   *
   * @return an <code>Object</code> value
   */
  public Object clone();
  
  /**
   * Returns the boolean variable which shows whether this
   * object is equal to the argument <b>anObj</b> or not.
   *
   * @param anObj an <code>Object</code> value
   * @return a <code>boolean</code> value
   */
  public boolean equals(Object anObj);

  /**
   * Check whether the time difference between the current
   * OhTimeImmutable object and <b>anObj</b> is less than
   * <b>anEps</b>.
   * <p>
   *
   * <japanese>
   * ݂ OhTimeImmutable IuWFNg <b>anObj</b> \킷
   * Ƃ̎ԍ <b>anEps</b> 菬 (true)Ał
   *  (false) Ԃ܂B
   * </japanese>
   *
   * @param anObj an <code>Object</code> value
   * @param anEps a <code>double</code> value
   * @return a <code>boolean</code> value
   */
  public boolean equals(Object anObj, double anEps);
}

