package ohymosj;

/**
 * Interface to access an OhTimePoint object which does not
 * change the state of the object.
 * <p>
 *
 * <japanese>
 * <a href="OhTimePoint.html">OhTimePoint</a> IuWFNgɃANZXC^[tF[
 * XBԂύXȂ\bh܂Ƃ߂Ă܂B
 * </japanese>
 *
 * @author SHIIBA,Michiharu
 * @version 1.0
 */
public interface OhTimePointImmutable {
  /**
   * Returns the year field.
   *
   * @return an <code>int</code> value
   */
  public int getYear();
  
  /**
   * Returns the tsec field.
   *
   * @return a <code>long</code> value
   */
  public long getTsec();
  
  /**
   * Returns the OneYearInSec field.
   *
   * @return a <code>long</code> value
   */
  public long getOneYearInSec();
  
  /**
   * Returns the frac field.
   *
   * @return a <code>double</code> value
   */
  public double getFrac();
  
  /**
   * Returns the CalendarArray
   *
   * @return an <code>int[]</code> value
   */
  public int[] toCalendarArray();
  
  /**
   * Returns the CalendaString to which this object
   * corresponds.  For example, this method returns
   * "20030101010101 0.0" for 2003 January 1st, 01:01:01 0.0
   *
   * <japanese>
   * <p>
   * J_[Ԃ܂BႦ΁Ã݂IuWFNg 
   * 2003 N11111b0.0ȂA"20030101010101 0.0" 
   * Ԃ܂B
   * </japanese>
   *
   * @return a <code>String</code> value
   */
  public String toCalendarString();
  
  /**
   * Returns a deep copy of the current object
   *
   * <japanese>
   * <p>
   * ݂̃IuWFNgƓԂ OhTimePoint ^̐VKIu
   * WFNg쐬ĕԂ܂B
   * </japanese>
   *
   * @return an <code>Object</code> value
   */
  public Object clone();

  /**
   * Tests if the class of the specified Object implements
   * the OhTimePointImmutable interface and if the time point
   * expressed by this object is equal to the time point 
   * expressed by 'arg'.
   *
   * <japanese>
   * <p>
   * Ɏw肵IuWFNg OhTimePointImmutable IuWF
   * NgłāAA݂ OhTimePointImmutable IuWFN
   * gƓԂł邩eXg܂B
   * </japanese>
   *
   * @param arg an <code>Object</code> value
   * @return a <code>boolean</code> value
   */
  public boolean equals(Object arg);
  
  /**
   * Tests if the class of the specified Object 'aObj'
   * implements the OhTimePointImmutable interface and if the
   * time differenc between the time point expressed by this
   * object and the time point expressed by 'anObj' is less than
   * 'anEps'<p>
   *
   * <japanese>
   *  'aObj'  OhTimePointImmutable C^[tF[X
   * NX̃IuWFNgłāAA݂ 
   * IuWFNgƂ̎ԍ 'aEps' 菬eXg܂B
   * <p>
   * </japanese>
   *
   * @param aObj an <code>Object</code> value
   * @param aEps a <code>double</code> value
   * @return a <code>boolean</code> value
   */
  public boolean equals(Object aObj, double aEps);

  public int compareTo(OhTimePointImmutable arg);
}

