/**
 * @(#) OperationFlag; class to express OperationFlag.
 *                     Make "OperationFlag.java" from "ohelement.h" by Satoh.
 */

package ohymosj;

import java.util.*;
import java.io.*;


/**
 * OperationFlag;
 *
 * OperationFlag class. <p>
 *
 * <ul>
 *  <li> E_UNDONE   Initial work could not be done.
 *  <li> E_DONE     Initial work was completed.
 *  <li> E_ALREADY  Initial work had been done already.
 * <li>
 *
 * This type is used in <a href="OhElement.html#method_summary">OhElement#doShareInfo()</a>,
 * <a href="OhElement.html#method_summary">OhElement#doInitialWork()</a>.
 *
 * <japanese>
 * <p>
 * ƂɊւtO\NXB<p>
 *
 * ^ɈSȒ萔̃NXłB<p>
 *
 * <ul>
 *  <li> E_UNDONE  : Ƃ邱ƂłȂB
 *  <li> E_DONE    : ƂsB
 *  <li> E_ALREADY : Ƃ͂łɎsς݂łB
 * <li>
 *
 * R̌^́A
 * <a href="OhElement.html#method_summary">OhElement#doShareInfo()</a>,
 * <a href="OhElement.html#method_summary">OhElement#doInitialWork()</a> ȂǂŗpĂ܂B
 *
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.9 $ $Date: 2004/01/18 15:32:40 $
 */
public final class OperationFlag implements Serializable
{
  // @begin {instance variable or static variable} -----------------------

  private static int nextOrdinal = 0;
  private final  int ordinal     = nextOrdinal++;

  /**
   * Initial work could not be done. <p>
   *
   * <japanese>
   * <p>
   * ^ E_UNDONE ́uƂ邱ƂłȂvƂ\܂B
   * </japanese>
   *
   */
  public static final OperationFlag E_UNDONE  = new OperationFlag();

  /**
   * Initial work was completed. <p>
   *
   * <japanese>
   * <p>
   * ^ E_DONE ́uƂsvƂ\܂B
   * </japanese>
   *
   */
  public static final OperationFlag E_DONE    = new OperationFlag();

  /**
   * Initial work had been done already.
   *
   * <japanese>
   * <p>
   * ^ E_ALREADY ́uƂ͂łɎsς݂łvƂ\܂B
   * </japanese>
   *
   */
  public static final OperationFlag E_ALREADY = new OperationFlag();

  private static final OperationFlag[] PRIVATE_VALUES = {E_UNDONE, E_DONE, E_ALREADY};
  
  // @end {instance variable or static variable} -------------------------
  
  
  // @begin {constructors} -----------------------------------------------
  
  private OperationFlag() {}

  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  private Object readResolve() throws ObjectStreamException {
    return PRIVATE_VALUES[ordinal];
  }

  // @end {methods} ------------------------------------------------------
}
