/**
 * @(#) DataFileContent; class to express TimeControl
 */

package ohymosj;

import java.util.*;
import java.io.*;


/**
 * DataFileContent;
 *
 * DataFileContent class. <p>
 *
 * Definition of DataFileContent
 * <dl>
 *  <dt>PARSET:  <dd>Parameter values are contained in the file.
 *  <dt>INITIAL: <dd>Initial state values are contained in the file.
 *  <dt>SDPDATA: <dd>Send port datapacks are contained in the file.
 * </dl>
 *
 * <japanese>
 * <p>
 * f[^t@C̓e\NXB<p>
 *
 * ^ɈSȒ萔̃NXłB<p>
 *
 * <ul>
 *  <li> PARSET  : f[^t@CɃp^L^ĂB
 *  <li> INITIAL : f[^t@CɏlL^ĂB
 *  <li> SDPDATA : f[^t@CɑM[qɎcĂf[^L^ĂB
 * </ul>
 *
 * (Ref "OhTotalSystem#deliverDataString(String, DataFileContent)")
 * ŎgpĂ܂B
 *
 * </japanese>
 * 
 * 
 * @author satoh, shiiba
 * @version $Revision: 1.9 $ $Date: 2004/01/19 08:00:35 $
 */
public final class DataFileContent implements Serializable
{
  // @begin {instance variable or static variable} -----------------------

  private static int nextOrdinal = 0;
  private final  int ordinal     = nextOrdinal++;

  /**
   * PARSET: <br>
   * Parameter values are contained in the file.
   *
   */
  public static final DataFileContent PARSET  = new DataFileContent();

  /**
   * INITIAL: <br>
   * Initial state values are contained in the file.
   *
   */
  public static final DataFileContent INITIAL = new DataFileContent();

  /**
   * SDPDATA: <br>
   * Send port datapacks are contained in the file.
   *
   */
  public static final DataFileContent SDPDATA = new DataFileContent();

  private static final DataFileContent[] PRIVATE_VALUES = {PARSET, INITIAL, SDPDATA};
  
  // @end {instance variable or static variable} -------------------------
  
  
  // @begin {constructors} -----------------------------------------------
  
  private DataFileContent() {}

  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  private Object readResolve() throws ObjectStreamException {
    return PRIVATE_VALUES[ordinal];
  }

  // @end {methods} ------------------------------------------------------

} // DataFileContent
