/**
 * @(#) Class to express InitialStateDataType.
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * InitialStateDataType;
 *
 * Class to express InitialStateDataType.
 *
 * <japanese>
 * ł̗ʂ̕zǂ̂悤ɗ^邩w肷NXB<p>
 *
 * ^ɈSȒ萔̃NXłB<p>
 *
 * <ul>
 *  <li> ONLY_DOWN_STREAM_END : ł̗ʂ̕z^ƂA
 *       [̗ʂŗ^B
 *  <li> EVERY_SECTION : ł̗ʂ̕z^ƂA
 *       ׂ̒fʂŗʂ^B
 * </ul>
 *
 * (Ref "PowerLawKinematicWave") NXŗpĂ܂B
 *
 * </japanese>
 *
 * @author satoh
 * @version $Revision: 1.5 $ $Date: 2004/01/12 19:57:19 $
 */
public final class InitialStateDataType implements Serializable
{
  // @begin {instance variable or static variable} -------------

  private static int nextOrdinal = 0;
  private final  int ordinal     = nextOrdinal++;

  /**
   * ONLY_DOWN_STREAM_END means that only the dicharge at the
   * downstream end is given.<p>
   *
   * <japanese>
   * ł̗ʂ̕z^ƂA[̗ʂŗ^B
   * </japanese>
   */
  public static final InitialStateDataType ONLY_DOWN_STREAM_END =
    new InitialStateDataType();

  /**
   * EVERY_SECTION means that discharges at every sections are
   * given.<p>
   *
   * <japanese>
   * ł̗ʂ̕z^ƂAׂ̒fʂŗʂ^B
   * </japanese>
   */
  public static final InitialStateDataType EVERY_SECTION =
    new InitialStateDataType();

  private static final InitialStateDataType[] PRIVATE_VALUES =
    {ONLY_DOWN_STREAM_END, EVERY_SECTION};
  
  // @end {instance variable or static variable} ---------------
  
  
  // @begin {constructors} -------------------------------------
  
  private InitialStateDataType() {}

  // @end {constructors} ---------------------------------------

  
  // @begin {methods} ------------------------------------------

  private Object readResolve() throws ObjectStreamException {
    return PRIVATE_VALUES[ordinal];
  }

  // @end {methods} --------------------------------------------
}
