/**
 * ListClasses.java
 *
 * Created: 2004/01/30
 *
 * @author shiiba
 * @version $Revision$
 *
 * @(#) ListClasses; list classes in ohymosj
 *
 * Lists classes contained in the specified jar file.
 *
 *
 * <japanese>
 * $B0z?t$G;XDj$5$l$?(B jar $B%U%!%$%k$K4^$^$l$F$$$k%/%i%9$r%j%9%H$7$^(B
 * $B$9!#(B
 *
 * $B;HMQJ}K!(B:<br>
 *   java -classpath ohymosj.jar ohymosj.ListClasses ohymosj.jar
 * </japanese>
 *
 */

package ohymosj;

import java.lang.reflect.Modifier;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;
import java.io.*;
import java.util.*;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
 * Lists classes contained in the specified jar file.
 * <p>
 *
 * <pre>
 * usage:
 *   java -classpath ~/lib/ohymosj.jar ohymosj.ListClasses ~/lib/foo.jar
 * </pre>
 *
 * @author shiiba
 * @version $Revision$ $Date$
 *
 */
public class ListClasses {
  static String _version = Main._version; 

  public static void main(String args[]) throws Exception {
    System.out.println("ohymosj version " + _version);
    ListClasses.viewClass(args[0]);
  }

  public static void viewClass(String aJarFileName) {
    try {
      JarFile jar = new JarFile(aJarFileName);

      Pattern p = Pattern.compile("/([^.]+)\\.class$");
      Enumeration enum = jar.entries();
      while (enum.hasMoreElements()) {
        JarEntry j = (JarEntry)enum.nextElement();
        Matcher matcher = p.matcher(j.toString());
        if (matcher.find()) {
          System.out.println(matcher.group(1));
        }
      }
    } catch (IOException ex) {
      ex.printStackTrace();
    } catch (SecurityException ex) {
      ex.printStackTrace();
    }      
  }

}// ListClasses
