/**
 * ListPorts.java
 *
 * Created: 2004/01/30
 *
 * @author shiiba
 * @version $Revision$
 *
 * @(#) ListPorts; list abstract classes in ohymosj
 */

package ohymosj;

import java.lang.reflect.Modifier;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;
import java.io.*;
import java.util.*;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
 * Lists abstract classes contained in the specified jar file.
 * <p>
 *
 * <pre>
 * usage:
 *   java -classpath ~/lib/ohymosj.jar ohymosj.ListPorts ~/lib/foo.jar
 * </pre>
 *
 * @author shiiba
 * @version $Revision$ $Date$
 *
 */
public class ListPorts {
  static String _version = Main._version; 

  public static void main(String args[]) throws Exception {
    System.out.println("List of port classes contained in " +
      "ohymosj version " + _version);
    System.out.println("");
    
    ListPorts.viewClass(args[0]);
  }

  public static void viewClass(String aJarFileName) throws Exception {
    try {
      JarFile jar = new JarFile(aJarFileName);
      Pattern p = Pattern.compile("ohymosj/([^.]+)\\.class$");

      OhAllignText line = new OhAllignText(new int[] {0, 20, 50});
      
      Enumeration enum = jar.entries();
      while (enum.hasMoreElements()) {
        JarEntry j = (JarEntry)enum.nextElement();
        Matcher matcher = p.matcher(j.toString());
        if (matcher.find()) {
          String classname = matcher.group(1);
          try {
            Class t = Class.forName("ohymosj." + classname);
            if (!Modifier.isAbstract(t.getModifiers())) {
                Object obj = t.newInstance();
                if (obj instanceof OhPort) {
                  line.clear();
                  line.put(classname);
                  line.nextColumn();
                  line.put("classname:" +
                    ((OhPort) obj).getClassName());
                  line.nextColumn();
                  line.put("datatype:" +
                    ((OhPort) obj).dataType());
                  System.out.println(line.toString());
                  /*
                  System.out.println(classname + "," +
                    "classname:" + ((OhPort) obj).getClassName() + "," +
                    "datatype:" + ((OhPort) obj).dataType()
                  );
                  */
                }
              }
            }
          catch (Exception e) {
          }

        }
      }
    } catch (IOException ex) {
      ex.printStackTrace();
    } catch (SecurityException ex) {
      ex.printStackTrace();
    }      
  }

}// Main
