/**
 * @(#) Main.java
 *
 * Created: 2004/01/30
 *
 * @author shiiba
 * @version $Revision$
 *
 */

package ohymosj;

import java.lang.reflect.Modifier;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;
import java.io.*;
import java.util.*;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Main class of OHyMoSJ.<p>
 *
 * <japanese>
 * Main NX́AOHyMoSJ ̃CNXłBOHyMoSJ ́Aʏ 
 * ohymosj.jar ɃpbP[W邱ƂɂA
 * <pre>
 *         $ java -jar somewhere/ohymosj.jar
 * </pre>
 * ƂƂɁA Main NX main \bhs
 * ɂ܂BŁAsomewhere ́Aohymosj.jar uĂfB
 * Ng̖OłBႦ΁Aohymosj.jar  c:/home/taro ɒu
 * Ă̂ȂAsomewhere  c:/home/taro łB<p>
 *
 * ȉ̐ł́AȒP̂߂ɁA
 * <pre>
 * alias ohymosj='java -jar somewhere/ohymosj.jar'
 * </pre>
 * ƒ`ĂƉ肵Ă܂B<p>
 *
      "usage: ohymosj [-a -e -p] jarfileName\n" +
      "      or\n" +
      "       ohymosj ScfFileName fileBasePart\n" +
      "sample:\n" +
      "   ohymosj -e build/lib/ohymosj.jar\n" +
      "   ohymosj foo.scf arakura\n"
 * <ol>
 * <li> w肵ȂŁA
 * <pre>
 *         $ ohymosj
 * </pre>
 * ƂƂɂ́Aohymosj ̎gp@\܂B
 * <p>
 *
 * <li> ohymosj.jar Ɋ܂܂Ă钊ۃNX\Ƃ́A
 * <pre>
 *         $ ohymosj -a somewhere/ohymosj.jar
 * </pre>
 * Ƃ܂B<p>
 *
 * <li> ohymosj.jar Ɋ܂܂Ă[qf\Ƃ́A
 * <pre>
 *         $ ohymosj -p somewhere/ohymosj.jar
 * </pre>
 * Ƃ܂B<p>
 *
 * <li> ohymosj.jar Ɋ܂܂Ăvff\Ƃ́A
 * <pre>
 *         $ ohymosj -e somewhere/ohymosj.jar
 * </pre>
 * Ƃ܂B<p>
 *
 * <li> ovZsɂ́A
 * <pre>
 *         $ ohymosj scfFileName fileBasePart
 * </pre>
 * Ƃ܂BscfFileName ́A\`t@C̃t@CłB
 * fileBasePart ́AvZs邽߂ɕKvȓ̓t@C̃t@
 * C̊łB\`t@C̒ɋLqt@C
 * ̃t@ĆAfileBasePart Ŏw肷镶t@C̊
 * ɂȂ悤ɂ܂B
 * <p>
 *
 * Ⴆ΁A
 * <pre>
 *         $ ohymosj totalkinArakura.scf tplkinArakura
 * </pre>
 * ̂悤ɂ܂BtotalkinArakura.scf ̍\`t@C
 * łB\`t@CɋLq鏉lt@CAp^t@
 * C̃t@ĆAtplkinArakura.i, tplkinArakura.pmt,
 * ... Ƃ܂B
 * </ol>
 * </japanese>
 *
 * @version $Revision$ $Date$
 *
 */
public class Main implements OhConst {
  static String _version = "2.0";

  public static void main(String args[]) throws Exception {
    OhOption op = OhOption.getDefault();
    
    boolean a_flag = false;
    boolean p_flag = false;
    boolean e_flag = false;
    String jarfileName = "";
    char c;
    int i = 0;
    while (i < args.length && args[i].charAt(0) == '-') {
      if (args[i] == "--") break;
      c = args[i].charAt(1);
      switch (c) {
      case 'a':
        a_flag = true;
        break;
      case 'e':
        e_flag = true;
        break;
      case 'p':
        p_flag = true;
        break;
      case 'w':
        op.setUseWinP(true);
        break;
      default:
        System.err.println("Unknown option character " + c);
        usage();
        System.exit(1);
      }
      i++;
    }
    int nstart = i;
    if (a_flag || e_flag || p_flag) {
      if (nstart >= args.length) {
        System.err.println(
          "You have to specify the jarfile name.");
        usage();
        System.exit(1);
      }
      else {
        jarfileName = args[nstart];
      }
    }
    else {
      if (nstart+1 > args.length) {
         usage();
         System.exit(0);
      }
    }
      
    if (a_flag) {
      System.out.println("List of abstract classes contained in " 
        + jarfileName);
      System.out.println("");
      ListAbstractClasses.viewClass(jarfileName);
      System.exit(0);
    }
    if (e_flag) {
      System.out.println("List of element classes contained in " 
        + jarfileName);
      System.out.println("");
      ListElements.viewClass(jarfileName);
      System.exit(0);
    }
    if (p_flag) {
      System.out.println("List of port classes contained in " 
        + jarfileName);
      System.out.println("");
      ListPorts.viewClass(jarfileName);
      System.exit(0);
    }

    // show the ohymos's version number
    boolean useWinP = op.getUseWinP();
    if (useWinP) {
      // When the user selects the window-based interface,
      // OhIcon is shown.
      //
      OhWin ow = OhWin.getDefault();
      ow.setSize(200,50);
      ow.setMessage("OHyMoSJ " + "version " + _version);
      ow.setVisible(true);
    }
    else {
      // When the user selects the terminal-based interface.
      //
      System.out.println("OHyMoSJ " + "version " + _version);
    }

    try {
      if (useWinP) {
        // JPanel pane = (JPanel)OhWin.getDefault().getContentPane();
        ImageIcon myIcon = new ImageIcon("ohymos.png");
        JOptionPane.showConfirmDialog(
          null,                        //; Parent frame.
          "܂vZ܂B",      //; Message.
          "OHyMoSJ",                   //; Title.
          JOptionPane.DEFAULT_OPTION,  //; Option type.
          JOptionPane.PLAIN_MESSAGE,   //; Message type.
          myIcon                       //; HY icon.
          );
        OhWin.getDefault().setMessage("Wait. Now, I am working.");
      }
      else {
        System.out.println("܂vZ܂BReturn L[ĂB");
        InputStreamReader isr = new InputStreamReader(System.in);
        BufferedReader br = new BufferedReader(isr);
        String ignore = br.readLine();
        System.out.println("Wait. Now, I am working.");
      }
//    Thread.sleep(4000);


//    System.out.println(args[0]);  --> total.scf
      
//    System.out.println("Now create totalSystem");
      OhScfTotalSystem totalSystem =
        new OhScfTotalSystem(args[nstart]);
//    System.out.println("complete creating totalSystem");

      String fileBasepart = args[nstart+1];
      
      totalSystem.setOhymosjOption();

//      System.out.println("construct");
      totalSystem.construct();

//      System.out.println("doSwitch");
      totalSystem.doSwitch();

//      System.out.println("areYouReady");
      
      if (totalSystem.areYouReady() != true) {
        System.out.print(NEWLINE);
        System.out.print("# Failure : areYouReady().");
        System.out.print(NEWLINE);
        System.exit(1);
      }

      String pmtFileName = fileBasepart + ".pmt";
      totalSystem.readParameterFile(pmtFileName);

      String istFileName = fileBasepart + ".ist";
      totalSystem.readInitialStateFile(istFileName);

      String isdFileName = fileBasepart + ".isd";
      totalSystem.readSendPortFile(isdFileName);

//;   totalSystem.doShareInfo();  //; KvH
      
      totalSystem.initialWork();

//;   System.out.println("Finish initialWork().");

      totalSystem.work();

//      System.out.println("Finish simulation!");  //; mFpWo

      
      String tstFileName = fileBasepart + ".tst";
      totalSystem.writeTerminalState(tstFileName);

      String tsdFileName = fileBasepart + ".tsd";
      totalSystem.writeSendPortStock(tsdFileName);

//      System.out.println("Finish saving!");  //; mFpWo

    }
//  catch (InterruptedException ignored) {
//  }
    catch (OhCannotGetDataError e) {
      System.out.println(e.what());
    }
    catch (OhError e) {
      OhInterface.pause(e.what());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (IllegalArgumentException e) {
      e.printStackTrace();
    }
    catch (ClassCastException e) {
      e.printStackTrace();
    }
    catch (NullPointerException e) {
      e.printStackTrace();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    
    //; OHyMoSJ ̏I ------------------------------------------------

    if (useWinP) {
      //; System.exit ĂԂƁASwing ֌W̊IĂ܂B
      System.exit(0);
    }
  }

  /**
   * Shows the usage of the Main class of ohymosj.jar
   *
   */
  public static void usage() {
    System.out.println(
      "usage: ohymosj [-a -e -p] jarfileName\n" +
      "      or\n" +
      "       ohymosj ScfFileName fileBasePart\n" +
      "sample:\n" +
      "   ohymosj -e build/lib/ohymosj.jar\n" +
      "   ohymosj foo.scf arakura\n"
    );
  }

}// Main
