/**
 * @(#) Class to allign texts.
 */

package ohymosj;

/**
 * Class to allign texts.<p>
 *
 * <japanese>
 * (Ref "OhAllignText") ́A̂悤ɃJ̊Jnʒu𑵂
 * o񂪂Ƃɗp܂B
 *
 *   shiiba     michiharu    old
 *   hori       tomoharu     middle
 * 
 * z tabpos ɁA̍ŏ̈ʒuZbg܂B̗ł΁A
 * 
 *   int[] tabpos = { 2, 9, 19 };
 * 
 * ĂāA
 * 
 *   AllignText line = new AllignText(tabpos);
 * 
 * Ƃ܂B
 *
 * ܂Ƃ߂āA
 *
 *   OhAllignText line = new OhAllignText(new int[] {0, 9, 19});
 *
 * ̂悤ɏĂ\܂B̌A
 * 
 *   line.clear();
 *   line.put("shiiba");
 *   line.nextColumn();
 *   line.put("michiharu");
 *   line.nextColumn();
 *   line.put("old");
 *   System.out.println(line.toString());
 * 
 *   line.clear();
 *   line.put("hori");
 *   line.nextColumn();
 *   line.put("tomoharu");
 *   line.nextColumn();
 *   line.put("middle");
 *   System.out.println(line.toString());
 * 
 * ̂悤Ɏgp܂B
 * </japanese>
 *
 * @version $Revision$ $Date$
 */
public class OhAllignText {
  private StringBuffer _line;
  private int[] _tabpos;
  private int _ntab;
  private int _column;
  private int _lasttab;

  private final int TABWIDTH = 8;

  /**
   * Creates a new <code>OhAllignText</code> instance.
   *
   */
  public OhAllignText() {
    _line = new StringBuffer();
    _tabpos = null;
    _ntab = 0;
    _lasttab = 0;
    clear();
  }

  public OhAllignText(int[] tabpos) {
    _tabpos = new int[tabpos.length];
    System.arraycopy(tabpos, 0, _tabpos, 0, tabpos.length);
    _line = new StringBuffer();
    _ntab = _tabpos.length;
    _lasttab = _tabpos[_tabpos.length - 1];
    clear();
  }

  public void clear() {
    _column = 0;
    if (_line.length() > 0) {
      _line.delete(0, _line.length());
    }
    if (_ntab > 0) {
      for (int i = 0; i < _tabpos[0]; i++)
        _line.append(" ");
    }
  }

  public String toString() {
    return _line.toString();
  }

  public void put(String str) {
    _line.append(str);
  }

  public void nextColumn() {
    int tpos = 0;
    _column += 1;
    if (_column < _ntab) {
      tpos = _tabpos[_column];
    }
    else {
      tpos = _lasttab + (_column - _ntab) * TABWIDTH;
    }
    _line.append(" ");
    int pos = _line.length();
    while (pos < tpos) {
      _line.append(" ");
      ++pos;
    }
  }

}// OhAllignText
