/**
 * @(#) OhCannotGetDataError; ohymosj error exception class.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * OhCannotGetDataError;
 *
 * OHyMoSJ error exception class. Error is a derived class of
 * the <code>Exception</code> class. <p>
 *
 * The <code>OhCannotGetDataError</code> class object
 * constructor is used as
 *
 * <pre>
 * .....
 *   try {
 *     ...
 *     OhCannotGetDataError e = new OhCannotGetDataError(
 *       errorMessage1, errorMessage2);
 *     throw e;
 *   }
 *   catch (OhCannotGetDataError e) {
 *     System.out.print(e.what());
 *   }
 * .....
 * </pre>
 *
 * The example of the creation method of an error message is as
 * follows.
 *
 * <pre>
 * .....
 *   if(some error status was detected) {
 *     StringWriter buffer = new StringWriter();
 *     PrintWriter  out    = new PrintWriter(buffer);
 *     oPrint(out, " > ");
 *     String str = buffer.toString();
 *     out.close();
 *     throw new OhCannotGetDataError("Already exits \"_dataPackArray\"." + str);
 *   }
 * .....
 * </pre>
 *   
 * @author satoh, shiiba
 * @version $Revision: 1.9 $ $Date: 2004/01/16 00:02:31 $
 */
public class OhCannotGetDataError extends Exception
{
  // @begin{field} -------------------------------------------------------

  /**
   * Error message.
   */
  String _errorMessage;

  // @end{field} ---------------------------------------------------------

  
  // @begin{constructor} -------------------------------------------------

  /**
   * Creates a new <code>OhCannotGetDataError</code> instance.
   *
   * @param aStr a <code>String</code> value
   */
  public OhCannotGetDataError(String aStr) {
    _errorMessage = aStr;
  }
  
  // @end{constructor} ---------------------------------------------------

  
  // @begin{methods} -----------------------------------------------------

  /**
   * Returns the message that was given when the exception object was thrown.
   *
   * @return a <code>String</code> value
   */
  public String what() {
    return _errorMessage;
  }

  /**
   * Returns the message which is made by adding 'aPrefix'
   * before each line of the message that was given when the
   * exception object was thrown.
   *
   * @param aPrefix a <code>String</code> value
   * @return a <code>String</code> value
   */
  public String what(String aPrefix) {
    return OhCommon.addPrefix(_errorMessage, aPrefix);
  }

  // @end{methods} -------------------------------------------------------

} // OhCannotGetDataError
