/**
 * @(#) Class to express datapacks of type double.
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * OhDataPackDouble is a class of datapacks of type double. <p>
 *
 * OhDataPackDouble is derived from OhDataPack. <p>
 *
 * A datapack of type double is a pair of data of data time and
 * data of type double.
 * <p>
 *
 * The class OhDataPackDouble is immutabale, that is, once the
 * values of the field varibles have been set by one of the
 * constructors, the values of the field variables cannot be
 * changed.
 *
 * <japanese>
 * <p>
 * OhDataPack pč쐬Adouble ^f[^pbNNX
 * B舵f[^ľ^́Adouble łB
 *
 * NX OhDataPackDouble ́Aimmutable łBRXgN^ŁA
 * tB[hϐݒ肵́AtB[hϐ̒lQƂ邱
 * Ƃ͂ł܂AύX͂ł܂BtB[hϐ̒lύX
 * 郁\bh͒񋟂Ă܂B
 *
 * </japanese>
 *
 * @author satoh, shiiba 
 * @version $Revision: 1.21 $ $Date: 2004/01/18 14:18:54 $
 */
public class OhDataPackDouble extends OhDataPack
  implements Serializable, Cloneable
{
  // @begin{field} ---------------------------------------------

  /**
   * <japanese>
   * <p>
   * double ^̃f[^i[郁oϐłB
   * </japanese>
   */
  private double _dataValue;

  // @end{field} -----------------------------------------------
  
  // @begin{constructors} --------------------------------------

  /**
   * Creates a new default <code>OhDataPackDouble</code> instance.
   *
   */
  public OhDataPackDouble() { _dataValue = 0.0; }
  
  /**
   * Creates a new <code>OhDataPackDouble</code> instance.
   *
   * @param aDataTime an <code>OhTimePointImmutable</code> value
   * @param aDataValue a <code>double</code> value
   */
  public OhDataPackDouble(OhTimePointImmutable aDataTime, double aDataValue) {
    _dataTime.copy(aDataTime);    //; Field of the super class OhDataPack.
    _dataValue = aDataValue;
  }

  /**
   * Creates a new <code>OhDataPackDouble</code> instance.
   *
   * <japanese>
   * <p>
   * aDataStr Ɋi[Ăf[^pbN񂩂f[^pbN
   * 𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackDouble(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;
    double dataValue = 0.0;

    try {    
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataTime.setFrac( Double.parseDouble(br.readWord()) );
      dataValue = Double.parseDouble(br.readWord());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
    _dataValue = dataValue;
  }
    
  // @end{constructors} ----------------------------------------

  // @begin{implementation of abstract methods} ----------------
  
  /**
   * Returns the data type "double".<p>
   *
   * <japanese>
   * f[^ľ^ƂāA"double" Ԃ܂B
   * </japanese>
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "double";
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B<p>
   *
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂
   * Ĉsɏo܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * 2003 38600 0.3 3.1
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() +
      " " + _dataTime.getTsec() +
      " " + _dataTime.getFrac() +
      " " + _dataValue);
  }

  /**
   * <japanese>
   * <p>
   * f[^pbNptH[}bgŃt@Cɏo܂B<p>
   *
   * ptH[}bgƂ́A_year, _tsec, _frac, _dataValue ̂
   * ijʂł̂ЂƂsďo
   * ܂B
   *
   * <pre>
   * [EXAMPLE]
   *
   * (2003)
   * (38600)
   * (0.3)
   * (3.1)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataTime.getFrac() + ")" + NEWLINE);
    os.print("(" + _dataValue + ")" + NEWLINE);
  }

  // @end{implementation of abstract methods} ------------------

  // @begin{methods} -------------------------------------------

  /**
   * Returns a deep copy of the current OhDataPackDouble object<p>
   *
   * <japanese>
   * ݂ OhDataPackDouble IuWFNg̃N[ ([Rs[) 
   * Ԃ܂B
   * </japanese>
   *
   * @return an <code>Object</code> value
   */
  public Object clone() {
    OhDataPackDouble dp = new OhDataPackDouble(_dataTime, _dataValue);
    return dp;
  }

  /**
   * Returns the data value.<p>
   *
   * <japanese>
   * f[^pbÑf[^l (^́Adouble) Ԃ܂B
   * </japanese>
   *
   * @return a <code>double</code> value
   */
  public double getDataValue() {
    return _dataValue;
  }

  
  /**
   * Multiplies the parameter aFactor to the instance variable
   * _dataValue.<p>
   *
   * <japanese>
   *  aFactor ̒lCX^Xϐ _dataValue Ɋ|܂B
   * </japanese>
   *
   * @param aFactor a <code>double</code> value
   */
  public void multiply(double aFactor) {
    _dataValue *= aFactor;
  }
  
  /**
   * Returns the string expression of this object
   *
   * @return a <code>String</code> value
   */
  public String toString() {
    return _dataTime.toString() + " " + _dataValue;
  }

  // @end{methods} ---------------------------------------------

} // OhDataPackDoule
