// @(#)

package ohymosj;

import junit.framework.*;
import java.io.*;

public class OhDataPackDoubleArrayTest extends TestCase
{
  OhDataPackDoubleArray _a;
  
  public OhDataPackDoubleArrayTest(String name) {
    super(name);
  }
  
  public void setUp() {
    _a = new OhDataPackDoubleArray();
  }
  
  public void tearDown() {
  }

  // @begin{constructors check} ----------------------------------

  public void testConstructors() {
    OhDataPackDoubleArray dp = new OhDataPackDoubleArray();
    OhTimePointImmutable tp = dp.getDataTime();
    assertEquals("testConstructorsA", true, tp.equals(new OhTimePoint()));

    double[] a = new double[2];
    a[0] = 1.;  a[1] = 2;
    dp = new OhDataPackDoubleArray(new OhTimePoint(2003, 10, 0.2), a);
    tp = dp.getDataTime();
    assertEquals("testConstructorsA", false, tp.equals(new OhTimePoint()));
    double[] b = dp.getDataValue();
    assertEquals("testConstructorsB", b[1], 2., 0.0001);
  }

  /**
   * test of
   *   public OhDataPackDoubleArray(String aDataStr)
   *
   */
  public void test_OhDataPackDoubleArrayString() {
    String str = "2003 10 0.1 2 1.1 2.2";
    OhDataPackDoubleArray dp = new OhDataPackDoubleArray(str);
    OhTimePointImmutable tp = dp.getDataTime();
    double[] a = dp.getDataValue();
    assertEquals("test_OhDataPackDoubleArrayString", 1.1, a[0], 0.001);
    assertEquals("test_OhDataPackDoubleArrayString", 2.2, a[1], 0.001);
  }
  
  // @end{constructors check} ------------------------------------

  // @begin{methods check} ---------------------------------------

  public void test_dataType() {
    OhDataPackDoubleArray dp = new OhDataPackDoubleArray();
    assertEquals("test_dataType", (Object)"doubleArray", dp.dataType());
  }

  public void test_dpack2Ofile() {
    OhTimePoint tp = new OhTimePoint(2003, 10, 0.3);
    double[] data = { 1.1, 2.2 };
    OhDataPackDoubleArray dp = new OhDataPackDoubleArray(tp, data);

    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    dp.dpack2Ofile(out);
    assertEquals("test_dpack2Ofile", (Object)"2003 10 0.3 2 1.1 2.2",
      buffer.toString());
  }

  public void test_dpack2File() {
    String newline = System.getProperty("line.separator");

    OhDataPackDoubleArray dpa =
      new OhDataPackDoubleArray("2003 10 0.1 2 1.1 2.2");

    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    dpa.dpack2File(out);
    String result = buffer.toString();
    String expected = "(2003)" + newline +
      "(10)" + newline +
      "(0.1)" + newline +
      "(2)" + newline +
      "(1.1)" + newline +
      "(2.2)" + newline;
    assertEquals("test_dpack2File", (Object) expected, result);
  }

  public void test_getDataValue() {
    OhTimePoint tp = new OhTimePoint(2003, 10, 0.3);
    double[] data = { 1.1, 2.2 };
    OhDataPackDoubleArray dp = new OhDataPackDoubleArray(tp, data);
    double[] cd = dp.getDataValue();
    boolean isequal = true;
    for (int i = 0; i < data.length; i++) {
      if (Math.abs(data[i] - cd[i]) > 0.0001) {
        isequal = false; break;
      }
    }
    assertEquals("test_getDataValue", true, isequal);
  }

  public void test_getDataTime() {
    OhTimePoint tp = new OhTimePoint(2003, 10, 0.3);
    double[] data = { 1.1, 2.2 };
    OhDataPackDoubleArray dp = new OhDataPackDoubleArray(tp, data);

    assertEquals("test_getDataA", true,
      dp.getDataTime().equals(tp, 0.001));
  }
  
  // @end{methods check} -----------------------------------------
}

