// @(#) class OhDataPackDoubleTest

package ohymosj;

import junit.framework.TestCase;
import java.io.*;

public class OhDataPackDoubleTest extends TestCase
{
  public OhDataPackDoubleTest(String name) {
    super(name);
  }
  
  public void setUp() {
  }
  
  public void tearDown() {
  }

  // @begin{constructors check} ----------------------------------

  public void testOhDataPackDoubleVoid() {
    OhDataPackDouble a = new OhDataPackDouble();
    assertEquals("testOhDaaPackDoubleVoidA",
      true, a.getDataTime().equals(new OhTimePoint()));
    assertEquals("testOhDaaPackDoubleVoidB",
      0., a.getDataValue(), 0.0001);
  }

  public void testOhDataPackDoubleTimePointDouble() {
    OhTimePoint tp = new OhTimePoint(2003, 10, 0.1);
    OhDataPackDouble a = new OhDataPackDouble(tp, 1.3);
    assertEquals("testOhDataPackDoubleTimePointDoubleA",
      true, a.getDataTime().equals(tp));
    assertEquals("testOhDataPackDoubleTimePointDoubleB",
      1.3, a.getDataValue(), 0.0001);
  }

  public void testOhDataPackDoubleString() {
    String str = "2003 10 0.1 1.3";
    OhDataPackDouble a = new OhDataPackDouble(str);
    OhTimePointImmutable tp = a.getDataTime();
    assertEquals("testOhDataPackDoubleTimePointString",
      2003, tp.getYear());
    assertEquals("testOhDataPackDoubleTimePointString",
      10, tp.getTsec());
    assertEquals("testOhDataPackDoubleTimePointString",
      0.1, tp.getFrac(), 0.0001);
    assertEquals("testOhDataPackDoubleTimePointString",
      1.3, a.getDataValue(), 0.0001);
  }
  
  // @end{constructors check} ------------------------------------

  // @begin{methods check} ---------------------------------------

  /**
   * test of
   *   public Object clone()
   *
   */
  public void test_clone() {
    OhDataPackDouble dp = new OhDataPackDouble(
      new OhTimePoint(2003, 10, 0.0), 2.1);
    OhDataPackDouble dpc = (OhDataPackDouble) dp.clone();
    assertEquals("test_clone", 2.1, dpc.getDataValue(), 0.0001);
    assertEquals("test_clone", true,
      dpc.getDataTime().equals(new OhTimePoint(2003, 10, 0.0)));
  }
  
  /**
   * test of
   *   public String dataType() 
   * 
   */
  public void test_dataType() {
    OhDataPackDouble dpd = new OhDataPackDouble();
    assertEquals("test_dataType", (Object)"double", dpd.dataType());
  }

  public void test_dpack2Ofile() {
    OhDataPackDouble dpa = new OhDataPackDouble("2003 10 0.1 1.3");
    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    dpa.dpack2Ofile(out);
    String result = buffer.toString();
    assertEquals("test_dpack2Ofile", (Object)"2003 10 0.1 1.3", result);
  }
  
  /**
   * test of
   *  public double getDataValue()
   */
  public void test_getDataValue() {
    OhDataPackDouble dp = new OhDataPackDouble(
      new OhTimePoint(2003, 10, 0.0), 2.1);
    assertEquals("test_getDataValue", 2.1, dp.getDataValue(), 0.0001);
  }
  
  /**
   * test of
   *   public void dpack2File(PrintWriter os)
   *
   */
  public void testDpack2File() {
    String newline = System.getProperty("line.separator");

    OhDataPackDouble dpa = new OhDataPackDouble("2003 10 0.1 1.3");

    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    dpa.dpack2File(out);
    String result = buffer.toString();
    String expected = "(2003)" + newline +
      "(10)" + newline +
      "(0.1)" + newline +
      "(1.3)" + newline;
    assertEquals("testDpack2File", expected, result);
  }

  public void test_multiply(double aFactor) {
    OhDataPackDouble dp =
      new OhDataPackDouble("2003 10 0.1 1.3");
    dp.multiply(2.0);
    assertEquals("1", 2.6, dp.getDataValue(), 0.001);
  }

  // @end{methods check} -----------------------------------------
}

