// @(#) OhInputFileDoubleTest.java

// <-- (Ref "OhObject")
// <-- (Ref "OhPort")
// <-- (Ref "OhSenderPort")
// <-- (Ref "OhSendPort")
// <-- (Ref "OhInputPort")
// <-- (Ref "OhInputFile")
// <-- (Ref "OhInputFileDouble")

package ohymosj;

import junit.framework.*;
import java.io.*;

public class OhInputFileDoubleTest extends TestCase
  implements OhConst
{
  public OhInputFileDoubleTest(String name) {
    super(name);
  }
  
  public void setUp() {
  }
  
  public void tearDown() {
  }

  // @begin{constructors check} ----------------------------------
  // @end{constructors check} ------------------------------------

  // @begin{methods check} ---------------------------------------

  public void test_dataType() {
    OhInputFileDouble ifd = new OhInputFileDouble();
    assertEquals("1", "double", ifd.dataType());
  }

  public void test_makeDataPack() {
    OhInputFileDouble ifd = new OhInputFileDouble();
    OhDataPack dpd = ifd.makeDataPack("2003 10 0.2 1.5");
    assertEquals("1", true,
      (new OhTimePoint(2003,10,0.2)).
      equals(dpd.getDataTime(), 0.001));
    assertEquals("2", 1.5,
      ((OhDataPackDouble) dpd).getDataValue(), 0.001);
  }

  /**
   * (Ref "
   *
   * @exception OhError if an error occurs
   */
  public void test_sendByString() throws OhError {
    OhInputFileDouble ifd = new OhInputFileDouble();
    ifd.doSwitch();
    ifd.sendByString("2003 0 0.0 1.5");
    ifd.setDataPackArray();
    OhArrayList alist = ifd.getDataPackArray();
    assertEquals("1", 1, alist.size());
    OhDataPackDouble dpd = (OhDataPackDouble) alist.get(0);
    assertEquals("2", true,
      (new OhTimePoint(2003, 0, 0.0)).
      equals(dpd.getDataTime(), 0.001));
    assertEquals("3", 1.5, dpd.getDataValue(), 0.001);
  }

  /**
   * <japanese>
   * ̃eXg (Ref "OhInputFile#inputData()") eXg邽
   * ߂̂̂łB
   * </japanese>
   *
   */
  public void test_inputData() {
    // Create sample data
    String lines =
      "# rainfall.ia" + NEWLINE +
      "# year sec frac r (m^3/sec)" + NEWLINE +
      "#& dctVec 1 1" + NEWLINE +
      NEWLINE +
      "2003 0 0.0 0.0" + NEWLINE +
      "2003 7200 0.0 5.0" + NEWLINE +
      "2003 14400 0.0 0.0" + NEWLINE;
    String filename = "sample.txt";
    try {
      PrintWriter pw = new PrintWriter(
        new FileOutputStream(filename));
      pw.print(lines);
      pw.flush();
      pw.close();
    } catch (IOException e) {}

    OhInputFileDouble ifd = new OhInputFileDouble("sample.txt");
    try {
      ifd.doSwitch();
      ifd.inputData();
      ifd.setDataPackArray();

      assertEquals("1", 3, ifd.getNumberOfDataPacks());
      OhArrayList dpa = ifd.getDataPackArray();
      assertEquals("2", 5.0,
        ((OhDataPackDouble) dpa.get(1)).getDataValue(), 0.001);
      assertEquals("3", true,
        ((OhDataPackDouble) dpa.get(2)).getDataTime().
        equals(new OhTimePoint(2003,14400,0.0), 0.001));
    }
    catch (OhError e) {
      System.out.println(e.what());
    }

    File f = new File(filename);
    if (!f.delete()) {
      System.out.println("could not delete " + filename);
    }
  }

  public void test_parseSpecLine() {
    // Create sample data
    String lines =
      "# rainfall.ia" + NEWLINE +
      "# year sec frac r (m^3/sec)" + NEWLINE +
      "#& dctVec 1 1" + NEWLINE +
      "#& multiply 2.0" + NEWLINE +
      NEWLINE +
      "2003 0 0.0 0.0" + NEWLINE +
      "2003 7200 0.0 5.0" + NEWLINE +
      "2003 14400 0.0 0.0" + NEWLINE;
    String filename = "sample.txt";
    try {
      PrintWriter pw = new PrintWriter(
        new FileOutputStream(filename));
      pw.print(lines);
      pw.flush();
      pw.close();
    } catch (IOException e) {}

    OhInputFileDouble ifd = new OhInputFileDouble("sample.txt");
    try {
      ifd.doSwitch();
      ifd.inputData();
      ifd.setDataPackArray();

      assertEquals("1", 3, ifd.getNumberOfDataPacks());
      OhArrayList dpa = ifd.getDataPackArray();
      assertEquals("2", 10.0,
        ((OhDataPackDouble) dpa.get(1)).getDataValue(), 0.001);
      assertEquals("3", true,
        ((OhDataPackDouble) dpa.get(2)).getDataTime().
        equals(new OhTimePoint(2003,14400,0.0), 0.001));
    }
    catch (OhError e) {
      System.out.println(e.what());
    }

    File f = new File(filename);
    if (!f.delete()) {
      System.out.println("could not delete " + filename);
    }
  }
  
  // @end{methods check} -----------------------------------------
}

