/**
 * @(#) OhInputFileInt: Derived from OhInputFile.
 *
 * (Ref "OhObject")
 * <-- (Ref "OhPort")
 * <-- (Ref "OhSenderPort")
 * <-- (Ref "OhSendPort")
 * <-- (Ref "OhInputPort")
 * <-- (Ref "OhInputFile")
 */

package ohymosj;

import java.util.*;
import java.io.*;


/**
 * The OhInputFileInt class is a class of input port which
 * reads in datapacks of type int from an input file and
 * sends them. The OhInputFileInt class extends
 * (Ref "OhInputFile").<p>
 *
 * OhInputFileInt can be connected with
 * (Ref "OhReceiveInt"),
 * (Ref "OhRelayInt"), and 
 * (Ref "OhOutputFileInt"). <p>
 *
 * OhInputFileInt reads an input file using "simple data format
 * for input and output." <p>
 *
 * "Simple data format for input and output" is a format in which
 *
 * <pre>
 *  .....
 *    all the data items for a datapack should be written in one
 *    line, and data items should be separated with a number of
 *    spaces or tabs.
 *
 *    Example: 2003 120 0.3 2 4 5
 * ....
 * </pre>
 *
 * If you need different format, please make a derived class of
 * OhInputFileInt and redefine "makeDataPack()".
 *
 * <japanese>
 * <p>
 * t@C int ^̃f[^pbNǂݎ͒[q <p>
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.3 $ $Date: 2004/01/17 13:03:35 $
 */
public class OhInputFileInt extends OhInputFile
{
  // @begin{initialization} ------------------------------------

  /**
   * Initializ the object. This method assigns the argument
   * aFileName to the instance variable
   * (Ref "OhObject._className").<p>
   *
   * @param aFileName a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void init(String aFileName) throws OhError {
    super.init(aFileName);
  }

  // @end{initialization} --------------------------------------

  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>OhInputFileInt</code> instance.
   */
  public OhInputFileInt() {
    super();
    _className = "OhInputFileInt";
  }


  /**
   * Creates a new <code>OhInputFileInt</code> instance.
   *
   * @param aFileName a <code>String</code> value
   */
  public OhInputFileInt(String aFileName) {
    super(aFileName);
    _className = "OhInputFileInt";
  }

  // @end {constructors} ---------------------------------------

  
  // @begin {methods} ------------------------------------------
  
  // 1. connection function.

  /**
   * Returns the data pack type which expresses the type of data
   * to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "int";
  }


  // 2. data input function

  /**
   * Reads string datapacks of type int, creates datapacks of
   * type int, and returns it.
   *
   * <japanese>
   * <p>
   *  aLineData Ɋi[Ăf[^pbN񂩂f[^
   * pbN쐬AObject ^ɃLXgĖ߂܂B<p>
   *
   * ygpz<p>
   * Object dataPack = makeDataPack("2003 12 0.3 3");<p>
   * 
   * ̗ႾƁAf[^ OhTimePoint(2003, 12, 0.3)AɃf[^
   * l 3 f[^pbNǂݎ܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   * @return an <code>OhDataPack</code> value
   */
  public OhDataPack makeDataPack(String aDataStr) {
    OhDataPackInt dp = new OhDataPackInt(aDataStr);
    return dp;
  }


  /**
   * Sends datapacks of type int which is stored in the form of
   * string.
   *
   * <japanese>
   * <p>
   *  aStr Ɋi[Ăf[^pbN񂩂f[^pb
   * N쐬A͒[qɑM܂B<p>
   *
   * \bh̒` (Ref "OhSendPort#dataPackStock()") gp
   * Ă܂B<p>
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public void sendByString(String aDataStr) {
    OhDataPackInt dp = new OhDataPackInt(aDataStr);
    dataPackStock(dp);
  }

  // @end {methods} --------------------------------------------

} // OhInputFileInt
