/**
 * @(#) Receives datapacks of type float and write them into a
 * file.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * OhOutputFileFloat;
 *
 * OhOutputFileFloat is a class of output ports which receive
 * datapacks of type float and print them in a file. <p>
 *
 * OhOutputFileFloat receives float data.
 * OhOutputFileFloat can be connected with the classes
 * (Ref "OhSendFloat"),
 * (Ref "OhRelayFloat"), and 
 * (Ref "OhInputFileFloat").
 *
 * <japanese>
 * <p>
 * float ^̃f[^pbNMăt@CɋL^o͒[q <p>
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.1 $ $Date: 2004/01/17 13:10:43 $
 */
public class OhOutputFileFloat extends OhOutputFile
{
  // @begin{initialization} ------------------------------------

  /**
   * Initialization.
   *
   * @param aFileName a <code>String</code> value
   * @param aLineLength an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public void init(String aFileName, int aLineLength)
    throws OhError {
    super.init(aFileName, aLineLength);
  }

  
  /**
   * Initialization.
   *
   * @param aFileName a <code>String</code> value
   * @return a <code>boolean</code> value
   */
  public void init(String aFileName) throws OhError {
    super.init(aFileName, 128);
  }

  // @end{initialization} --------------------------------------
  

  // @begin{constructors} --------------------------------------

  /**
   * Creates a new default <code>OhOutputFileFloat</code> instance.
   *
   */
  public OhOutputFileFloat() {
    super();
    _className = "OhOutputFileFloat";
  }
  
    
  /**
   * Creates a new <code>OhOutputFileFloat</code> instance.
   *
   * @param aFileName a <code>String</code> value
   * @param aLineLength an <code>int</code> value
   */
  public OhOutputFileFloat(String aFileName, int aLineLength) {
    super(aFileName, aLineLength);
    _className = "OhOutputFileFloat";
  }


  /**
   * Creates a new <code>OhOutputFileFloat</code> instance.
   *
   * @param aFileName a <code>String</code> value
   */
  public OhOutputFileFloat(String aFileName) {
    super(aFileName, 128);
    _className = "OhOutputFileFloat";
  }

  // @end{constructors} ----------------------------------------

  
  // @begin{methods} -------------------------------------------

  // 1. Connection function.
  
  /**
   * Returns the datapack type, "float", which expresses the
   * type of data to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "float";
  }


  // 2. The write-in function of output files.

  /**
   * Nothing is treated.
   *
   * @param aStr a <code>String</code> value
   * @param aDataPack an <code>Object</code> value
   */
  public void makeStringData(String aStr, Object aDataPack) {}


  /**
   * Though this method is defined in the base class
   * (Ref "OhOutputFile"),
   * it is redifined in the class
   * (Ref "OhOutputFileFloat").
   *
   * <japanese>
   * <p>
   * o̓f[^̏ <p>
   *
   * ܂Ao̓t@C _objectName ǉ[hŃI[vA
   * I[vłȂꍇ return ܂B<p>
   * 
   * ɁAMf[^pbNz _dataPackArray ̐擪Ɋi[
   * Ăf[^pbN珇Ԃ
   * (Ref "OhDataPackFloat#dpack2Ofile(PrintWriter)")
   * pďo̓t@CɋL܂B<p>
   * 
   *  <p>
   *
   * Mf[^pbN͂̃\bh̒Ŏ擾܂B
   * f OhOutputFile ͎M[q̔hfłāA
   * f[^pbNXgbN@\͂܂B
   * 
   * </japanese>
   *
   * @exception IOException if an error occurs
   */
  public void outputData() throws IOException {
    
    OhArrayList   dpArray      = new OhArrayList();
    OhArrayList[] dpArrayArray = {dpArray};         //; For work.

    int   numOfDps      = 0;
    int[] numOfDpsArray = {numOfDps};  //; For work.

    getDataPackArray(dpArrayArray, numOfDpsArray);

    dpArray  = dpArrayArray[0];
    numOfDps = numOfDpsArray[0];

    
    // _dataPackArray is not updated.
    //
    if (((OhDataPack)dpArray.get(numOfDps - 1)).
      getDataTime().compareTo(_lastDataTime) <= 0)
      return;    


    /* first time output */

    if (_lastDataTime == OhTimePoint.OhTimePointMin)
      writeHeader();

    ////////////////////////////////////////////////
    // Additional writing.
    //
    // <japanese>
    // ݗpo̓t@C̒ǉI[vB
    // </japanese>
    //
    BufferedWriter bw = null;
    
    try {
      bw = new BufferedWriter(new FileWriter(_objectName, true));
    }
    catch(IOException e) {
      throw e;
    }


    PrintWriter pw = new PrintWriter(bw);  

    if (((OhDataPack)dpArray.get(0)).getDataTime().
      compareTo(OhTimePoint.OhTimePointMin) == 0) {
      ((OhDataPack)dpArray.get(0)).dpack2Ofile(pw);
      pw.print(NEWLINE);
    }


    int k = 0;

    // Skip the data which have been output.
    //
    for (k = 0; k < numOfDps; k++) {
      if (((OhDataPack)dpArray.get(k)).getDataTime().
        compareTo(_lastDataTime) > 0)
        break;
    }

    
    /* output the data */

    for (; k < numOfDps; k++) {
      ((OhDataPack)dpArray.get(k)).dpack2Ofile(pw);
      pw.print(NEWLINE);
    }

    pw.close();

    _lastDataTime = ((OhDataPack)dpArray.get(numOfDps - 1)).
      getDataTime();

    reduceDataPackStock();
  }

  // @end{methods} ---------------------------------------------

} // OhOutputFileFloat
