// @(#) OhPortTest.java

package ohymosj;

import junit.framework.*;
import java.io.*;
import java.util.regex.*;

public class OhPortTest extends TestCase
  implements OhConst
{
  OhPort _a;
  
  public OhPortTest(String name) {
    super(name);
  }
  
  public void setUp() {
  }
  
  public void tearDown() {
  }

  // @begin{constructors check} ----------------------------------

  class MyPort extends OhPort {

    // @begin{abstract methods} --------------------------------

    public String dataType() {
      return "Foo"; 
    }
    public boolean areYouReady() {
      return true;
    }

    public void setDataPackArray() {}
    public boolean canYouGetData(
      OhTimePointImmutable aTFrom, OhTimePointImmutable aTTo) {
      return true;
    }
    public void getDataPackArray(
      OhArrayList[] aDpArray, int[] aNumOfDp) {}
    public OhTimePointImmutable getOldestDataTime() {
      return new OhTimePoint();
    }
    public OhTimePointImmutable getNewestDataTime() {
      return new OhTimePoint();
    }
    public int getNumberOfDataPacks() {
      return 0;
    }
    public boolean getDataPack(
      Object[] aFormer, OhTimePoint _dataTime,
      Object[] aLater, char aMode)
      {
        return true;
      }
    public OhTimePointImmutable getNecessaryTimeTo() {
      return new OhTimePoint();
    }
    public OhTimePointImmutable getNecessaryTimeFrom() {
      return new OhTimePoint();
    }
    public void reduceDataPackStock() {}

    // @end{abstract methods} ----------------------------------

    // @begin{constructors} ------------------------------------

    public MyPort() { _className = "MyPort"; }
    public MyPort(String s, int n) {
      super(s, n);
      _className = "MyPort";
    }
    
    // @end{constructors} --------------------------------------

  }

  public void test_constructors() {
    MyPort a = new MyPort();
    assertEquals("test_constructorsA", (Object)"MyPort",
      a.getClassName());

    MyPort b = new MyPort("myport", 1);
    assertEquals("test_constructorsB", "myport",
      b.getObjectName());
    assertEquals("test_constructorsB", 1,
      b.getObjectNumber());
  }
  
 
  // @end{constructors check} ------------------------------------

  // @begin{methods check} ---------------------------------------

  public void test_init() {
    MyPort a = new MyPort();
    try {
      a.init("MyPort", 1);
    }
    catch (OhError e) {
    }
    assertEquals("test_init", "MyPort", a.getObjectName());
  }

  public void test_registerReceiver() {
    MyPort a = new MyPort();
    try {
      OhObject obj = new OhObject("foo", 1);
      a.registerReceiver(obj);
    }
    catch (OhError e) {
      String str =
        "This Port cannot register receiver. " + NEWLINE +
        " > (" + NEWLINE +
        " >  MyPort null -1" + NEWLINE +
        " > )" + NEWLINE +
        " >   receiver > (" + NEWLINE +
        " >   receiver >  OhObject foo 1" + NEWLINE +
        " >   receiver > )" + NEWLINE;

      assertEquals("test_registerReceiver",
        str, e.what());
    }
  }
  
  public void test_doSwitch() {
    MyPort a = new MyPort();
    try {
      OhObject obj = new OhObject("foo", 1);
      a.doSwitch();
    }
    catch (OhError e) {
      String str =
        "This Port cannot switch. " + NEWLINE +
        " > (" + NEWLINE +
        " >  MyPort null -1" + NEWLINE +
        " > )" + NEWLINE;

      assertEquals("test_doSwitch",
        (Object) str, e.what());
    }
  }

  public void test_canYouGetData() {
    MyPort a = new MyPort();
    OhTimePoint t = new OhTimePoint();
    boolean bool = a.canYouGetData(t, t);
    assertEquals("test_canYouGetData", true, bool);
  }

  public void test_getDataPack() {
    MyPort a = new MyPort();
    Object[] oa = new Object[1];
    OhTimePoint tp = new OhTimePoint();
    boolean bool = a.getDataPack(oa, tp);
    assertEquals("test_getDataPackF", true, bool);

    bool = a.getDataPack(tp, oa);
    assertEquals("test_getDataPackL", true, bool);
  }

  public void test_cPrint() {
                               
    MyPort a = new MyPort();

    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    // PrintWriter out = new PrintWriter(System.out);
    a.cPrint(out, "> ");
    out.flush();
    String str = buffer.toString();
    assertEquals("test_cPrint",
      (Object)("> MyPort null -1" + NEWLINE), str);
  }

  public void test_printConnection() {
    MyPort a = new MyPort();

    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);

    a.printConnection(out, "> ");
    out.flush();
    String str = buffer.toString();
    String expected =
      "> (" + NEWLINE +
      ">  MyPort null -1" + NEWLINE +
      "> )" + NEWLINE +
      ">     This port is not connected." + NEWLINE;
    
    assertEquals("test_printConnection",
      (Object)expected, str);
  }

  public void test_setSender() {
    MyPort a = new MyPort();
    MyPort b = new MyPort();
    try {
      a.setSender(b);
    }
    catch (OhError e) {
      boolean match = Pattern.matches(
        "(?s)^This Port cannot set sender\\..*", e.what());
      if (!match) {
        fail("OhError message does not match with one expteced.");
      }
    }
  }

  public void test_calReq() {}

  // @end{methods check} -----------------------------------------
}

