/**
 * @(#) Class to express a port which receives datapacks of type int.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * OhReceiveInt;
 *
 * OhReceiveInt is the base class to express a port which
 * receive datapacks of type int.
 *
 * <japanese>
 * <p>
 * int ^̃f[^pbNMM[q̃NX <p>
 *
 * (Ref "OhReceivePort") pč쐬M[q̃NXłB
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.4 $ $Date: 2004/01/17 13:26:25 $
 */
public class OhReceiveInt extends OhReceivePort
{
  // @begin{initialization} ------------------------------------

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void init(String aPortName, int aPortNum) throws OhError {
    super.init(aPortName, aPortNum);
  }

  // @end{initialization} --------------------------------------

  // @begin {constructors} -------------------------------------

  /**
   * Creates a new <code>OhReceiveInt</code> instance.
   *
   */
  public OhReceiveInt() {
    super();
    _className = "OhReceiveInt";
  }

  /**
   * Creates a new <code>OhReceiveInt</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNumber an <code>int</code> value
   */
  public OhReceiveInt(String aPortName, int aPortNumber) throws OhError {
    super(aPortName, aPortNumber);
    _className = "OhReceiveInt";
  }

  // @end {constructors} ---------------------------------------

  // @begin {methods} ------------------------------------------

  /**
   * Copy method.
   *
   * @param aRd an <code>OhReceiveInt</code> value
   */
  public void copy(OhReceiveInt aRd) {
    _className    = aRd._className;
    _objectName   = aRd._objectName;
    _objectNumber = aRd._objectNumber;
    _owner        = aRd._owner;

    _sender = aRd._sender;
  }
  
  /**
   * Returns the data pack type which expresses the type of data
   * to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {    //; (*5)
    return "int";
  }

  /**
   * Returns data value at time t.
   * This data connection type is brokenline or step or pulse.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>int</code> value
   */
  public int getData(OhTimePoint aT, Vector aDctVec) throws OhError {
    int dataValue = 0;
    
    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray}; // Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 0) {
      throw new OhError("Error in Get_Data",
        oPrintString(" >"));
    }


    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackInt)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackInt)dataPackArray.
               get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) <= 0) {    // aT <= ta ?
      return ((OhDataPackInt)dataPackArray.get(0)).getDataValue();
    }

    if (aT.compareTo(tb) >= 0) {    // aT >= tb ?
      return ((OhDataPackInt)dataPackArray.
        get(numberOfDataPacks - 1)).getDataValue();
    }

    DataConnectionType dataConnectionType =
      (DataConnectionType)aDctVec.elementAt(0);

    if(dataConnectionType == DataConnectionType.BROKENLINE)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = ((OhDataPackInt)dataPackArray.get(0)).
          getDataValue();
      }
      else if (aT.compareTo(tb) <= 0) {
        dataValue = ((OhDataPackInt)dataPackArray.
          get(numberOfDataPacks - 1)).getDataValue();
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackInt)dataPackArray.get(ip1)).getDataTime() );

        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackInt)dataPackArray.get(ip2)).getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;

          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackInt)dataPackArray.get(ip)).getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);  //; t1 = tp;  is NG.
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);  //; t2 = tp;  is NG.
          }
          else {
            return ((OhDataPackInt)dataPackArray.get(ip)).getDataValue();
          }
        }

        int y1 = ((OhDataPackInt)dataPackArray.get(ip1)).getDataValue();
        
        if (ip1 == ip2) {
          return y1;
        }

        int y2 = ((OhDataPackInt)dataPackArray.get(ip2)).getDataValue();

        OhTimePoint aTBackUp = new OhTimePoint();
        aTBackUp.copy(aT);
        //;     v = ((aT-t1)*y2                  + (t2-aT)*y1)                  / (t2-t1);
        dataValue = (int)
          ( ( OhTimePoint.sub(aTBackUp, t1).toDouble()*y2 +
              OhTimePoint.sub(t2, aTBackUp).toDouble()*y1
            ) / OhTimePoint.sub(t2, t1).toDouble() );
      }
    }
    else if (dataConnectionType == DataConnectionType.STEP)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = ((OhDataPackInt)dataPackArray.get(0)).
          getDataValue();
      }
      else if (aT.compareTo(tb) <= 0) {
        dataValue = ((OhDataPackInt)dataPackArray.
          get(numberOfDataPacks - 1)).getDataValue();
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackInt)dataPackArray.get(ip1)).
          getDataTime() );

        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackInt)dataPackArray.get(ip2)).
          getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;

          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackInt)dataPackArray.get(ip)).
            getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);  //; t1 = tp;  is NG.
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);  //; t2 = tp;  is NG.
          }
          else {
            return ((OhDataPackInt)dataPackArray.get(ip)).
              getDataValue();
          }
        }
        dataValue = ((OhDataPackInt)dataPackArray.get(ip2)).
          getDataValue();
      }
    }
    else if (dataConnectionType == DataConnectionType.PULSE)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = 0;
      }
      else if (aT.compareTo(tb) <= 0) {
        dataValue = 0;
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackInt)dataPackArray.get(ip1)).
          getDataTime() );

        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackInt)dataPackArray.get(ip2)).
          getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;

          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackInt)dataPackArray.get(ip)).
            getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);  //; t1 = tp;  is NG.
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);  //; t2 = tp;  is NG.
          }
          else {
            return ((OhDataPackInt)dataPackArray.get(ip)).
              getDataValue();
          }
        }
        if (aT.compareTo(t1) == 0) {
          dataValue = ((OhDataPackInt)dataPackArray.get(ip1)).
            getDataValue();
        }
        else if (aT.compareTo(t2) == 0) {
          dataValue = ((OhDataPackInt)dataPackArray.get(ip2)).
            getDataValue();
        }
        else {
          dataValue = 0;
        }
      }
    }
    else {
      throw new OhError("Illegal data_connection_type.",
        oPrintString("> "));
    }
    
    return dataValue;
    
  }

  /**
   * Returns the data value at time t.
   * This data connection type is brokenline.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @return a <code>int</code> value
   */
  public int getData(OhTimePoint aT) throws OhError {    //; (*1-2)
    int dataValue = 0;
    
    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    
    if(numberOfDataPacks <= 0) {
      throw new OhError("Error in Get_Data",
        oPrintString(" >"));
    }


    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackInt)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackInt)dataPackArray.
               get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) <= 0) {    // aT <= ta ?
      return ((OhDataPackInt)dataPackArray.get(0)).getDataValue();
    }

    if (aT.compareTo(tb) >= 0) {    // aT >= tb ?
      return ((OhDataPackInt)dataPackArray.
        get(numberOfDataPacks - 1)).getDataValue();
    }

    int ip1 = 0;
    int ip2 = numberOfDataPacks - 1;

    OhTimePoint t1 = new OhTimePoint();
    t1.copy( ((OhDataPackInt)dataPackArray.
               get(ip1)).getDataTime() );

    OhTimePoint t2 = new OhTimePoint();
    t2.copy( ((OhDataPackInt)dataPackArray.
               get(ip2)).getDataTime() );
    
    while (ip2 - ip1 > 1) {
      ip = (ip2 + ip1)/2;

      OhTimePoint tp = new OhTimePoint();
      tp.copy( ((OhDataPackInt)dataPackArray.
                 get(ip)).getDataTime() );

      if (tp.compareTo(aT) < 0) {
        ip1 = ip;
        t1.copy(tp);  //; t1 = tp;  is NG.
      }
      else if (tp.compareTo(aT) > 0) {
        ip2 = ip;
        t2.copy(tp);  //; t2 = tp;  is NG.
      }
      else {
        return ((OhDataPackInt)dataPackArray.get(ip)).
          getDataValue();
      }
    }

    int y1 = ((OhDataPackInt)dataPackArray.get(ip1)).
      getDataValue();
    
    if (ip1 == ip2) {
      return y1;
    }

    int y2 = ((OhDataPackInt)dataPackArray.get(ip2)).
      getDataValue();

    OhTimePoint aTBackUp = new OhTimePoint();
    aTBackUp.copy(aT);
    //;     v = ((aT-t1)*y2                  + (t2-aT)*y1)                  / (t2-t1);
    dataValue = (int) (
      ((OhTimePoint.sub(aTBackUp, t1).toDouble()*y2 +
        OhTimePoint.sub(t2, aTBackUp).toDouble()*y1))
      / OhTimePoint.sub(t2, t1).toDouble() );
    
    return dataValue;
  }


  /**
   * Set the data value at time t to reference aDataValue[0].
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>int[]</code> value
   * @param aDctVec a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public void setData(OhTimePoint aT, int[] aDataValue, Vector aDctVec)
    throws OhError {  //; (*4-1)
    aDataValue[0] = getData(aT, aDctVec);
  }


  /**
   * Set the data value at time t to reference aDataValue[0].
   * This data connection type is brokenline.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>int[]</code> value
   * @exception OhError if an error occurs
   */
  public void setData(OhTimePoint aT, int[] aDataValue)
    throws OhError {  //; (*4-2)
    aDataValue[0] = getData(aT);
  }


  /**
   * Returns the integrated value from time aT1 till time aT2.
   * aT1 should be less than or equal to t2. <p>
   *
   * When data connection type is Pulse, the value summed from
   * aT1 till aT2 will be returned. The pulse at time aT1 is not
   * contained in the sum, and the pulse at time aT2 is
   * contained in the sum.
   *
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>int</code> value
   */
  public int getDataSum(OhTimePoint aT1, OhTimePoint aT2, Vector aDctVec)
    throws OhError {    //; (*3-1)
    int sum = 0;

    if(aT1.compareTo(aT2) == 0) {
      return sum;
    }

    if(aT1.compareTo(aT2) > 0) {
      OhTimePoint aT3 = aT1;
      aT1 = aT2;
      aT2 = aT3;
    }

    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 1) {
      throw new OhError("Error in Get_Data", oPrintString(" >"));
    }


    OhTimePointImmutable ta =
      ((OhDataPackInt)dataPackArray.get(0)).getDataTime();
    OhTimePointImmutable tb =
      ((OhDataPackInt)dataPackArray.get(numberOfDataPacks - 1)).
      getDataTime();

    if (aT1.compareTo(ta) < 0 || aT2.compareTo(tb) > 0) {
      throw new IllegalArgumentException("Illegal aT1 or aT2.");
    }

    DataConnectionType dataConnectionType =
      (DataConnectionType)aDctVec.elementAt(0);

    if(dataConnectionType == DataConnectionType.BROKENLINE) {
      int y  = 0;
      int ya = 0;
      int yb = 0;
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePointImmutable tp =
          ((OhDataPackInt)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip;
          ipa = ip - 1;
          tb  = tp;

          ta = ((OhDataPackInt)dataPackArray.get(ipa)).
            getDataTime();
          break;
        }
      }

      ya = ((OhDataPackInt)dataPackArray.get(ipa)).getDataValue();
      yb = ((OhDataPackInt)dataPackArray.get(ipb)).getDataValue();

      if(ipa == ipb) {
        y = ya;
      }
      else {
        y = (int)( (
                     OhTimePoint.sub(aT1, ta).toDouble()*yb +
                     OhTimePoint.sub(tb, aT1).toDouble()*ya
                   ) / OhTimePoint.sub(tb, ta).toDouble() );
      }

      OhTimePointImmutable t  = aT1;
      ip = ipb;
      OhTimePointImmutable ts = tb;
      int ys = yb;

      while(ts.compareTo(aT2) < 0) {
        sum += (y + ys) * OhTimePoint.sub(ts, t).toDouble() / 2;
        t = ts;
        y = ys;
        ip++;

        ts = ((OhDataPackInt)dataPackArray.get(ip)).getDataTime();
        ys = ((OhDataPackInt)dataPackArray.get(ip)).getDataValue();
      }

      ys = (int)( (
                    y*OhTimePoint.sub(ts, aT2).toDouble() +
                    ys*OhTimePoint.sub(aT2, t).toDouble()
                   ) / OhTimePoint.sub(ts, t).toDouble() );
      sum += (y+ys) * OhTimePoint.sub(aT2, t).toDouble() / 2;      
    }
    else if(dataConnectionType == DataConnectionType.STEP) {
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePointImmutable tp =
          ((OhDataPackInt)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip;
          ipa = ip - 1;
          tb = tp;
          break;
        }
      }

      OhTimePointImmutable tn = tb;
      ip = ipb;

      int yn = ((OhDataPackInt)dataPackArray.get(ip)).getDataValue();
      OhTimePointImmutable ts = aT1;

      while(tn.compareTo(aT2) < 0) {
        sum += yn * OhTimePoint.sub(tn, ts).toDouble();
        ts = tn;
        ip++;

        tn = ((OhDataPackInt)dataPackArray.get(ip)).getDataTime();
        yn = ((OhDataPackInt)dataPackArray.get(ip)).getDataValue();
      }

      if(aT2.compareTo(tn) <= 0) {
        sum += yn * OhTimePoint.sub(aT2, ts).toDouble();
      }
    }
    else if(dataConnectionType == DataConnectionType.PULSE) {
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePointImmutable tp =
          ((OhDataPackInt)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip + 1;
          break;
        }
      }

      OhTimePointImmutable tn = new OhTimePoint();
      int yn = 0;

      for(; ip < numberOfDataPacks; ip++) {
        tn = ((OhDataPackInt)dataPackArray.get(ip)).getDataTime();
        yn = ((OhDataPackInt)dataPackArray.get(ip)).getDataValue();
        
        if(tn.compareTo(aT2) > 0)
          break;

        sum += yn;
      }
    }
    else {
      throw new IllegalArgumentException("Illegal data_connection_type.");
    }
    
    return sum;
  }


  /**
   * Gets sum data whose data connection type is brokenline.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @return a <code>int</code> value
   */
  public int getDataSum(OhTimePoint aT1, OhTimePoint aT2)
    throws OhError {    //; (*3-2)
    int sum = 0;

    if(aT1.compareTo(aT2) == 0) {
      return sum;
    }

    if(aT1.compareTo(aT2) > 0) {
      OhTimePoint aT3 = aT1;
      aT1 = aT2;
      aT2 = aT3;
    }


    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 1) {
      throw new OhError("Error in Get_Data", oPrintString(" >"));
    }

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackInt)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackInt)dataPackArray.
               get(numberOfDataPacks - 1)).getDataTime() );

    if (aT1.compareTo(ta) < 0 || aT2.compareTo(tb) > 0) {
      throw new IllegalArgumentException("Illegal aT1 or aT2.");
    }

    int y  = 0;
    int ya = 0;
    int yb = 0;
    int ipa = 0;
    int ipb = numberOfDataPacks - 1;
    
    while(ipb - ipa > 1) {
      ip = (ipa + ipb)/2;

      OhTimePoint tp = new OhTimePoint();
      tp.copy( ((OhDataPackInt)dataPackArray.get(ip)).
        getDataTime() );
      
      if(tp.compareTo(aT1) < 0) {
        ipa = ip;
        ta.copy(tp);  //; ta = tp;  is NG.
      }
      else if(tp.compareTo(aT1) > 0) {
        ipb = ip;
        tb.copy(tp);  //; tb = tp;  is NG.
      }
      else {
        ipb = ip;
        ipa = ip - 1;
        tb.copy(tp);  //; tb = tp;  is NG.

        ta.copy( ((OhDataPackInt)dataPackArray.get(ipa)).
          getDataTime() );

        break;
      }
    }

    ya = ((OhDataPackInt)dataPackArray.get(ipa)).getDataValue();
    yb = ((OhDataPackInt)dataPackArray.get(ipb)).getDataValue();

    if(ipa == ipb) {
      y = ya;
    }
    else {
      y = (int)( (
                   OhTimePoint.sub(aT1, ta).toDouble()*yb +
                   OhTimePoint.sub(tb, aT1).toDouble()*ya
                  ) / OhTimePoint.sub(tb, ta).toDouble() ) ;
    }

    
    OhTimePoint t = new OhTimePoint();
    t.copy(aT1);

    ip = ipb;

    OhTimePoint ts = new OhTimePoint();
    ts.copy(tb);

    int ys = yb;
    
    while(ts.compareTo(aT2) < 0) {
      sum += (y + ys) * OhTimePoint.sub(ts, t).toDouble() / 2;

      t.copy(ts);  //; t = ts;  is NG.

      y = ys;
      ip++;

      ts.copy( ((OhDataPackInt)dataPackArray.
                 get(ip)).getDataTime() );
      ys = ((OhDataPackInt)dataPackArray.get(ip)).getDataValue();
    }

    ys = (int)( (y*OhTimePoint.sub(ts, aT2).toDouble() +
                 ys*OhTimePoint.sub(aT2, t).toDouble()
                ) / OhTimePoint.sub(ts, t).toDouble() );
    sum += (y+ys) * OhTimePoint.sub(aT2, t).toDouble() / 2;      

    return sum;
  }
  

  /**
   * Gets pulse data.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>int[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetPulseData(OhTimePoint aT, int[] aDataValue) {

    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};
      // Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];


    if (numberOfDataPacks <= 0)
      return false;


    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackInt)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackInt)dataPackArray.
               get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) < 0 || aT.compareTo(tb) > 0)
      return false;

    OhTimePoint tn = new OhTimePoint();
    OhTimePoint tp = new OhTimePoint();
    int yn = 0;
    int yp = 0;

    tn.copy(ta);  //; tn = ta;  is NG.
    yn = ((OhDataPackInt)dataPackArray.get(0)).getDataValue();

    if (aT.compareTo(tn) == 0) {
      aDataValue[0] = yn;
      return true;
    }

    boolean hit = false;

    for (ip = 1; ip < numberOfDataPacks; ip++) {
      tp.copy(tn);  //; tp = tn;  is NG.
      yp = yn;

      tn.copy( ((OhDataPackInt)dataPackArray.get(ip)).getDataTime() );
      yn = ((OhDataPackInt)dataPackArray.get(ip)).getDataValue();
      
      if (tn.compareTo(aT) > 0) {
        hit = false;
        break;
      }
      else if (tn.compareTo(aT) == 0){
        aDataValue[0] = yn;
        hit = true;
        break;
      }
    }
    return hit;
  }


  /**
   * Gets mean data.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>int</code> value
   * @exception OhError if an error occurs
   */
  public int getMeanData(OhTimePoint aT1, OhTimePoint aT2, Vector aDctVec)
    throws OhError {
    if(aT2.compareTo(aT1) == 0) {
      return getData(aT2, aDctVec);
    }
    else {
      return (int)( getDataSum(aT1, aT2, aDctVec)
        / OhTimePoint.sub(aT2, aT1).toDouble() );
    }
  }



  
  /**
   * Gets mean data whose data connection type is brokenline.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @return a <code>int</code> value
   */
  public int getMeanData(OhTimePoint aT1, OhTimePoint aT2)
    throws OhError {
    if(aT2.compareTo(aT1) == 0) {
      return getData(aT2);
    }
    else {
      return (int)( getDataSum(aT1, aT2) /
        OhTimePoint.sub(aT2, aT1).toDouble() );
    }
  }

  // @end {methods} --------------------------------------------

} // OhReceiveInt
