/**
 * @(#) OhSendFloat
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * The OhSendFloat class is the base class to express a port
 * which send float data.
 *
 * <japanese>
 * <p>
 * float ^̃f[^pbN𑗐M鑗M[q̃NX <p>
 *
 * (Ref "OhReceivePort") pč쐬M[q̃NX
 * B
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.1 $ $Date: 2004/01/17 13:15:23 $
 */
public class OhSendFloat extends OhSendPort
{
  // @begin{initialization} ------------------------------------

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public void init(String aPortName, int aPortNum) throws OhError {
    super.init(aPortName, aPortNum);
  }

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   */
  public void init(String aPortName, int aPortNum,
    Vector aDctVec) throws OhError {
    super.init(aPortName, aPortNum, aDctVec);
  }

  // @end{initialization} --------------------------------------

  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>OhSendFloat</code> instance.
   *
   * @exception OhError if an error occurs
   */
  public OhSendFloat() throws OhError {  //; 2003.11.18
    super();
    _className = "OhSendFloat";
  }


  /**
   * Creates a new <code>OhSendFloat</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSendFloat(String aPortName, int aPortNum) {
    super(aPortName, aPortNum);
    _className = "OhSendFloat";
  }

  
  /**
   * Creates a new <code>OhSendFloat</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public OhSendFloat(String aPortName, int aPortNum,
    Vector aDctVec) {
    super(aPortName, aPortNum, aDctVec);
    _className = "OhSendFloat";
  }

  // @end {constructors} ---------------------------------------


  // @begin {methods} ------------------------------------------

  // 1. function for sending data.
  
  /**
   * Returns the data pack type which expresses the type of data
   * to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "float";
  }


  /**
   * Send data whose data time is <b>aT</b> and data value is
   * <b>aValue</b>.
   *
   * <japanese>
   * <p>
   * ɂƂ aT, aValue 
   * (Ref "OhDataPackFloat") ^f[^pbN쐬A
   * (Ref "OhSendPort#dataPackStock(Object)") p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aValue a <code>float</code> value
   */
  public void sendData(OhTimePoint aT, float aValue) {    //; (*1)
    OhDataPackFloat dpd = new OhDataPackFloat();
    OhTimePoint tp = new OhTimePoint();  //; For work.
    tp.copy(aT);
    dpd.setData(tp, aValue);
    dataPackStock(dpd);
  }


  /**
   * Send data of type float whose value is stored in
   * <b>aDataStr</b>.
   *
   * <japanese>
   * <p>
   * ɂƂ aDataStr ŕ\ꂽf[^pbN
   * (Ref "OhDataPackFloat") ^f[^pbN쐬A
   * (Ref "OhSendPort#dataPackStock(Object)") p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public void sendByString(String aDataStr) {
    OhDataPackFloat dp = new OhDataPackFloat(aDataStr);
    dataPackStock(dp);
  }
    
  // @end {methods} --------------------------------------------

} // OhSendFloat
