/**
 * @(#) OhSendInt
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * The OhSendInt class is the base class to express a port which
 * send datapacks of type int.
 *
 * <japanese>
 * <p>
 * int ^̃f[^pbN𑗐M鑗M[q̃NX <p>
 *
 * OhReceivePort pč쐬M[q̃NXłB
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.1 $ $Date: 2004/01/17 13:13:31 $
 */
public class OhSendInt extends OhSendPort
{
  // @begin{initialization} ------------------------------------
  
  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public void init(String aPortName, int aPortNum) throws OhError {
    super.init(aPortName, aPortNum);
  }
  
  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>boolean</code> value
   */
  public void init(String aPortName, int aPortNum, Vector aDctVec)
    throws OhError {
    super.init(aPortName, aPortNum, aDctVec);
  }

  // @end{initialization} --------------------------------------

  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>OhSendInt</code> instance.
   *
   * @exception OhError if an error occurs
   */
  public OhSendInt() {
    super();
    _className = "OhSendInt";
  }
    
  /**
   * Creates a new <code>OhSendInt</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSendInt(String aPortName, int aPortNum) {
    super(aPortName, aPortNum);
    _className = "OhSendInt";
  }

  
  /**
   * Creates a new <code>OhSendInt</code> instance.
   *
   * <japanese>
   * OԖڂ̈́Af[^Ԍ^ (Ref "DataConnectionType") 
   * ϐ^ Vector łBƂ΁A
   *
   * <pre>
   *   Vector dctvec = new Vector();
   *   dctvec.add(DataConnectionType.STEP);
   * </pre>
   *
   * Ƃ΁Af[^Ԍ^̕ϐ1^ Vector 𐶐
   * Ƃł܂B
   * </japanese>
   * 
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   */
  public OhSendInt(String aPortName, int aPortNum, Vector aDctVec) {
    super(aPortName, aPortNum, aDctVec);
    _className = "OhSendInt";
  }

  // @end {constructors} ---------------------------------------


  // @begin {methods} ------------------------------------------

  // 1. function for sending data.
  
  /**
   * Returns the data pack type which expresses the type of data
   * to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "int";
  }
  
  /**
   * Send data whose data time is "aT" and data value is "aValue".
   *
   * <japanese>
   * <p>
   * ɂƂ aT, aValue 
   * OhDataPackInt ^f[^pbN쐬A
   * OhSendPort.dataPackStock(Object) p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aValue a <code>int</code> value
   */
  public void sendData(OhTimePoint aT, int aValue) {
    dataPackStock(new OhDataPackInt(aT, aValue));
  }
    
  /**
   * Send datapacks of type int whose value is stored in
   * <b>aDataStr</b>.
   *
   * <japanese>
   * <p>
   * ɂƂ aDataStr ŕ\ꂽf[^pbN
   * (Ref "OhDataPackInt") ^f[^pbN쐬A
   * (Ref "OhSendPort.dataPackStock(Object)") p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public void sendByString(String aDataStr) {
    dataPackStock(new OhDataPackInt(aDataStr));
  }
    
  // @end {methods} --------------------------------------------

} // OhSendInt
