// @(#) OhSendPortTest.java

package ohymosj;

import junit.framework.*;
import java.util.*;
import java.io.*;
import java.util.regex.*;

public class OhSendPortTest extends TestCase
  implements OhConst
{
  public OhSendPortTest(String name) {
    super(name);
  }
  
  public void setUp() {
  }
  
  public void tearDown() {
  }

  class OhSendPortMoc extends OhSendPort {
    public String dataType() { return "double"; }
    public void sendByString(String aDstr) {}
    public OhTimePointImmutable getNecessaryTimeFrom() {
      return new OhTimePoint(2003, 25, 0.0);
    }
  }

  // @begin{constructors check} ----------------------------------
  // @end{constructors check} ------------------------------------

  // @begin{methods check} ---------------------------------------

  public void test_setDctVec() {
    OhSendPort sp = new OhSendPortMoc();
    Vector dctvec = new Vector();
    dctvec.add(DataConnectionType.STEP);
    sp.setDctVec(dctvec);
    Vector anotherDctvec = sp.getDctVec();
    assertEquals("test_setDctVecA",
      dctvec.size(), anotherDctvec.size());
    assertEquals("test_setDctVecB",
      dctvec.get(0), anotherDctvec.get(0));
  }

  /**
   * This method is tested in test_setDctVec.
   */
  public void test_getDctVec() {}

  public void test_doSwitch() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
      System.out.println(e.what());
    }
    assertEquals("test_doSwitchA",
      0, sp._numberOfDataPacks);

    String str = "";
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
      str = e.what();
    }
    boolean match = Pattern.matches(
        "(?s).*has already been set.*", str);
    if (!match) {
      fail("OhError message does not match with one expteced.");
    }
  }

  public void test_calReq() {
    //; omitted
  }

  public void test_dataPackStock() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    
    OhDataPackDouble dp = new OhDataPackDouble(
      new OhTimePoint(), 1.5);
    sp.dataPackStock(dp);
    sp.setDataPackArray();
    assertEquals("test_dataPackStock",
      1, sp.getNumberOfDataPacks());
  }

  public void test_getDataPackArray0() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    
    OhDataPackDouble dp = new OhDataPackDouble(
      new OhTimePoint(2003, 0, 0.0), 1.5);
    sp.dataPackStock(dp);
    sp.setDataPackArray();
    OhArrayList list = sp.getDataPackArray();
    assertEquals("test_getDatapackArray0",
      true, ((OhDataPackDouble) list.get(0)).equals(dp));
  }
    
  public void test_getDataPackArray1() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhDataPackDouble dp = new OhDataPackDouble(
      new OhTimePoint(2003, 0, 0.0), 1.5);
    sp.dataPackStock(dp);
    sp.setDataPackArray();
    OhArrayList[] listPack = {null};
    int[] listSizePack = new int[1];
    sp.getDataPackArray(listPack, listSizePack);
    assertEquals("test_getDatapackArray1",
      true, ((OhDataPackDouble) listPack[0].get(0)).equals(dp));
  }

  public void test_getDataPack() {
    /*
    public boolean getDataPack(
    Object[] aFormer, OhTimePoint _dataTime,
    Object[] aLater, char aMode) {  //; (*6-1)

    public boolean getDataPack(
    Object[] aFormer, OhTimePoint _dataTime,
    Object[] aLater) {  //; (*6-2)
    */
  }

  public void test_getOldestDataTime() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhDataPackDouble dp0 = new OhDataPackDouble(
      new OhTimePoint(2003, 0, 0.0), 1.5);
    sp.dataPackStock(dp0);
    OhDataPackDouble dp1 = new OhDataPackDouble(
      new OhTimePoint(2003, 10, 0.0), 1.5);
    sp.dataPackStock(dp1);
    sp.setDataPackArray();
    OhTimePointImmutable tp = sp.getOldestDataTime();
    assertEquals("test_getOldestDataTime",
      true, dp0.getDataTime().equals(tp));
  }

  public void test_getNewestDataTime() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhDataPackDouble dp0 = new OhDataPackDouble(
      new OhTimePoint(2003, 0, 0.0), 1.5);
    sp.dataPackStock(dp0);
    OhDataPackDouble dp1 = new OhDataPackDouble(
      new OhTimePoint(2003, 10, 0.0), 1.5);
    sp.dataPackStock(dp1);
    sp.setDataPackArray();
    OhTimePointImmutable tp = sp.getNewestDataTime();
    assertEquals("test_getNewestDataTime",
      true, dp1.getDataTime().equals(tp));
  }

  public void test_getNumberOfDatapacks() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhDataPackDouble dp0 = new OhDataPackDouble(
      new OhTimePoint(2003, 0, 0.0), 1.5);
    sp.dataPackStock(dp0);
    OhDataPackDouble dp1 = new OhDataPackDouble(
      new OhTimePoint(2003, 10, 0.0), 1.5);
    sp.dataPackStock(dp1);
    sp.setDataPackArray();
    assertEquals("test_getNumberOfDatapacks",
      2, sp.getNumberOfDataPacks());
  }

  /**
   * public boolean getDataPack(
   *  Object[] aFormer, OhTimePoint aDataTime,
   *  Object[] aLater, char aMode) {  //; (*6-1)
   */
  public void test_getDataPack0() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhDataPackDouble dp0 = new OhDataPackDouble(
      new OhTimePoint(2003, 10, 0.0), 1.5);
    sp.dataPackStock(dp0);
    OhDataPackDouble dp1 = new OhDataPackDouble(
      new OhTimePoint(2003, 20, 0.0), 1.5);
    sp.dataPackStock(dp1);
    sp.setDataPackArray();

    OhTimePoint tpa = new OhTimePoint(2003,  5, 0.0);
    OhTimePoint tpb = new OhTimePoint(2003, 15, 0.0);
    OhTimePoint tpc = new OhTimePoint(2003, 25, 0.0);
    Object[] former = {null};
    Object[] later  = {null};
    // System.out.println(sp.getNumberOfDataPacks());
    boolean f;
    f = sp.getDataPack(former, tpb, later, 'b');
    // System.out.println((OhDataPackDouble) former[0]);
    // System.out.println((OhDataPackDouble) later[0]);
    assertEquals("test_getDataPack0A", true, f);
    assertEquals("test_getDataPack0B", false,
      sp.getDataPack(former, tpa, later, 'f'));
    assertEquals("test_getDataPack0C", false,
      sp.getDataPack(former, tpc, later, 'l'));
  }
  public void test_getDataPack1() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhDataPackDouble dp0 = new OhDataPackDouble(
      new OhTimePoint(2003, 10, 0.0), 1.5);
    sp.dataPackStock(dp0);
    OhDataPackDouble dp1 = new OhDataPackDouble(
      new OhTimePoint(2003, 20, 0.0), 1.5);
    sp.dataPackStock(dp1);
    sp.setDataPackArray();

    OhTimePoint tpa = new OhTimePoint(2003,  5, 0.0);
    OhTimePoint tpb = new OhTimePoint(2003, 15, 0.0);
    OhTimePoint tpc = new OhTimePoint(2003, 25, 0.0);
    Object[] former = {null};
    Object[] later  = {null};
    // System.out.println(sp.getNumberOfDataPacks());
    boolean f;
    f = sp.getDataPack(former, tpb, later);
    // System.out.println((OhDataPackDouble) former[0]);
    // System.out.println((OhDataPackDouble) later[0]);
    assertEquals("test_getDataPack0A", true,
      f);
    assertEquals("test_getDataPack0B", false,
      sp.getDataPack(former, tpa, later));
    assertEquals("test_getDataPack0C", false,
      sp.getDataPack(former, tpc, later));
  }
  
  public void test_reduceDatapackStock() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhTimePoint tp0 = new OhTimePoint(2003, 10, 0.0);
    OhDataPackDouble dp0 = new OhDataPackDouble(tp0, 1.5);
    sp.dataPackStock(dp0);
    OhTimePoint tp1 = new OhTimePoint(2003, 20, 0.0);
    OhDataPackDouble dp1 = new OhDataPackDouble(tp1, 2.5);
    sp.dataPackStock(dp1);
    OhTimePoint tp2 = new OhTimePoint(2003, 30, 0.0);
    OhDataPackDouble dp2 = new OhDataPackDouble(tp2, 3.5);
    sp.dataPackStock(dp2);
    sp.setDataPackArray();
    assertEquals("1", tp0.toString(),
      sp.getOldestDataTime().toString());
    sp.reduceDataPackStock();
    assertEquals("2", tp1.toString(),
      sp.getOldestDataTime().toString());
    assertEquals("3", tp2.toString(),
      sp.getNewestDataTime().toString());
  }
  public void test_forgetDataPackStock() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhTimePoint tp0 = new OhTimePoint(2003, 10, 0.0);
    OhDataPackDouble dp0 = new OhDataPackDouble(tp0, 1.5);
    sp.dataPackStock(dp0);
    OhTimePoint tp1 = new OhTimePoint(2003, 20, 0.0);
    OhDataPackDouble dp1 = new OhDataPackDouble(tp1, 2.5);
    sp.dataPackStock(dp1);
    OhTimePoint tp2 = new OhTimePoint(2003, 30, 0.0);
    OhDataPackDouble dp2 = new OhDataPackDouble(tp2, 3.5);
    sp.dataPackStock(dp2);
    sp.setDataPackArray();
    assertEquals("1", tp0.toString(),
      sp.getOldestDataTime().toString());
    
    sp.forgetDataPackStock(new OhTimePoint(2003, 25, 0.0));
    assertEquals("2", tp1.toString(),
      sp.getOldestDataTime().toString());
    assertEquals("3", tp2.toString(),
      sp.getNewestDataTime().toString());
  }
  
  public void test_clearDataPack() {
    //; omitted
  }

  public void test_setProtectPastDataFlag() {
    //; omitted
  }
  
  public void test_saveSendPortStock() {
    OhSendPort sp = new OhSendPortMoc();
    try {
      sp.doSwitch();
    }
    catch (OhError e) {
    }
    OhTimePoint tp0 = new OhTimePoint(2003, 10, 0.0);
    OhDataPackDouble dp0 = new OhDataPackDouble(tp0, 1.5);
    sp.dataPackStock(dp0);
    OhTimePoint tp1 = new OhTimePoint(2003, 20, 0.0);
    OhDataPackDouble dp1 = new OhDataPackDouble(tp1, 2.5);
    sp.dataPackStock(dp1);
    sp.setDataPackArray();

    StringWriter buffer = new StringWriter();
    PrintWriter out = new PrintWriter(buffer);
    sp.saveSendPortStock(out);
    String result = buffer.toString();
    String expected =
      "(2) # _numberOfDataPacks" + NEWLINE + NEWLINE +
      "(2003)" + NEWLINE +
      "(10)" + NEWLINE +
      "(0.0)" + NEWLINE +
      "(1.5)" + NEWLINE + NEWLINE +
      "(2003)" + NEWLINE +
      "(20)" + NEWLINE +
      "(0.0)" + NEWLINE +
      "(2.5)" + NEWLINE;
    assertEquals("1", (Object) expected, result);
  }

  // @end{methods check} -----------------------------------------
}

