/**
 * @(#) OhSubSystem; base class for creating subsystems.
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * OhSubSystem is a class to express a subsystem which contains
 * a number of elements.
 *
 * <japanese>
 * <p>
 * nNX <p>
 *
 * nf́AvfEnȂǁÃCX^XiȉA
 * \vfA\IuWFNg; componentj ɂ\郂f
 * łB{VXeł́Anf͌ƂĐvZs
 * Ȃ킸Af[^𒼐ڏƂ͓̗vfɂs
 * Ȃ̂Ƃ܂B]āAnf̌vZƂ͍\vf
 * ɏvZ߂𑗂邱ƂɂȂ܂B<p>
 *
 * ƂŁAnf́AMpp[qEMpp[q
 * L܂A̒p[qo^Eڑ葱́Avf
 * f̎M[qEM[q̏ꍇƓ̎葱ƂĒ`ł
 * B<p>
 *
 * ŁA{VXeł͕nfVɍ쐬̂ł͂ȂA
 * vffpč쐬邱ƂƂ܂Bnfł́A
 * Mpp[qEMpp[q(Ref "OhElement")  
 * receivePortArray, (Ref "OhElement")  sendPortArray ɋL
 * Ă܂B܂A[q̓o^Ɋւ (Ref "OhElement") ̃\b
 * h\vf̓o^ƂɂpA[q̓o^ƑS̎葱
 * ō\vf̓o^sȂAnf쐬҂̗֐߂
 * ܂B<p>
 *
 * nfł́AL̂悤ɓo^e\vfEp[q
 * ̐ڑ`Ȃ΂Ȃ܂BA{^nf
 * ɐڑ̂߂̃\bhɒ`Ă̂ŁA{^n
 * fpč쐬nfł́AڑƂ͊{^
 * nf̃\bhɈ^݂̂̍ƂɒPĂ
 * ܂B<p>
 *
 * ܂AvfEnǂڒʐMpĔvZsȂ
 * ߂ɂ́AL镔nǗKv
 * B<p>
 *
 * ŁAnfł͔vẐ߂̃\bh`Ă
 * ܂B
 *
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.84 $ $Date: 2004/01/08 00:09:19 $
 */
public abstract class OhSubSystem extends OhElement
{
  // @begin {instance variable or static variable} -------------

  /**
   * Array for memorizing constituent elements.
   *
   * <japanese>
   * <p>
   * \vfLpz
   * </japanese>
   */
  Object[] _componentArray;

  /**
   * Number of constituent elements.
   *
   * <japanese>
   * <p>
   * \vf̐
   * </japanese>
   *
   */
  int _numberOfComponents;


  /**
   * Array for memorizing the costituent elements which carry
   * out repeated calculation.
   *
   * <japanese>
   * <p>
   * vZsȂ\vfLpz
   * </japanese>
   *
   */
  Object[][] _iteratorArray;

  /**
   * The number of the element groups which carry out repeated
   * calculation.
   *
   * <japanese>
   * <p>
   * vZsȂvfQ̐
   * </japanese>
   *
   */
  int _numberOfIteratorSets;


  /**
   * The number of the elements which constitute an element group.
   *
   * <japanese>
   * <p>
   * ꂼ̗vfQ\vf̐
   * </japanese>
   */
  int[] _numberOfIterators;


  // @end {instance variable or static variable} ---------------


  // @begin {abstract methods} ---------------------------------

  // 1. creation and reference of "_receivePortArray",
  // "_sendPortArray" and "_componentArray"

  /**
   * <japanese>
   * <p>
   * Mp[q̓o^ <p>
   *
   * Mp[qo^܂B
   * hNXŒ`܂B
   * (Ref "OhElement#construct()") Ă΂܂B
   * o^ĂȂMp[q͐ڑł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerReceivePorts() throws OhError;

  /**
   * <japanese>
   * <p>
   * Mp[q̓o^ <p>
   *
   * Mp[qo^܂B
   * hNXŒ`܂B
   * construct() Ă΂܂B
   * o^ĂȂMp[q͐ڑł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerSendPorts() throws OhError;

  /**
   * <japanese>
   * <p>
   * vZsȂȂ\IuWFNg _componentArray ɓo
   * ^B<p>
   *
   * S\vf̂AvZsȂȂ\IuWFNg 
   * _componentArray ɓo^܂B<p>
   *
   * o^ĂȂ\IuWFNgɂ͌vZs߂܂
   * B<p>
   *
   * Ⴆ΁AhNXō\IuWFNg abc oϐƂ
   * `ꍇÂ悤 abc o^܂B
   *
   * <pre>
   *  public void registerComponents() {
   *    Register((Object) abc);
   *  }
   * </pre>
   *
   * ̎AK abc  Object ^ɃLXgĉB<p>
   *
   * ɂ abc ́AꎞI _objectList Ɏ^ꂽA
   * ŏII Object ^z _componentArray ɋL܂B
   * iOhSubSystem.construct() QƁj
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerComponents() throws OhError;

  /**
   * <japanese>
   * <p>
   * vZvf _iteratorArray ɓo^B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerIterators() throws OhError;

  
  // 2. connection of constituent elements

  /**
   * Abstract method which is used for connecting ports of
   * elements within this subsystem.  The programmer who
   * provides a new OhSubSystem class or OhTotalSystem must
   * define this method.  He can use various 
   * (Ref "OhSubSystem#connect()") methods to write the
   * connectPort() method.
   *
   * <japanese>
   * <p>
   * nf̒[qڑ邽߂Ɏgp钊ۃ\b
   * hłBV OhSubSystem NX OhTotalSystem NX
   * 񋟂悤ƂvO}́Ã\bh`Ȃ
   * Ȃ܂B̃\bh߂ɁA(Ref "OhSubSystem") 
   * Œ`ĂlX (Ref "OhSubSystem#connect()") \b
   * hgpł܂B<p>
   *
   * : (Ref "OhScfSubSystem#connectPorts()") 
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void connectPorts() throws OhError;

  /**
   * <japanese>
   * <p>
   * nf̍\IuWFNg̐ڑ <p>
   *
   * nf̒ڒʐMڑ
   * (Ref "OhElement#register(Object, int)")
   * pĒ`܂B
   * Ⴆ΁Â悤ɒ`܂B
   *
   * <pre>
   *  public void connectComponents() {
   *    a.register(b, 0);
   *    b.register(a, 0);
   *    a.register(c, 1);
   *    c.register(a, 0);
   *  }
   * </pre>
   *
   * ŁAa, b, c ͂̕ñoϐłB
   * ɂAvf a  _objectArray[0] ɗvf b i[A
   * vf a  _objectArray[1] ɗvf c i[邱ƂɂȂ܂B
   * vf b  _objectArray[0] ɗvf a i[܂B
   * vf c  _objectArray[0] ɗvf a i[܂B<p>
   *
   * ۃ\bhȂ̂ŔhNXŕK`ĉB
   *
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void connectComponents() throws OhError;

  // @end {abstract methods} -----------------------------------

  
  // @begin {initialization} -----------------------------------

  public void init(String aObjName, int aObjNum,
    int aNumOfObjs, int aNumOfItr, String aLine) throws OhError {
    if (aNumOfItr < 0) {
      throw new IllegalArgumentException(
        "Invalid argument for \"_numberOfIteratorSets\"." +
        NEWLINE + oPrintString(" > ")
      );
    }

    _numberOfIteratorSets = aNumOfItr;
    super.init(aObjName, aObjNum, aNumOfObjs, aLine);
  }

  public void init(String aObjName, int aObjNum, int aNumOfObjs,
    int aNumOfItr) throws OhError {
    init(aObjName, aObjNum, aNumOfObjs, aNumOfObjs, null);
  }

  public void init(String aObjName, int aObjNum,
    int aNumOfObjs, String aLine) throws OhError {
    // Second init().
    init(aObjName, aObjNum, aNumOfObjs, 0, aLine);
  }

  public void init(String aObjName, int aObjNum,
    int aNumOfObjs) throws OhError {
    init(aObjName, aObjNum, aNumOfObjs, 0, null);
  }

  public void init(String aObjName, int aObjNum, String aLine)
    throws OhError {
    init(aObjName, aObjNum, 0, 0, aLine);
  }

  public void init(String aObjName, int aObjNum) throws OhError {
    init(aObjName, aObjNum, 0, 0, null);
  }

  // @end {initialization} -------------------------------------

  // @begin {constructors} -------------------------------------

  /**
   * Creates a new default <code>OhSubSystem</code> instance.
   *
   */
  public OhSubSystem() {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
  }    

  public OhSubSystem(String aObjName, int aObjNum,
    int aNumOfObjs, int aNumOfItr, String aLine) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";

    init(aObjName, aObjNum, aNumOfObjs, aNumOfItr, aLine);
  }

  
  public OhSubSystem(String aObjName, int aObjNum,
    int aNumOfObjs, int aNumOfItr) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
    
    init(aObjName, aObjNum, aNumOfObjs, aNumOfItr);
  }

  public OhSubSystem(String aObjName, int aObjNum,
    int aNumOfObjs, String aLine) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
    
    init(aObjName, aObjNum, aNumOfObjs, aLine);
  }

  public OhSubSystem(String aObjName, int aObjNum,
    int aNumOfObjs) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
    
    init(aObjName, aObjNum, aNumOfObjs);
  }

  public OhSubSystem(String aObjName, int aObjNum) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
    
    init(aObjName, aObjNum);
  }

  // @end {constructors} ---------------------------------------


  // @begin {methods} ------------------------------------------
  
  // 1. creation and reference of "_receivePortArray",
  // "_sendPortArray" and "_componentArray"

  /**
   * <japanese>
   * <p>
   * _iteratorArray vf _numberOfIteratorAets ̔zɂB
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void initializeIteratorArray() throws OhError {                   //; (*8)
    if ((_iteratorArray = new Object[256][_numberOfIteratorSets]) == null) {
      throw new OhError(
        "Cannot get memory for \"_iteratorArray\"." + NEWLINE +
        oPrintString(" > "));
    }

    if ((_numberOfIterators = new int[_numberOfIteratorSets]) == null) {
      throw new OhError(
        "Cannot get memory for \"_numberOfIterators\"." + NEWLINE +
        oPrintString(" > "));
    }

    for (int i = 0; i < _numberOfIteratorSets; i++) {
      _iteratorArray[i]     = null;
      _numberOfIterators[i] = -1;
    }
  }


  /**
   * <japanese>
   * <p>
   * [q̓o^AڒʐMvf̓o^
   * </japanese>
   *
   * @param aP an <code>Object[]</code> value
   * @exception OhError if an error occurs
   */
  public void register(Object[] aP) throws OhError {    //; (*9)

    // Look for "i" which is "_iteratorArray = null".
    // Exit if it is nothing.
    //
    int i = 0;
    boolean flag = false;

    for (i = 0; i < _numberOfIteratorSets; i++) {
      if (_iteratorArray[i] == null) {
        flag = true;
        break;
      }
    }

    if (flag == false) {
      throw new OhError(
        "_iteratorArray is already full. Please check " + NEWLINE +
        "\"_numberOfIteratorSets\"." + NEWLINE +
        oPrintString(" > ")
        );
    }

    int j = 0;
    for (j = 0; j < aP.length; j++) {
      register((OhObject) aP[j]);
    }

    int[] numberOfIteratorsArray = {_numberOfIterators[i]};
    
    _iteratorArray[i]     = makeObjectArray(numberOfIteratorsArray);    //; OhElement (*4)
    _numberOfIterators[i] = numberOfIteratorsArray[0];

    _objectList.clear();
  }


  /**
   * <japanese>
   * <p>
   * [q̓o^AڒʐMvf̓o^
   * </japanese>
   *
   * @param aP an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void register(OhObject aP) throws OhError {
    super.register(aP);
  }


  /**
   * <japanese>
   * <p>
   * [q̓o^AڒʐMvf̓o^
   * </japanese>
   *
   * @param aP an <code>Object</code> value
   * @param aObj an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void register(Object aP, int aObj) throws OhError {
    register(aP, aObj);
  }

  
  /**
   * <japanese>
   * <p>
   * NXAIuWFNgAIuWFNgԍw肵āA_
   * componentArray \IuWFNgiOhObject ^j擾
   * ܂B<p>
   *
   * w肳ꂽIuWFNgo^ĂȂꍇ́Anull Ԃ
   * ܂B<p>
   *
   * _numberOfComponents ̏ꍇA_componentArray 쐬
   * ĂȂƔfAOhError  throw ܂B
   * <p>
   * </japanese>
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>OhObject</code> value
   * @exception OhError if an error occurs
   */
  public OhObject getComponent(String aClassName, String aObjName,
    int aObjNum) throws OhError {
    OhObject p = null;

    if ((p = getFromArray(aClassName, aObjName, aObjNum, _componentArray, _numberOfComponents)) != null)
      return p;
    

    for (int i = 0; i < _numberOfIteratorSets; i++) {
      if ((p = getFromArray(aClassName, aObjName, aObjNum, _iteratorArray[i], _numberOfIterators[i])) != null)
        return p;
    }

    throw new OhError(
      "Cannot get component from array." + NEWLINE +
      "\tAsked > " + aClassName + " : " + aObjName +
        " : " + aObjNum + NEWLINE +
      oPrintString(" > ")
    );
  }

  
  /**
   * <japanese>
   * <p>
   * IuWFNgAIuWFNgԍw肵āA
   * _componentArray \IuWFNgiObject ^j擾܂B<p>
   *
   * w肳ꂽIuWFNgo^ĂȂꍇ́Anull Ԃ܂B<p>
   *
   * _numberOfComponents ̏ꍇA_componentArray 쐬ĂȂƔfA
   * G[bZ[W\Anull Ԃ܂B<p>
   *
   * <pre>
   * [ARGUMENT]
   *  EaObjName ---> 擾\IuWFNg̃IuWFNgB
   *  EaObjNum ----> 擾\IuWFNg̃IuWFNgԍB
   * [RETURN]
   *  EObject -----> w肳ꂽ\IuWFNgBw̃IuWFNg擾łȂꍇ nullB
   * [ACCIDENT]
   *  E_componentArray  Object ^ȊO̕ϐ^ĂꍇB
   * </pre>
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getComponent(String aObjName, int aObjNum) throws OhError {    //; (*11)
    Object p = null;

    if ((p = getFromArray(aObjName, aObjNum, _componentArray, _numberOfComponents)) != null)
      return p;
    

    for (int i = 0; i < _numberOfIteratorSets; i++) {
      if ((p = getFromArray(aObjName, aObjNum, _iteratorArray[i], _numberOfIterators[i])) != null)
        return p;
    }

    throw new OhError(
      "Cannot get component from array." + NEWLINE +
      "\tAsked > " + aObjName + " : " + aObjNum + NEWLINE +
      oPrintString(" > ")
    );
  }

  
  // 2. connection of constituent elements

  /**
   * <japanese>
   * <p>
   * e\vfEp[qԂ̐ڑ <p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * Mp[q܂͑Šn̓͒[qƁA
   * \IuWFNg̎M[qƂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[q́A_className, _objectName,
   * _objectNumber 画ʂ܂B<p>
   *
   * Lp[qƕnfO̎M[qƂ̐ڑ _owner sȂ܂B
   *
   * </japanese>
   *
   * @param aMyP an <code>Object</code> value
   * @param aElement an <code>OhElement</code> value
   * @param aRpClassName a <code>String</code> value
   * @param aRpName a <code>String</code> value
   * @param aRpNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public void connect(OhPort aMyP, OhElement aElement,
    String aRpClassName,
    String aRpName, int aRpNum) throws OhError {                           //; (*1)

    // Gets the receiving port of an internal object.
    //
    OhPort rp = (OhPort) ((OhElement)aElement).
      getReceivePort(aRpClassName, aRpName, aRpNum);

    if (rp == null) {
      throw new
        OhError(
          "Cannot connect ports. Cannot get '_receivePort' to connect."
          + NEWLINE + oPrintString("\t"));
    }

    rp.setSender(aMyP);
    aMyP.registerReceiver(rp);
  }

  
  /**
   * <japanese>
   * <p>
   * e\vfEp[qԂ̐ڑ <p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[qAM[q̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[qAM[q́A_className,
   * _objectName, _objectNumber 画ʂ܂B<p>
   *
   * Lp[qƕnfO̎M[qƂ̐ڑ _owner 
   * sȂ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aS an <code>OhElement</code> value
   * @param aSpClassName a <code>String</code> value
   * @param aSpName a <code>String</code> value
   * @param aSpNum an <code>int</code> value
   * @param aR an <code>OhElement</code> value
   * @param aRpClassName a <code>String</code> value
   * @param aRpName a <code>String</code> value
   * @param aRpNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public void connect(OhElement aS, String aSpClassName,
    String aSpName, int aSpNum,
    OhElement aR, String aRpClassName, String aRpName, int aRpNum)
    throws OhError {

    // Gets the sending port of an internal object.
    // ---------------------------------------------------------
    OhPort sp = (OhPort) aS.getSendPort(aSpClassName, aSpName, aSpNum);
    
    if (sp == null) {
      throw new OhError("Cannot get '_sendPort' to connect." +
        NEWLINE + oPrintString(" > "));
    }
    
    // Gets the receiving port of an internal object.
    //
    OhPort rp = (OhPort) aR.getReceivePort(aRpClassName, aRpName, aRpNum);

    if (rp == null) {
      throw
        new OhError("Cannot get '_receivePort' to connect." +
          NEWLINE + oPrintString(" > "));
    }

    rp.setSender(sp);
    sp.registerReceiver(rp);

  }

  /**
   * <japanese>
   * <p>
   * e\vfEp[qԂ̐ڑ <p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̑M[qƁAMp[q܂͑Šn
   * o͒[qƂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̑M[q́A_className, _objectName,
   * _objectNumber 画ʂ܂B<p>
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aS an <code>Object</code> value
   * @param aSpClassName a <code>String</code> value
   * @param aSpName a <code>String</code> value
   * @param aSpNum an <code>int</code> value
   * @param aMyP an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void connect(
    OhElement aS, String aSpClassName, String aSpName, int aSpNum,
    OhPort aMyP) throws OhError {
    // Gets the sending port to connect.
    //
    OhPort sp = (OhPort) aS.getSendPort(aSpClassName, aSpName, aSpNum);

    if (sp == null) {
      throw new
        OhError(
          "Cannot connect ports. Cannot get '_sendPort' to connect."
          + NEWLINE + oPrintString(" > "));
    }

    aMyP.setSender(sp);
    sp.registerReceiver(aMyP);
  }

  
  /**
   * <japanese>
   * <p>
   * e\vfEp[qԂ̐ڑ <p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * Mp[q܂͑Šn̓͒[qƁA\IuWFNg
   * M[qƂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[q́A_objectName, _objectNumber 
   * 画ʂ܂B<p> ܂ANX̃`FbNȗ܂B
   * <p>
   *
   * Lp[qƕnfO̎M[qƂ̐ڑ _owner 
   * sȂ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aMyP an <code>OhPort</code> value
   * @param aR an <code>OhElement</code> value
   * @param aRpName a <code>String</code> value
   * @param aRpNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void connect(OhPort aMyP,
    OhElement aR, String aRpName, int aRpNum) throws OhError {

    // Gets the receiving port of an internal object.
    //
    OhPort rp = (OhPort) aR.getReceivePort(aRpName, aRpNum);

    if (rp == null) {
      throw
        new OhError(
          "Cannot connect ports. Cannot get '_receivePort' to connect."
          + NEWLINE + oPrintString(" > "));
    }

    rp.setSender(aMyP);
    aMyP.registerReceiver(rp);

  }


  /**
   * <japanese>
   * <p>
   * e\vfEp[qԂ̐ڑ <p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[qAM[q̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[qAM[q́A_objectName,
   * _objectNumber 画ʂ܂B܂ANX̃`FbN
   * ȗ܂B<p>
   *
   * Lp[qƕnfO̎M[qƂ̐ڑ _owner 
   * sȂ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aS an <code>OhElement</code> value
   * @param aSpName a <code>String</code> value
   * @param aSpNum an <code>int</code> value
   * @param aR an <code>OhElement</code> value
   * @param aRpName a <code>String</code> value
   * @param aRpNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void connect(OhElement aS, String aSpName, int aSpNum,
    OhElement aR, String aRpName, int aRpNum) throws OhError {

    // Gets the sending port of an internal object.
    //
    OhPort sp = (OhPort) aS.getSendPort(aSpName, aSpNum);

    if (sp == null) {
      throw
        new
        OhError("Cannot connect ports. Cannot get '_sendPort' to connect."
          + NEWLINE + oPrintString(" > "));
    }

    // Gets the receiving port of an internal object.
    //
    OhPort rp = (OhPort) aR.getReceivePort(aRpName, aRpNum);

    if (rp == null) {
      throw
        new
        OhError("Cannot connect ports. Cannot get '_receivePort' to connect."
          + NEWLINE + oPrintString(" > "));
    }

    rp.setSender(sp);
    sp.registerReceiver(rp);
  }


  /**
   * <japanese>
   * <p>
   * e\vfEp[qԂ̐ڑ <p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̑M[qƁAMp[q܂͑Šn
   * o͒[qƂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̑M[q́A_objectName, _objectNumber 
   * 画ʂ܂B<p> ܂ANX̃`FbNȗ܂B
   * <p>
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aS an <code>OhElement</code> value
   * @param aSpName a <code>String</code> value
   * @param aSpNum an <code>int</code> value
   * @param aMyP an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void connect(
    OhElement aS, String aSpName, int aSpNum,
    OhPort aMyP) throws OhError {  //; (*6)

    // Gets the sending port to connect.
    //
    OhPort sp = (OhPort) aS.getSendPort(aSpName, aSpNum);

    if (sp == null) {
      throw
        new
        OhError(
          "Cannot connect ports. Cannot get '_sendPort' to connect."
          + oPrintString(" > "));
    }

    aMyP.setSender(sp);
    sp.registerReceiver(aMyP);

  }

  /**
   * <japanese>
   * <p>
   * e\vfEp[qԂ̐ڑ <p>
   *
   * nf̒[qԂ̐ڑ
   * </japanese>
   *
   * @param aMySp an <code>OhPort</code> value
   * @param aMyRp an <code>OhPort</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void connect(OhPort aMySp, OhPort aMyRp) throws OhError {    //; (*7)
    aMyRp.setSender(aMySp);
    aMySp.registerReceiver(aMyRp);
  }

  /**
   * <japanese>
   * <p>
   * _receivePortArray, _sendPortArray, _objectArray 쐬A
   * ڑ֌W̒`ɏ]ĕn̐ڑsȂB<p>
   *
   * ̕nfA\IuWFNg̍\ݒ肵܂B
   * ̓Iɂ́Aȉ̎葱sȂ܂B
   *
   * <ol>
   *  <li> OhLinkedList ^IuWFNgq[vɍ쐬 _objectList ɐݒ肷B
   *  <li> Element.registerReceivePorts(), OhElement.makeObjectArray() ɂ
   *       _receivePortArray 쐬B
   *  <li> OhLinkedList.clear() ɂ _objectList ɂB
   *  <li> OhElement.registerSendPorts(), OhElement.makeObjectArray() ɂ
   *       _sendPortArray 쐬B
   *  <li> OhLinkedList.clear() ɂ _objectList ɂB
   *  <li> OhSubSystem.registerComponents(), OhElement.makeObjectArray ɂ
   *       _componentArray 쐬B
   *  <li> OhSubSystem.initializeIteratorArray() ɂ _iteratorArray ̑ꎟzmۂB
   *  <li> OhSubSystem.registerIterators() ɂ _iteratorArray 쐬B
   *  <li> _objectList  null ɐݒ肷B
   *  <li> OhElement.construct() ɂA\IuWFNg̍\ݒ肷B
   *  <li> OhSubSystem.connectComponents() ɂAnf̐ڑsȂB
   * </ol>
   *
   * <pre>
   * [RETURN]
   *  Etrue --> nf̍\ɐݒ肳ꂽꍇB
   *  Efalse -> ̃IuWFNg̍\ݒɈُ킪ꍇB
   *             \IuWFNg̍\ݒɈُ킪ꍇB
   *             nf̐ڑɈُ킪ꍇB
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void construct() throws OhError {
    _objectList = new OhLinkedList();
    
    // --- Makes _receivePortArray. ---

    registerReceivePorts();    // Abstract method call.
                               //; Throws "OhError".
                               //; OhElement  (*3)canYouRegister() pāA
                               //; ̌n̒[q _bufferList ɓo^B

    int[] numberOfReceivePortsArray = {_numberOfReceivePorts};

    _receivePortArray     =
      makePortArray(numberOfReceivePortsArray);
      // (Ref "OhElement#makeObjectArray()")
    _numberOfReceivePorts = numberOfReceivePortsArray[0];
    
    _objectList.clear();
    
    // --- Makes _sendPortArray. ---

    registerSendPorts();    // Abstract method call. Throws "OhError".

    int[] numberOfSendPortsArray = {_numberOfSendPorts};
    
    _sendPortArray     = makeObjectArray(numberOfSendPortsArray);
    _numberOfSendPorts = numberOfSendPortsArray[0];

    _objectList.clear();
    
    // --- Makes _componentArray. ---

    registerComponents();

    int[] numberOfComponentsArray = {_numberOfComponents};
    
    _componentArray     = makeObjectArray(numberOfComponentsArray);
    _numberOfComponents = numberOfComponentsArray[0];

    
    _objectList.clear();


    initializeIteratorArray();    //; (*8)
    registerIterators();          // Abstract method call.

    _objectList = null;

    boolean flag = true;
    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++)
      ((OhElement)_componentArray[k]).construct();

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        ((OhElement)_iteratorArray[i][k]).construct();
      }
    }

    if (flag != true) {
      throw
        new
        OhError("Cannot construct \"components\"." + NEWLINE +
          oPrintString(" > "));
    }

    connectPorts(); 
    connectComponents();
  }


  /**
   * <japanese>
   * <p>
   * MڑƂIɂ̃\bhĂяoA
   * MJn̂߂̏Ƃ܂B<p>
   *
   * ܂ OhElement.canYouSwitch() sA̖߂l false 
   * ̏ꍇAG[bZ[W\Afalse Ԃ܂B<p>
   *
   * ɁA\IuWFNg canYouSwitch () s܂B
   * ŁA߂l false ̍\IuWFNg݂ꍇ́A
   * S\IuWFNg canYouSwitch() sA
   * G[bZ[W\ false Ԃ܂B<p>
   *
   * nfɑ݂S[q canYouSwitch() s
   * ܂B<p>
   *
   * M[qł́A_receiverArray 쐬Af[^pbN^̏
   * ܂B̓IɌ΁A_dataPackArray ̃mۂA_
   * numberOfDataPacks ̒l -1  0 ɂ܂Bp[qł 
   * _receiverArray 쐬邾łB<p>
   *
   * ܂Ƃ߂ƁAȉsȂƂɂȂ܂B
   *
   * <ol>
   *  <li> _receiverArray 쐬B
   *  <li> _numberOfDataPacks ̒l -1  0 ɕύXB
   * </ol>
   *
   * <pre>
   * [RETURN]
   *   Etrue --> nf̑S[q𐳏ɓ]łꍇB
   *   Efalse -> nf̒[q̓, ɓ]łȂ݂̂ꍇB
   * </pre>
   *
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public void doSwitch() throws OhError {    //; (*15)
    super.doSwitch();
    boolean flag = true;    // For judgment.
    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++)
      ((OhElement)_componentArray[k]).doSwitch();

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        ((OhElement)_iteratorArray[i][k]).doSwitch();
      }
    }

    if (flag != true) {
      throw new OhError("Cannot switch those 'components'."
          + NEWLINE + oPrintString(" > "));
    }

  }


  /**
   * <japanese>
   * <p>
   * vZJnO̊mF <p>
   *
   * ̃\bh̖߂l true Ȃ OhSubSystem ͌vZJn
   * ł܂B<p>
   *
   * ܂ OhElement.areYouReady() pďLMp[qA
   * LMp[qɑ΂ČvZO̊mFƂs܂B̖
   * l false ̏ꍇAG[bZ[W\ false Ԃ
   * ܂B <p>
   *
   * ɁAS\IuWFNg areYouReady() pČvZO̊m
   * FƂsȂ܂BŁA߂l false ̃IuWFNg
   * ݂ꍇAS\IuWFNgareYouRedy() ŊmFG[
   * bZ[W\ false Ԃ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> vZJnB
   *  Efalse -> vZJnsB
   * </pre>
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean areYouReady() throws OhError {
    if (super.areYouReady() != true) {
      throw new OhError(oPrintString(" > "));
    }

    boolean flag = true;    // For judgment.
    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++)
      if (((OhElement)_componentArray[k]).areYouReady() != true)
        flag = false;
        // The element model which cannot start calculating exists.

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if (((OhElement)_iteratorArray[i][k]).areYouReady() != true)
          flag = false;
      }
    }

    if (flag != true) {
      throw
        new
        OhError("These 'components' are not ready." + NEWLINE +
          oPrintString(" > "));
    }

    return flag;
  }


  
  // 3. initialization function

  /**
   * Deletion of the string of components.
   *
   * <japanese>
   * <p>
   * \IuWFNg̃f[^̏
   * </japanese>
   *
   */
  public void clearDataString() {    //; (*17)
    super.clearDataString();

    for (int k = 0; k < _numberOfComponents; k++)
      ((OhElement)_componentArray[k]).clearDataString();

    for (int i = 0; i < _numberOfIteratorSets; i++) {
      for (int k = 0; k < _numberOfIterators[i]; k++) {
        ((OhElement)_iteratorArray[i][k]).clearDataString();
      }
    }
  }
      

  /**
   * <japanese>
   * <p>
   * ̃\bh́A(Ref "OhElement") NX̒ۃ\bhłB
   * WIȕnf́AI\IuWFNgɂ
   * sȂ̂ŁAp[^AԗʂȂ̂ƍl܂B
   * ŁÃNXł͉Ȃ悤ɒ`܂BKv
   * ΔhNXŕύXĉB
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setParameter(OhBufferedReader aBr) throws OhError {}

  /**
   * <japanese>
   * <p>
   * ̃\bh́Ang̃p^ƁAn̍\vf̃p^ǂݎ܂B
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void getParameter(OhBufferedReader aBr) throws OhError {    //; (*18)
    setParameter(aBr);

    try {
      if (aBr.skipWhites() == -1) {
        OhError e = new OhError("Parameter file error.");
        throw e;
      }

      if ((char)aBr.read() != '{') {
        throw new OhError("{ should come first" + NEWLINE +
          oPrintString(" > "));
      }
      
      aBr.readLine();

      
      String headLine = null;
      StringTokenizer strToken = null;
      String className = "";
      String objName = "";
      int objNum = 0;
      Object obj = null;
      
      while (true) {
        if (aBr.skipWhites() == -1) {
          OhError e = new OhError("Parameter file error.");
          throw e;
        }
        
        headLine = aBr.readLine();
        
        if (headLine.substring(0, 1).equals("}"))
          break;
        
        String s = headLine;
        StringTokenizer st = new StringTokenizer(s);
        int i = 0;
        while (st.hasMoreTokens()) {
          st.nextToken();
          i++;
        }

        strToken = new StringTokenizer(headLine);
        
        if (i == 1) {
          objName = strToken.nextToken();
          obj = this.getComponent(objName, 0);
        }
        else if (i == 2) {
          objName = strToken.nextToken();
          objNum = Integer.parseInt(strToken.nextToken());
          obj = this.getComponent(objName, objNum);
        }
        else if (i == 3) {
          className = strToken.nextToken();
          objName = strToken.nextToken();
          objNum = Integer.parseInt(strToken.nextToken());
          obj = this.getComponent(className, objName, objNum);
        }
        else {
          throw new OhError(oPrintString("    "));
        }

        ((OhElement)obj).getParameter(aBr);

      }
    }
    catch (IOException e) {
      throw new OhError("Error in parameter line." + NEWLINE +
        oPrintString("\t"));
    }
  }


  /**
   * Initialization of the time of constituent objects. <p>
   *
   * "_currentTime" is read.
   * If it is 0, "_currentTime" will not be changed.
   *
   * <japanese>
   * <p>
   * g _currentTime ƍ\IuWFNg _currentTime 
   * w肵ɏ܂B<p>
   *
   * ʏAvZJnƂ̏́ÂƂ̂ŁA
   * TuVXȅ̃TuVXe\vf̏
   * Ƃ鑀Ă܂B<p>
   *
   * (Ref "OhElement#initialTime()") QƂĂB
   * </japanese>
   *
   * @param aInitTime an <code>OhTimePoint</code> value
   */
  public void initialTime(OhTimePoint aInitTime) {
    super.initialTime(aInitTime);

    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++) {
      ((OhElement)_componentArray[k]).initialTime(aInitTime);
    }
    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        ((OhElement)_iteratorArray[i][k]).initialTime(aInitTime);
      }
    }
  }


  /**
   * Sets the the initial state of sub systems itself.
   *
   * <japanese>
   * <p>
   * ng̏ԗʂ̐ݒ <p>
   *
   * ng̃NXAIuWFNgAIuWFNgԍǂݎA
   * ԗʂĂs̐擪ɓǂݎʒuԂł̃\bhĂ΂܂B<p>
   * 
   * ԗʂǂ݂ƂAt@C|C^͏ԂǂݎsɒuĂ܂B<p>
   *
   * ̃\bh́AgetInitialState() Ă΂܂B
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setInitialState(OhBufferedReader aBr) throws OhError {      //; (*19)

    int  year = 0;
    long tsec = 0L;
    double frac = 0.0;

    try {
      aBr.skipWhites();
      year = Integer.parseInt(aBr.readWord());
      tsec = Long.parseLong(aBr.readWord());
      frac = Double.parseDouble(aBr.readWord());
      System.out.println("" + year + " " + tsec + " " + frac);
    }
    catch (IOException e) {
      throw new OhError("Error in setInitialState()." + NEWLINE +
        oPrintString("\t"));
    }
    catch (NumberFormatException e) {
      throw new OhError("Error in setInitialState()." + NEWLINE +
        oPrintString("\t"));
    }

    if (year > 0) {
      _currentTime = new OhTimePoint(year, tsec, frac);
    }
  }


  /**
   * Sets the the initial state of sub systems itself and components.
   *
   * <japanese>
   * <p>
   * ng̏ԗʂƍ\IuWFNg̏ԗʂ̐ݒ
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void getInitialState(OhBufferedReader aBr) throws OhError {

    setInitialState(aBr);
    try {
      if (aBr.skipWhites() == -1) {
        OhError e = new OhError(
          "Initial file error in \"OhSubSystem\".");
        throw e;
      }

      char c = (char)aBr.read();
      if (c != '{') {
        throw new OhError("{ should come first, but " + c + NEWLINE +
          oPrintString(" > "));
      }
      
      aBr.readLine();
      
      
      String headLine = null;
      StringTokenizer strToken = null;
      String className = "";
      String objName = "";
      int objNum = 0;
      Object obj = null;
      
      while (true) {
        if (aBr.skipWhites() == -1) {
          OhError e = new OhError("Parameter file error.");
          throw e;
        }
        
        headLine = aBr.readLine();
        
        if (headLine.substring(0, 1).equals("}"))
          break;
        
        String s = headLine;
        StringTokenizer st = new StringTokenizer(s);
        int i = 0;
        while (st.hasMoreTokens()) {
          st.nextToken();
          i++;
        }

        strToken = new StringTokenizer(headLine);
        
        if (i == 1) {
          objName = strToken.nextToken();
          obj = this.getComponent(objName, 0);
        }
        else if (i == 2) {
          objName = strToken.nextToken();
          objNum = Integer.parseInt(strToken.nextToken());
          obj = this.getComponent(objName, objNum);
        }
        else if (i == 3) {
          className = strToken.nextToken();
          objName = strToken.nextToken();
          objNum = Integer.parseInt(strToken.nextToken());
          obj = this.getComponent(className, objName, objNum);
        }
        else {
          throw new OhError(oPrintString("    "));
        }
        
        ((OhElement)obj).getInitialState(aBr);
      }
    }
    catch (IOException e) {
      throw new OhError("Error in initial state line." + NEWLINE +
        oPrintString("\t"));
    }
  }


  /**
   * Save to the file of the own last state of the subsystems.
   *
   * <japanese>
   * <p>
   * ng̍ŏIԂ̃t@Cւ̕ۑ
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveTerminalState(PrintWriter aPw) {
    if (aPw != null)
      aPw.print(_currentTime.getYear() + " " + _currentTime.getTsec());
  }
      


  
  /**
   * Save the last state of the own subsystems and the components.
   *
   * <japanese>
   * <p>
   * ng̍ŏIԂƍ\vf̍ŏIԂ̕ۑ <p>
   *
   * ̃\bh́A\vfɍŏIԂo悤Ɏw߂𑗂܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void terminalState(PrintWriter aPw) {    //; (*24)
    int i = 0;
    int k = 0;

    if (aPw != null) {
      // Save the last state of the own subsystem.
      //
      aPw.print(_className + " " + _objectName + " " + _objectNumber + NEWLINE);
      saveTerminalState(aPw);
      aPw.print(NEWLINE);

      // Save the last state of the components.
      //
      aPw.print("{" + NEWLINE);
      for (k = 0; k < _numberOfComponents; k++) {
        if (k > 0) {
          aPw.print(NEWLINE);
        }
        ((OhElement)_componentArray[k]).terminalState(aPw);
      }

      for (i = 0; i < _numberOfIteratorSets; i++) {
        for (k = 0; k < _numberOfIterators[i]; k++) {
          aPw.print(NEWLINE);
          ((OhElement)_iteratorArray[i][k]).terminalState(aPw);
        }
      }
      aPw.print("}" + NEWLINE);
    }
  }


  /**
   * <japanese>
   * <p>
   * ̃\bh́A܂gɑ΂ doShareInfo() sA
   * Ɏ̍\vfɑ΂ doShareInfo() s܂B
   * ܂߂Ĉł doShareInfo() svf
   * OperationFlag.E_DONE
   * Ԃ܂B
   * SĂ̗vfł doShareInfo sĂ
   * OperationFlag.E_ALREADY
   * Ԃ܂B
   * ǂ̏ꍇłȂƂ
   * OperationFlag.E_UNDONE
   * Ԃ܂B
   * </japanese>
   *
   * @return an <code>OperationFlag</code> value
   * @exception OhError if an error occurs
   */
  public OperationFlag doShareInfo() throws OhError {    //; (*14)
    OperationFlag flag;

    boolean calcany = false;
    boolean calcall = true;

    if (_shareInfoDoneP == false) {
      if (shareInfo() == true) {
        _shareInfoDoneP = true;
        calcany = true;
      }
    }

    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++) {
      flag = ((OhElement)_componentArray[k]).doShareInfo();
      if (flag == OperationFlag.E_DONE)
        calcany = true;
      if (flag == OperationFlag.E_UNDONE)
        calcall = false;
    }

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        flag = ((OhElement)_iteratorArray[i][k]).doShareInfo();
        if (flag == OperationFlag.E_DONE)
          calcany = true;
        if (flag == OperationFlag.E_UNDONE)
          calcall = false;
      }
    }

    if (calcany == true)
      return OperationFlag.E_DONE;
    if (calcall == true)
      return OperationFlag.E_ALREADY;
    
    return OperationFlag.E_UNDONE;
  }

  
  /**
   * <japanese>
   * <p>
   * n̍\vf̂ꂼɎw߂𑗂A
   * Xg[ aPwijŎw肳Ăt@Cɏo܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveSendPortStock(PrintWriter aPw) {    //; (*21)
    int i = 0;
    int k = 0;

    if (aPw != null) {
      if (_numberOfComponents <= 0)
        return;

      if (_numberOfComponents > 0) {
        aPw.print(_className + " " + _objectName + " " + _objectNumber + NEWLINE);
        aPw.print("{" + NEWLINE);

        for (k = 0; k < _numberOfComponents; k++) {
          ((OhElement)_componentArray[k]).saveSendPortStock(aPw);
        }

        for (i = 0; i < _numberOfIteratorSets; i++) {
          for (k = 0; k < _numberOfIterators[i]; k++) {
            ((OhElement)_iteratorArray[i][k]).saveSendPortStock(aPw);
          }
        }

        aPw.print("}" + NEWLINE);
      }
    }
  }



  // 4. information output function

  /**
   * Prints out the information on a constituent objects.
   *
   * <japanese>
   * <p>
   * \IuWFNg̏o͂܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printComponents(PrintWriter aPw, String aHead) {    //; (*22 cpp)
    printObjects(aPw, aHead, "Components :", _componentArray, _numberOfComponents);

    for (int i = 0; i < _numberOfIteratorSets; i++)
      printObjects(aPw, aHead, "Components :", _iteratorArray[i], _numberOfIterators[i]);
  }


  /**
   * Prints out the information on a constituent objects.
   *
   * <japanese>
   * <p>
   * \IuWFNg̏o͂܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void printComponents(PrintWriter aPw) {                  //; (*22 h)
    printObjects(aPw, "    ", "Components :", _componentArray, _numberOfComponents);

    for (int i = 0; i < _numberOfIteratorSets; i++)
      printObjects(aPw, "    ", "Components :", _iteratorArray[i], _numberOfIterators[i]);
  }



  // 5. the execution function of calculation

  /**
   * <japanese>
   * <p>
   * ̏o <p>
   *
   * ԗʂ߂ɁA_currentTime ł̏o͂`܂B
   * M[qɎƏo͒l݂܂B
   * _currnet_time ł̏o͂vZłꍇ true Ԃ܂B
   * Kvȓ̓f[^ĂȂȂǂāA
   * _currentTime ̏o͂vZłȂꍇ false Ԃ܂B<p>
   *
   * ȂȀꍇ͕n̏̏o͂͂ȂƎv̂ŁA
   * ł͏ true Ԃ悤ɒ`Ă܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean initialOutput() throws OhError {
    return true;
  }

  
  /**
   * <japanese>
   * <p>
   *  <p>
   *
   * ɂāASf[^擾܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean initialWork() throws OhError {
    return true;
  }

  
  /**
   * A first-time output.
   *
   * <japanese>
   * <p>
   * ̏o
   * </japanese>
   *
   * @return an <code>OperationFlag</code> value
   * @exception OhError if an error occurs
   */
  public OperationFlag doInitialWork() throws OhError {    //; (*13)
    OperationFlag flag;

    boolean calcany = false;
    boolean calcall = true;

    if (_initialWorkDoneP == false) {
      if (initialWork() == true) {
        _initialWorkDoneP = true;
        calcany = true;
      }
    }

    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++) {
      flag = ((OhElement)_componentArray[k]).doInitialWork();    //; Throws "OhError".
      if (flag == OperationFlag.E_DONE)
        calcany = true;
      if (flag == OperationFlag.E_UNDONE)
        calcall = false;
    }

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        flag = ((OhElement)_iteratorArray[i][k]).doInitialWork();    //; Throws "OhError".
        if (flag == OperationFlag.E_DONE)
          calcany = true;
        if (flag == OperationFlag.E_UNDONE)
          calcall = false;
      }
    }

    if (calcany == true)
      return OperationFlag.E_DONE;
    if (calcall == true)
      return OperationFlag.E_ALREADY;
    
    return OperationFlag.E_UNDONE;
  }
    

  /**
   * <japanese>
   * <p>
   * vf̌vZɕKvȃf[^̂AaRp (arg) 󂯎ׂf[^̃f[^L^Ԃ܂B<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[^pbN
   * M[q aRp MĂȂ΁Avf͌vZJn邱Ƃł܂B
   * </japanese>
   *
   * @param aRp an <code>Object</code> value
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePointImmutable getNecessaryTimeFrom(Object aRp) {    //; (*25)
    int k = 0;
    int j = -1;

    for (k = 0; k < _numberOfReceivePorts; k++) {
      if (aRp == _receivePortArray[k])
        j = k;
    }

    if (j == -1)
      return _currentTime;

    return ((OhSenderPort)aRp).getNecessaryTimeFrom();
  }


  /**
   * <japanese>
   * <p>
   * vf̌vZɕKvȃf[^̂AaRp (arg) 󂯎ׂf[^̃f[^L^Ԃ܂B<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[^pbN
   * M[q aRp MĂȂ΁Avf͌vZJn邱Ƃł܂B
   * </japanese>
   *
   * @param aRp an <code>Object</code> value
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePointImmutable getNecessaryTimeTo(Object aRp) {    //; (*26)
    int k = 0;
    int j = -1;

    for (k = 0; k < _numberOfReceivePorts; k++) {
      if (aRp == _receivePortArray[k])
        j = k;
    }

    if (j == -1)
      return _currentTime;

    return ((OhSenderPort)aRp).getNecessaryTimeTo();
  }



  /**
   * The check before calculation.
   *
   * <japanese>
   * <p>
   * vZO̊mF
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean canYouCalculate() throws OhCannotGetDataError, OhError {
    return true;
  }


  /**
   * Calculation of the time step.
   *
   * <japanese>
   * <p>
   * ^CXebv̌vZ
   * </japanese>
   *
   * @return a <code>long</code> value
   * @exception OhError if an error occurs
   */
  public OhTime calculateTimeStep() throws OhError {
    return new OhTime(0L, 0.0);
  }


  /**
   * Calculation for 1 step.
   *
   * <japanese>
   * <p>
   * PXebv̌vZ
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean calculate() throws OhCannotGetDataError, OhError {
    return true;
  }


  /**
   * <japanese>
   * <p>
   * vZvfQvZڕWɓBǂ`FbNB<p>
   *
   * ̃NXł́AwǗҁxƂĂ OhSubSystem ̗ꂩ`Ă܂B
   * Ȃ킿A̓ɔvZsȂvfQƂƂOƂĂA
   * gvZsȂꍇɑΉĒ`Ă킯ł͂܂B
   * āAgvZsȂꍇ́Aꂪ\ł悤ɍĒ`ĉB
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean hasReachedTargetTime(int aNum) {    //; (*27)
    boolean flag = true;

    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      if (((OhElement)_iteratorArray[aNum][i]).hasReachedTargetTime() == false)
        flag = false;

    return flag;
  }


  /**
   * <japanese>
   * <p>
   * vZvfQ`FbNB<p>
   *
   * ̃NXł́AwǗҁxƂĂ OhSubSystem ̗ꂩ`Ă܂B
   * Ȃ킿A̓ɔvZsȂvfQƂƂOƂĂA
   * gvZsȂꍇɑΉĒ`Ă킯ł͂܂B
   * āAgvZsȂꍇ́Aꂪ\ł悤ɍĒ`ĉB
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean hasConverged(int aNum) throws OhError {
    boolean flag = true;

    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      if (((OhElement)_iteratorArray[aNum][i]).hasConverged() == false)
        flag = false;

    return flag;
  }

  /**
   * Renewal of the current time.
   *
   * <japanese>
   * <p>
   * ݎ̍XV <p>
   *
   * _componentArray ɓo^Ăvf _currentTime 
   * _iteratorArray ɓo^Ăvf _fixedTime ̂A
   * łxĂ _currentTime ƂB
   * </japanese>
   *
   * @return a <code>boolean</code> value
   */
  public void renewCurrentTime() throws OhError {
    int k = 0;
    OhTimePoint[] tp = new OhTimePoint[2];
    tp[0] = OhTimePoint.OhTimePointMax;

    // _componentArray
    //
    for (k = 0; k < _numberOfComponents; k++) {
      tp[1] = ((OhElement)_componentArray[k]).getCurrentTime();
      if (tp[1].compareTo(tp[0]) < 0) {
        OhTimePoint tP = new OhTimePoint();
        tP.copy(tp[1]);
        tp[0] = tP;
      }
    }

    // _iteratorArray
    //
    for (k = 0; k < _numberOfIteratorSets; k++) {
      tp[1] = ((OhElement)_iteratorArray[k][0]).getFixedTime();
      if (tp[1].compareTo(tp[0]) < 0) {
        OhTimePoint timeP = new OhTimePoint();
        timeP.copy(tp[1]);
        tp[0] = timeP;
      }
    }

    if (tp[0].compareTo(_currentTime) < 0) {
      throw new OhError(
        "error" + NEWLINE +
        "\tPut back _currentTime from " + _currentTime +
          " to " + tp[0] + NEWLINE +
        oPrintString(" > ")
        );
    }
    _currentTime = tp[0];
  }
      

  /**
   * This orders to perform "rewindCurrentTime()" to
   * the element group which repeated calculation did not converge.
   *
   * <japanese>
   * <p>
   * vZȂvfQ(_iteratorArray[aNum])ɑ΂
   * rewindCurrentTime() s悤߂B
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   */
  public void rewindCurrentTime(int aNum) throws OhError {          //; (*30)
    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      ((OhElement)_iteratorArray[aNum][i]).rewindCurrentTime();
  }

  
  public void flushData(int aNum) throws OhError {                  //; (*31)
    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      ((OhElement)_iteratorArray[aNum][i]).flushData();
  }


  public void renewFixedTime(int aNum) throws OhError {             //; (*32)
    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      ((OhElement)_iteratorArray[aNum][i]).renewFixedTime();
  }


  public void renewTargetTime(int aNum, OhTime aItrs) {            //; (*33)
    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      ((OhElement)_iteratorArray[aNum][i]).renewTargetTime(aItrs);
  }


  public void initializeTargetTime() throws OhError {    //; (*34)
    int i = 0;

    // _iteratorArray
    //
    for (i = 0; i < _numberOfIteratorSets; i++)
      renewTargetTime(i, calculateIterationStep(i));

    // _componentArray
    //
    for (i = 0; i < _numberOfComponents; i++)
      ((OhElement)_componentArray[i]).initializeTargetTime();
  }


  /**
   * This returns the minimum value among the return values of "calculateIterationStep()"
   * of "_iteratorArray[aNum]".
   *
   * <japanese>
   * <p>
   * ̃\bh́A_iteratorArray[aNum]  calculateIterationStep() ̖߂l̂Aŏ̒lԂB
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   * @return a <code>long</code> value
   */
  public OhTime calculateIterationStep(int aNum) throws OhError {    //; (*35)
    OhTime tmin = new OhTime(Long.MAX_VALUE, 0.0);
    OhTime time;

    for (int i = 0; i < _numberOfIterators[aNum]; i++) {
      time = ((OhElement)_iteratorArray[aNum][i]).calculateIterationStep();
      if (time.toDouble() < tmin.toDouble())
        tmin.copy(time);
    }
    return tmin;
  }


  /**
   * Operation function.
   *
   * <japanese>
   * <p>
   * Z@\ <p>
   *
   * e\vfɏvZs߂𑗂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   */
  public boolean work() throws OhCannotGetDataError, OhError {    //; (*36)
    int i = 0;
    int j = 0;
    int k = 0;
    boolean calc    = false;    // Whether the function was called and it calculated also at once.
                                //; ֐Ă΂ĈxłvZǂB
    boolean flagAll = false;    // "true" when a component calculates at least one.
                                //; \vfivZvf܂ށjPłvZ trueB
    boolean flagItr = false;    // "true" when a repeated calculation element calculates at least one.
                                //; vZvfPłvZ trueB

    int[] itrCounter = new int[_numberOfIteratorSets];

    for (i = 0; i < _numberOfIteratorSets; i++)
      itrCounter[i] = 0;

    // Calculation of components.
    //
    do {
      flagAll = false;

      // _componentArray
      //
      for (k = 0; k < _numberOfComponents; k++) {
        if (((OhElement)_componentArray[k]).work() == true) {    //; Throws "OhCannotGetDataError".
          calc = flagAll = true;
          break;
        }
      }
      while (++k < _numberOfComponents)
        ((OhElement)_componentArray[k]).work();    //; Throws "OhCannotGetDataError".

      // _iteratorArray
      //
      for (i = 0; i < _numberOfIteratorSets; i++) {
        do {
          flagItr = false;
          for (j = 0; j < _numberOfIterators[i]; j++) {
            if (((OhElement)_iteratorArray[i][j]).work() == true) {
              calc = flagAll = flagItr = true;
              break;
            }
          }
          while (++j < _numberOfIterators[i])
            ((OhElement)_iteratorArray[i][j]).work();    //; Throws "OhCannotGetDataError".
        }
        while (flagItr == true);

        if (hasReachedTargetTime(i) == true) {
          if (hasConverged(i) == true) {
            flushData(i);
            renewFixedTime(i);
            renewTargetTime(i, calculateIterationStep(i));
            itrCounter[i] = 0;
          }
          else {
            rewindCurrentTime(i);
            if (++itrCounter[i] >= 1000) {
              throw new OhError(
                "Iterative calculation cannot converge" + NEWLINE +
                "tin group [" + i + "]." + NEWLINE +
                oPrintString(" > ")
              );
            }
          }
        }
      }
    }
    while (flagAll == true);

    if (calc == false) {
      return false;
    }

    renewCurrentTime();
    return true;
  }
    
} // OhSubSystem
