/**
 * @(#) Class to express time control condition.
 */

package ohymosj;

import java.util.*;
import java.io.*;


/**
 * OhTimeControl;
 *
 * TimeControl class. <p>
 *
 * Definition of TimeControl. <p>
 *
 * <ul>
 *  <li> GO_BEYOND_TERMINAL_TIME        : Can go beyond the teminal time
 *  <li> DO_NOT_GO_BEYOND_TERMINAL_TIME : Cannot go beyond the terminal time
 * </ul>
 *
 * <japanese>
 * <p>
 * ԖڕW𒴂ČvZ邩ۂ\萔NXB<p>
 *
 * <ul>
 *  <li> GO_BEYOND_TERMINAL_TIME        : ԖڕW𒴂ČvZB
 *  <li> DO_NOT_GO_BEYOND_TERMINAL_TIME : ԖڕW𒴂ɌvZB
 * </ul>
 *
 * ԖڕWƂ́A
 * ڕW (Ref "OhTotalSystem#_terminalTime") ȑO
 * iʂȏꍇƂāA(Ref "OhTotalSystem#_terminalTime")
 * ɈvĂ悢j̎ŁASĂ̗vf͂̎𒴂邱
 * ڕWɌvZi߂ĂƂłB<p>
 *
 * ԖڕẂA
 * (Ref "OhTotalSystem") ̃oϐ
 * (Ref "OhTotalSystem#_mediumTermTargetTime") Ƃ
 * ^Ă܂B<p>
 *
 * (Ref "OhTotalSystem#_terminalTime") ȑOɒԖڕWݒ
 * ASĂ̗vf͒ԖڕW܂ŌvZΐ摖肹A̗v
 * f̌vZ҂悤ɂ̂悤ɂĂ܂B<p>
 *
 * ܂AԖڕW̓ɂāAŠn̗vfׂĂ
 * ŌvZi߂lɂȂłȂAʂ̍팸}
 * ܂B܂AvfXgbNĂ˂΂ȂȂf[^pbN
 * ́AԖڕW玟̒ԖڕW܂ł̎ԕł悭A
 * vZԂɂĎgpʂϓ邱ƂȂȂƍl
 * ܂B
 *
 * </japanese>
 * 
 *
 * @author satoh, shiiba
 * @version $Revision: 1.8 $ $Date: 2004/01/19 07:49:25 $
 */
public final class OhTimeControl implements Serializable
{
  // @begin {instance variable or static variable} -------------
  
  private static int nextOrdinal = 0;
  private final  int ordinal     = nextOrdinal++;

  /**
   * GoBeyondTerminalTime: <p>
   * Can go beyond the termminal time
   *
   * <japanese>
   * <p>
   * ԖڕW𒴂ČvZB
   * </japanese>
   *
   */
  public static final OhTimeControl GO_BEYOND_TERMINAL_TIME =
    new OhTimeControl();

  /**
   * DoNotGoBeyondTerminalTime: <p>
   * Cannot go beyond the terminal time
   *
   * <japanese>
   * <p>
   * ԖڕW𒴂ɌvZB
   * </japanese>
   *
   */
  public static final
    OhTimeControl DO_NOT_GO_BEYOND_TERMINAL_TIME =
    new OhTimeControl();

  private static final
    OhTimeControl[] PRIVATE_VALUES =
    {GO_BEYOND_TERMINAL_TIME, DO_NOT_GO_BEYOND_TERMINAL_TIME};
  
  // @end {instance variable or static variable} ---------------
  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>OhTimeControl</code> instance.
   *
   */
  private OhTimeControl() {}

  // @end {constructors} ---------------------------------------

  
  // @begin {methods} ------------------------------------------

  private Object readResolve() throws ObjectStreamException {
    return PRIVATE_VALUES[ordinal];
  }

  // @end {methods} -------------------------------------------

} // OhTimeControl
