/**
 * @(#) Base class for creating total systems.
 */

package ohymosj;

import java.util.*;
import java.io.*;

/**
 * Base class for creating total systems.<p>
 *
 * <japanese>
 * <p>
 * {^ŠnfɑΉNX <p>
 *
 * Šnf́At@CóAlbg[NɂóAL[
 * {[h́ACRT o͂ȂǁAXGȃf[^̓o͂ɑΉ
 * BŠnf́AGȃf[^n̓Ɏ荞
 * ́AS(Ref "OhDataPack") ^̃f[^ɕϊAV~[V
 * ʂV~[Vs҂ɒ񋟂鎞́A
 * (Ref "OhDataPack") ^Œ~ςĂf[^V~[V
 * s҂̖]ރf[^`ɕϊ܂BŁAf[^̕ϊɊ
 * ̓ÍAVɍ쐬͒[qAo͒[qōsȂA
 * Šnfł́A͒[qEo͒[q̎葱ő삷
 * ̂Ƃ܂B<p>
 *
 * ܂A(Ref "OhDataPack") ^ɕϊ̓f[^̑ɁAS
 * nf́AŠnɊ܂܂Svf̃p[^Eԗʂ
 * lV~[Vs҂pӂf[^t@C
 * ݎ@\Ă܂B܂AV~[Vs҂Ƃ̑
 * bɂAV~[VڕW肷@\Ă
 * B<p>
 *
 * Šnf́AvfEnɂ\AƂĐv
 * ZsȂ킸AnfƓl̍\Ă܂B
 * {VXeł́AnfpđŠnf쐬
 * B<p>
 *
 * Anfnɑ΂Cvbgf[^𕔕nO
 * Mē̂ɑ΂AŠńA͒[qpăCvb
 * gf[^琶A\vfɑM܂B܂A\vf
 * AnOɃAEgvbgf[^Anf
 * PɊOɑM邾łAŠnf͏o͒[qpāA
 * Ⴆ΁Ao̓t@CɋL^ƂsȂ܂B
 *
 * </japanese>
 *
 * @author satoh, shiiba
 * @version $Revision: 1.78 $ $Date: 2004/01/19 07:49:25 $
 */
public abstract class OhTotalSystem extends OhSubSystem
  implements OhConst
{

  // @begin {instance variable or static variable} -------------

  /**
   * terminal time.
   *
   * <japanese>
   * <p>
   * ڕWB
   * </japanese>
   */
  protected OhTimePoint _terminalTime;

  /**
   * Medium-term target time.
   *
   * <japanese>
   * <p>
   * ԖڕWB
   * </japanese>
   */
  OhTimePoint _mediumTermTargetTime;


  /**
   * The string which contains the list of the newest data times
   * of data packs which are obtained through the input
   * ports. This information will be given to the user of the
   * program, wheh the user is asked to enter the target time.<p>
   *
   * <japanese>
   * <p>
   * ͒[q瓾f[^̍ŐVf[^Ԃ̃Xg^
   * B(Ref "#calculateTimeStep()") ̒ŁAsɁAv
   * ZڕW̓͂߂ƂɁA[Uւ̏ƂĒ񋟂
   * ܂B<p>
   *
   * </japanese>
   */
  String _newestDataTimeCol;

  // @end {instance variable or static variable} ---------------


  // @begin {abstract methods} ---------------------------------

  // 1. registration of possession ports, possession element
  // objects and possession subsystem objects.

  /**
   * <japanese>
   * <p>
   * ͒[qo^܂B<p>
   *
   * ۃ\bhłB
   * Šnf̃vO}`Kv܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerInputPorts() throws OhError;

  /**
   * <japanese>
   * <p>
   * o͒[qo^܂B<p>
   *
   * ۃ\bhłB
   * Šnf̃vO}`Kv܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerOutputPorts() throws OhError;

  /**
   * <japanese>
   * <p>
   * Šn̍\vfo^钊ۃ\bhłB<p>
   *
   * ̃NXpđŠn쐬vO}́A
   * ̃\bh`Kv܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerComponents() throws OhError;

  /**
   * <japanese>
   * <p>
   * vZvf _iteratorArray ɓo^B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerIterators() throws OhError;
  //;
  //; Added by Y.Satoh (2003.12.10)

  /**
   * <japanese>
   * <p>
   * Šnf̒[q̐ڑ (Ref "OhSubSystem#connect()")
   * pĒ`܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void connectPorts() throws OhError;

  /**
   * Abstract method which is used for connecting ports of
   * elements within the OhTotalSystem.  The programmer who
   * provides a new OhTotalSystem class must define this method.
   * He can use various (Ref "OhSubSystem#connect()") methods to
   * write the connectPort() method.
   *
   * <japanese>
   * <p>
   * Šnf̒[qڑ邽߂Ɏgp钊ۃ\b
   * hłBV OhTotalSystem NX񋟂悤ƂvO
   * }́Ã\bh`Ȃ΂Ȃ܂B̃\b
   * h߂ɁA(Ref "OhSubSystem") Œ`ĂlX 
   * (Ref "OhSubSystem#connect()") \bhgpł܂B<p>
   *
   * : (Ref "OhScfTotalSystem#connectPorts()") 
   * <japanese>
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void connectComponents() throws OhError;

  // @end {abstract methods} -----------------------------------

  
  // @begin {initialization} -----------------------------------

  public void init(String aObjName, int aObjNum, int aNumOfItr)
    throws OhError {
    super.init(aObjName, aObjNum, 0, aNumOfItr, null);
  }

  public void init(String aObjName) throws OhError {
    super.init(aObjName, 0, 0, 0, null);
  }
  
  public void init(int aObjNum, int aNumOfItr) throws OhError {
    super.init("OhTotalSystem", aObjNum, 0, aNumOfItr, null);
  }

  public void init() throws OhError {
    super.init("OhTotalSystem", 0, 0, 0, null);
  }

  
  // @end {initialization} -------------------------------------

  // @begin {constructors} -------------------------------------

  /**
   * Creates a new default <code>OhTotalSystem</code> instance.
   *
   */
  public OhTotalSystem() {
    super();
    _terminalTime      = OhTimePoint.OhTimePointMin;
    _newestDataTimeCol = "";
    _className         = "OhTotalSystem";
  }    

  // @end {constructors} ---------------------------------------


  // @begin {methods} ------------------------------------------

  /**
   * Returns "_newestDataTimeCol".
   *
   * @return a <code>String</code> value
   */
  public String getNewestDataTimeCol() {
    return _newestDataTimeCol;
  }

  /**
   * Add aStr to _newestDataTimeCol.
   *
   * @param aStr a <code>String</code> value
   */
  public void addNewestDataTime(String aStr) {
    _newestDataTimeCol = _newestDataTimeCol + aStr;
  }
  
  // 0-2. Simulation option.

  /**
   * <japanese>
   * <p>
   * [UƂ̑ΘbɂAOHyMoSJ IvV肷B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void setOhymosjOption() throws OhError {

    int opt;
    
    StringWriter sw = new StringWriter();
    PrintWriter  pw = new PrintWriter(sw);
    
    pw.print("# Key in simulation option." + NEWLINE);
    pw.print("# TimeControlOption" + NEWLINE);

    // ---------------------------------------------------------

    if (OhOption.getDefault().getTimeControlOption()
      == TimeControl.GO_BEYOND_TERMINAL_TIME) {
      pw.print("#   current selection: " +
        "GO_BEYOND_TERMINAL_TIME" + NEWLINE);
    }
    else if (OhOption.getDefault().getTimeControlOption() ==
      TimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME) {
      pw.print("#   current selection: " +
        "DO_NOT_GO_BEYOND_TERMINAL_TIME" + NEWLINE);
    }

    pw.print("#   select 1 or 2" + NEWLINE);
    pw.print("    1. GO_BEYOND_TERMINAL_TIME (Default)" + NEWLINE);
    pw.print("    2. DO_NOT_GO_BEYOND_TERMINAL_TIME" + NEWLINE);

    String message = sw.toString();

    pw.close();

    String textLine = OhInterface.getReply(message, "1");

    if (textLine.equals(null))
      throw new OhError("Quit");
    

    StringReader sr = new StringReader(textLine);

    try {
//    opt = sr.read();    //; Wrong!!! (2003.12.05)

      int c = sr.read();
      opt = Character.getNumericValue((char)c);
      
//;   ͂ꂽlȂꍇ -1 ԂB
//;   l𕉂łȂƂĕ\łȂꍇilȂǁj́A-
//;   2 ԂB
      
      switch (opt) {
        
      case 1:
        OhOption.getDefault().
          setTimeControlOption(TimeControl.GO_BEYOND_TERMINAL_TIME);
        break;
        
      case 2:
        OhOption.getDefault().
          setTimeControlOption(TimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME);
        break;
        
      default:
        OhOption.getDefault().
          setTimeControlOption(TimeControl.GO_BEYOND_TERMINAL_TIME);
        break;
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }


  // 1. registration of possession ports, possession element
  // objects and possession subsystem objects.

  /**
   * <japanese>
   * <p>
   * Šn registerSendPorts() ́A
   * Šn̒ۃ\bh
   * (Ref "OhTotalSystem#registerInputPorts()")
   * s悤ɒ`܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerSendPorts() throws OhError {
    registerInputPorts();
  }


  /**
   * <japanese>
   * <p>
   * Šn registerReceivePorts() ́A
   * Šn̒ۃ\bh
   * (Ref "OhTotalSystem#registerOutputPorts()")
   * s悤ɒ`܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerReceivePorts() throws OhError {
    registerOutputPorts();
  }


  // 2. initialization of total systems.

  //; 5 ̔{ɂȂĂ邩ǂ͂ł͊m߂ĂȂB

  /**
   * <japanese>
   * <br>
   * f[^쐬܂B
   *
   * <pre>
   * (
   * (0)
   * (1)
   * (2000)
   * (38600)
   * (2.19956e+07)
   * )
   * </pre>
   *
   * ̃tH[}bgŋLqĂf[^t@ĆA
   * f[^ "0 1 2000 38600 2.19956e+07" 擾ł܂B
   *
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @return a <code>String</code> value
   */
  public String makeDataString(OhBufferedReader aBr) {
    String dataString = null;
    String[] data = new String[1024];
    int i = 0;
    int k = 0;
    try {
      while ((data[i] = aBr.readWord()).equals(")") != true) {
        if ((data[i].startsWith("(")) == false) {
          System.err.println("Data file format error.");
          System.exit(1);
        }
        int length = data[i].length();
        data[i] = data[i].substring(1, length - 1);
        if (aBr.skipWhites() == -1) {
          System.err.println("Data file format error.");
          System.exit(1);
        }
        i++;
      }

      StringBuffer sb = new StringBuffer();

      for (k = 0; k < i; k++) {
        sb.append(data[k]);
        sb.append(" ");
      }

      dataString = sb.toString();
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    return dataString;
  }
        

  /**
   * <japanese>
   * <p>
   * evfɕ^ăp^܂͏l̐ݒB<p>
   *
   * f[^t@C͂āAevfɕ^ăp^
   * ݒ肷邩lݒ肷B
   *
   * content == PARSET ̂ƂAp^ݒA
   * content == INITIAL ̂ƂAlݒA
   * content == SDPDATA ̂ƂAM[qɎcĂf[^̍Đ
   *  <p>
   *
   * ̃NX readSendPortFile() ŎgpĂB
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @param aContent a <code>DataFileContent</code> value
   * @exception OhError if an error occurs
   */
  public void deliverDataString(
    String aFileName, DataFileContent aContent) throws OhError {
    FileReader fr;
    OhBufferedReader br;

    try {
      fr = new FileReader(aFileName);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      if (aContent == DataFileContent.SDPDATA) {
        return;
      }
      throw new OhError("File open error.", oPrintString("\t"));
    }

    int  current;
    char currentChar;
    
    int paren = 0;    //; ʁop̐[B
    
    Object element = null;
    Object owner = null;
    
    String dataStr;
    
    String[] objArray = new String[3];
      // objArray[0] = className, objArray[1] = objName.
    int objNum;

    try {
      
    LOOP:
      while (true) {  // ------------------- "LOOP" loop starts.
        current = br.skipWhites();
        if (current == -1)
          break LOOP;
        currentChar = (char)current;
        switch (current) {    // ---------------- switch starts.
        case '{':    //; owner ̕ύXBiPցj
          br.read();    //; ۂ '{' ǂݎB
          paren++;
          if ((owner = element) == null) {
            System.err.println("Trouble about '{' in data file.");
            System.exit(1);
          }
          element = this;
          continue LOOP;

        case '}':    //; owner ̕ύXBiPOցj
          br.read();    //; ۂ '}' ǂݎB
          if ((--paren) < 0) {
            System.err.println("Trouble about '}' in data file.");
            System.exit(1);
          }
          element = owner;
          continue LOOP;

        case '(':    //; element  _dataString ^B
          br.read();    //; ۂ '(' ǂݎB
          if (br.skipWhites() == -1) {
            System.err.println("Trouble about '(' in data file.");
            System.exit(1);
          }
          
          if (current == -1) {
            System.err.println("Trouble about '(' in data file.");
            System.exit(1);
          }

          dataStr = makeDataString(br);    //(*Original)

          ((OhElement)element).setDataString(dataStr);
          if (aContent == DataFileContent.SDPDATA)
            ((OhElement)element).restoreSendPortStock();

          ((OhElement)element).clearDataString();

          continue LOOP;

        case ')':
          System.err.println("Trouble about ')' in data file.");
          System.exit(1);

        default:
          objArray[0] = br.readWord();
          objArray[1] = br.readWord();
          objArray[2] = br.readWord();

          if (objArray[1].equals("(") || objArray[1].equals(")") ||
              objArray[1].equals("{") || objArray[1].equals("}") ||
              objArray[1].equals("")) {

            System.err.println("Trouble about ')' in data file.");
            System.exit(1);
          }

          if (objArray[2].equals("(") || objArray[2].equals(")") ||
              objArray[2].equals("{") || objArray[2].equals("}") ||
              objArray[2].equals("")) {
            objNum = 0;
            br.pushBack();
            //; '(' ȂǂǂݎȂƂɂăt@C|C
            //; ^P߂B
          }
          else {
            objNum = Integer.parseInt(objArray[2]);
          }

          if (objCompare(objArray[0], objArray[1], objNum) == true) {
            element = this;
            owner   = this;
          }
          else if ((element = ((OhElement)owner).
                     getComponent(objArray[0], objArray[1], objNum))
                     == null) {
            System.err.println("Trouble about ')' in data file.");
            System.exit(1);
          }

          continue LOOP;
          
        }    // ----------------------------------- switch ends.

      }    // -------------------------------- "LOOP" loop ends.

      if (paren != 0) {
        System.err.println("Trouble in data file.");
        System.exit(1);
      }

      br.close();
      return;

    }
    catch (OhError e) {
      System.err.println("Trouble in data file.");
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }

          
  /**
   * <japanese>
   * <p>
   * p^̓ǂݍ <p>
   *
   * Ŏw肳ꂽp^t@Cp^ǂݎ܂B
   * p^́AꊇĂP̃t@Cɏ܂ĂƂ
   * B<p>
   *
   * yp^t@C̗z
   * <pre>
   *   # data for test
   *   #
   *   TestOfElement totalSystem 0 
   *   3
   *   {
   *     Mountain ibuki 0 
   *     1.5 2
   *     
   *     Lake biwa 0 
   *     5
   *     {
   *         Sky sky 0 
   *         foo 2
   *     
   *         Outlet outlet 0 
   *         3.5 2
   *         3.1 8.0
   *     }
   *   }
   * </pre>
   *
   * PsQŝ悤ɁA# Ŏn܂s̓RgsłB
   *
   * <pre>
   * TestOfElement totalSystem 0 
   * Mountain ibuki 0 
   * Lake biwa 0 
   * Sky sky 0 
   * Outlet outlet 0
   * </pre>
   *
   * ̂悤ȍśAnfvffʂ邽߂̏
   * łBNXAIuWFNgAIuWFNgԍ̂R̏
   * 󔒂ŋ؂ď܂B<p>
   *
   * fʂ邽߂̂̏ɑāA󔒍sԂɒu
   * ɁÃfǂ݂Ƃׂ܂B̕
   * ́AsetParameter(OhBufferedReader) ǂݍޕłBs
   * ̃f[^Kv́Aǂݍ݃vO 
   * setParameter(OhBufferedReader) ɂČ܂܂Bf
   * ʂ邽߂̏sƁAǂ݂Ƃׂf[^̊ԂɁA
   * Ŏn܂RgsuƂł܂B<p>
   *
   * fʂ邽߂̏sƂɑf[^s
   * ł܂Ƃčl΁Ǎł܂ƕʂ̌ł܂Ƃ̊Ԃɂ́A
   * 󔒍s # Ŏn܂sĂ\܂B<p>
   *
   * Lake biwa 0 ̂悤ɁA\vfnf́A{ } 
   * ͂ŁA̍\vfɊւf[^܂Bnł
   * ĂA\vf̃p^ݒ̂߂̃f[^ǂݍ݂sv
   * A{ } ̕Kv͂܂B
   *
   * </japanese>
   *
   * @param aParamFile a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void readParameterFile(String aParamFile)
    throws OhError {

    FileReader       fr;
    OhBufferedReader br;

    try {
      fr = new FileReader(aParamFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      throw new OhError("cannot open \"" + aParamFile + "\"!");
    }

    try {
      
      String headLine = null;
      StringTokenizer strToken = null;
      String className = "";
      String objName = "";
      int objNum = 0;
      Object obj = null;

      if (br.skipWhites() == -1) {
        OhError e = new OhError("Parameter file error.");
        throw e;
      }
        
      headLine = br.readLine();
        
      String str = headLine;                       //; For work.
      StringTokenizer st = new StringTokenizer(str);
      int i = 0;
      while (st.hasMoreTokens()) {
        st.nextToken();
        i++;
      }

      strToken = new StringTokenizer(headLine);
        
      if (i == 1) {
        objName = strToken.nextToken();

        if (objCompare(objName, 0) == false) {
          throw new OhError(
            "readParameterFile(): unmatched element" + NEWLINE +
            "  " + className + "  " + objName + "  " + objNum +
            NEWLINE +
            oPrintString("    "));
        }
        
      }
      else if (i == 2) {
        objName = strToken.nextToken();
        objNum  = Integer.parseInt(strToken.nextToken());
        
        if (objCompare(objName, objNum) == false) {
          throw new OhError(
            "readParameterFile(): unmatched element" + NEWLINE +
            "  " + className + "  " + objName + "  " + objNum +
            NEWLINE + oPrintString("    "));
        }
        
      }
      else if (i == 3) {
        className = strToken.nextToken();
        objName   = strToken.nextToken();
        objNum    = Integer.parseInt(strToken.nextToken());

        if (objCompare(className, objName, objNum) == false) {
          throw new OhError(
            "readParameterFile(): unmatched element" + NEWLINE +
            "  " + className + "  " + objName + "  " + objNum +
            NEWLINE + oPrintString("    "));
        }
        
      }
      else {
        throw new OhError(
          "illegal line" + NEWLINE +
          "  " + className + "  " + objName + "  " + objNum +
          NEWLINE + oPrintString("      "));
      }

      super.getParameter(br);

    }
    catch (OhError e) {
      throw new OhError(
        "readParameterFile() method detected an OhError." + NEWLINE +
        "when reading \"" + aParamFile + "\"" + NEWLINE +
        e.what(" > "));
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("readParameterFile() method detected " +
        "an IOExeption." + NEWLINE);
      out.print("when reading \"" + aParamFile + "\"");
      String errStr= buffer.toString();
      out.close();
      throw new OhError(errStr);
    }
    finally {
      if (br != null) {
        try { br.close(); } catch (Exception e) {}
      }
    }
  }


  /**
   * <japanese>
   * <p>
   * l̓ǂݍ <p>
   *
   * Ŏw肳ꂽlt@C珉lǂݎ܂B
   * ĺAꊇĂP̃t@Cɏ܂ĂƂ܂B
    </japanese>
   *
   * @param aInitStateFile a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void readInitialStateFile(String aInitStateFile)
    throws OhError {
    FileReader       fr;
    OhBufferedReader br;

    try {
      fr = new FileReader(aInitStateFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      throw new OhError("cannot open " + aInitStateFile);
    }

    try {
      
      String headLine = null;
      StringTokenizer strToken = null;
      String className = "";
      String objName = "";
      int objNum = 0;
      Object obj = null;

      if (br.skipWhites() == -1) {
        OhError e = new OhError("Initial State file error.");
        throw e;
      }
        
      headLine = br.readLine();
        
      String str = headLine;                       //; For work.
      StringTokenizer st = new StringTokenizer(str);
      int i = 0;
      while (st.hasMoreTokens()) {
        st.nextToken();
        i++;
      }
        
      strToken = new StringTokenizer(headLine);

      if (i == 1) {
        objName = strToken.nextToken();

        if (objCompare(objName, 0) == false) {
          throw new OhError(
            "readInitialStateFile(): unmatched element" + NEWLINE +
            "  " + className + "  " + objName + "  " + objNum +
            NEWLINE + oPrintString("    ")
          );
        }
      }
      else if (i == 2) {
        objName = strToken.nextToken();
        objNum  = Integer.parseInt(strToken.nextToken());
        
        if (objCompare(objName, objNum) == false) {
          throw new OhError(
            "readInitialStateFile(): unmatched element" +
            NEWLINE + "  " + className + "  " + objName + "  " +
            objNum + NEWLINE + oPrintString("   "));
        }
        
      }
      else if (i == 3) {
        className = strToken.nextToken();
        objName   = strToken.nextToken();
        objNum    = Integer.parseInt(strToken.nextToken());

        if (objCompare(className, objName, objNum) == false) {
          throw new OhError(
            "readInitialStateFile(): unmatched element" +
            NEWLINE + "  " + className + "  " + objName + "  " +
            objNum + NEWLINE + oPrintString("   "));
        }
      }
      else {
        throw new OhError(
          "illegal line" + NEWLINE +
          "  " + className + "  " + objName + "  " +
          objNum + NEWLINE +
          oPrintString("      "));
      }

      super.getInitialState(br);
    }
    catch (OhError e) {
      throw new OhError(
        "readInitialStateFile() method detected an error" +
        NEWLINE + "when reading \"" + aInitStateFile + "\"" +
        NEWLINE +
        e.what(" > "));
    }
    catch (IOException e) {
      throw new OhError(
        "readInitialStateFile() method detected an error" +
        NEWLINE + "when reading \"" + aInitStateFile + "\""
        );
    }
    finally {
      if (br != null) {
        try { br.close(); } catch (Exception e) {}
      }
    }
  }

  /**
   * Read the data of the sending port recorded on the file.
   *
   * <japanese>
   * <p>
   * t@CɎ^ĂM[q̃f[^ǂݍ݂܂B
   * </japanese>
   *
   * @param aSendFile a <code>String</code> value
   */
  public void readSendPortFile(String aSendFile) throws OhError {
    deliverDataString(aSendFile, DataFileContent.SDPDATA);
  }


  // Setting of initial state.

  /**
   * <japanese>
   * <p>
   * _currentTime ̓ǂݍ <p>
   *
   * t@C _currentTime ǂ݂Ƃ܂B
   * KvȂApĂŠnfŕύXĂB
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setInitialState(OhBufferedReader aBr)
    throws OhError {
    int  year = 0;
    long tsec = 0L;
    double frac = 0.0;
    
    try {
      aBr.skipWhites();
      year = Integer.parseInt(aBr.readWord());
      tsec = Long.parseLong(aBr.readWord());
      frac = Double.parseDouble(aBr.readWord());
    }
    catch (IOException e) {
      throw new OhError("Error in setInitialState()." +
        NEWLINE + oPrintString("\t")
      );
    }
    catch (NumberFormatException e) {
      throw new OhError("Error in setInitialState()." +
        NEWLINE + oPrintString("\t"));
    }
    
    if (year > 0) {
      _currentTime = new OhTimePoint(year, tsec);
    }

    super.initialTime(_currentTime);
  }


  /**
   * <japanese>
   * <p>
   * (Ref "OhElement#shareInfo()") sB
   * </japanese>
   *
   * @return an <code>OperationFlag</code> value
   * @exception OhError if an error occurs
   */
  public OperationFlag doShareInfo() throws OhError {
    OperationFlag flag;

    boolean calcany = false;
    boolean calcall = true;

    
    if (_shareInfoDoneP == false) {
      if (shareInfo() == false) {
        throw new OhError(oPrintString("    "));
      }
      _shareInfoDoneP = true;
    }

    int i = 0;
    int k = 0;

    do {
      calcany = false;
      calcall = true;
    
      for (k = 0; k < _numberOfComponents; k++) {
        flag = ((OhElement)_componentArray[k]).doShareInfo();
        if (flag == OperationFlag.E_DONE)
          calcany = true;
        if (flag == OperationFlag.E_UNDONE)
          calcall = false;
      }
      
      for (i = 0; i < _numberOfIteratorSets; i++) {
        for (k = 0; k < _numberOfIterators[i]; k++) {
          flag = ((OhElement)_iteratorArray[i][k]).doShareInfo();
          if (flag == OperationFlag.E_DONE)
            calcany = true;
          if (flag == OperationFlag.E_UNDONE)
            calcall = false;
        }
      }
    } while (calcany == true);
    

    if (calcall == true)
      return OperationFlag.E_ALREADY;


    for (k = 0; k < _numberOfComponents; k++) {
      if ( ((OhElement)_componentArray[k]).doShareInfo()
        == OperationFlag.E_UNDONE ) {
        ((OhElement)_componentArray[k]).
          oPrint(new PrintWriter(System.err, true), "\t");
      }
    }

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if ( ((OhElement)_iteratorArray[i][k]).doShareInfo() ==
          OperationFlag.E_UNDONE ) {
          ((OhElement)_iteratorArray[i][k]).
            oPrint(new PrintWriter(System.err, true), "\t");
        }
      }
    }

    return OperationFlag.E_UNDONE;
  }


  // 3. Calculation of the total system model.

  /**
   * Returns "_terminalTime".
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getTerminalTime() {
    return _terminalTime;
  }


  /**
   * Returns "_mediumTermTargetTime"
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getMediumTermTargetTime() {
    return _mediumTermTargetTime;
  }



  /**
   * Inputs data time.
   *
   * @param aInputFrom an <code>OhTimePoint</code> value
   * @param aInputTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouInputDataTime(OhTimePoint aInputFrom,
    OhTimePoint aInputTo) {
    OhTimePointImmutable inputFrom = OhTimePoint.OhTimePointMin;
    OhTimePointImmutable inputTo   = OhTimePoint.OhTimePointMax;

    OhArrayList dpArray  = null;
    int         numOfDps = 0;

    OhTimePointImmutable buf = null;

    OhArrayList[] dpArrayArray  = {dpArray};
    int[]         numOfDpsArray = {numOfDps};

    for (int k = 0; k < _numberOfSendPorts; k++) {
      ((OhInputPort)_sendPortArray[k]).
        getDataPackArray(dpArrayArray, numOfDpsArray);
      dpArray  = dpArrayArray[0];
      numOfDps = numOfDpsArray[0];

      if (numOfDps <= 0)
        return false;

      buf = ((OhDataPack)dpArray.get(0)).getDataTime();

      inputFrom = (aInputFrom.compareTo(buf) > 0) ? inputFrom : buf;

      buf = ((OhDataPack)dpArray.get(numOfDps - 1)).getDataTime();

      inputTo = (aInputTo.compareTo(buf) > 0) ? inputTo : buf;
    }

    aInputFrom.copy(inputFrom);
    aInputTo.copy(inputTo);

    return true;
  }


  /**
   * Orders all input ports to read data.
   *
   * <japanese>
   * <p>
   * O̓ <p>
   *
   * ׂĂ̓͒[qɃf[^ǂݍނ悤Ɏw߂܂B
   * (Ref "OhTotalSystem#initialWork()"), 
   * (Ref "OhTotalSystem#work()")
   * Ă΂܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void inputData() throws OhError {
    for (int k = 0; k < _numberOfSendPorts; k++) {
      ((OhInputPort)_sendPortArray[k]).inputData();
      // (Ref "OhInputFile#inputData()") will be called. 
    }
  }

  
  /**
   * Orders all output ports to write out their data.
   *
   * <japanese>
   * <p>
   * o͒[qւ̃f[^̏o <p>
   *
   * ׂĂ̏o͒[qɃf[^o悤Ɏw߂܂B
   * ʂ́At@Cɏo͂܂B̃\bh́A
   * (Ref "OhTotalSystem#work()") 
   * Ă΂܂B̃\bh̓ŁA
   * (Ref "OhOutputPort#outputData()") Ăł܂B
   * <p>
   *
   * </japanese>
   *
   * @exception IOException if an error occurs
   */
  public void outputData() throws IOException {
    for (int k = 0; k < _numberOfReceivePorts; k++) {
      ((OhOutputPort)_receivePortArray[k]).outputData();
    }
  }


  /**
   * Usually there is no output from total system.
   *
   * <japanese>
   * <p>
   * ̏o <p>
   *
   * ԗʂ߂ɁA_currentTime ł̏o͂`܂B
   * M[qɎƏo͒l݂܂B_currnet_time ł̏o
   * ͂vZłꍇ true Ԃ܂BKvȓ̓f[^
   * ĂȂȂǂāA_currentTime ̏o͂vZłȂ
   * ꍇ false Ԃ܂B<p>
   *
   * ȂȀꍇ͕n̏̏o͂͂ȂƎv
   * ̂ŁAł͏ true Ԃ悤ɒ`Ă܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   */
  public boolean initialOutput() throws OhError {
    return true;
  }


  /**
   * <japanese>
   * <p>
   *  <p>
   *
   * ɂāASf[^擾܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean initialWork() throws OhError {    //(*11)
    
    if (_initialWorkDoneP == false) {
      if (initialOutput() == false) {
        throw new OhError(
          "Cannot do initialOutput() for total system.");
      }
      _initialWorkDoneP = true;
    }

    inputData();

    boolean calCall;

    while (true) {

      int i;
      int k;
      OperationFlag flag = null;
      boolean calcany = false;
      boolean calcall = true;
      
      for (k = 0; k < _numberOfComponents; k++) {
        flag = ((OhElement)_componentArray[k]).doInitialWork(); 
        
        if (flag == OperationFlag.E_DONE)
          calcany = true;
        if (flag == OperationFlag.E_UNDONE)
          calcall = false;
      }

      for (i = 0; i < _numberOfIteratorSets; i++) {
        for (k = 0; k < _numberOfIterators[i]; k++) {
          flag = ((OhElement)_iteratorArray[i][k]).doInitialWork();
          
          if (flag == OperationFlag.E_DONE)
            calcany = true;
          if (flag == OperationFlag.E_UNDONE)
            calcall = false;
        }
      }
      
      if (calcany == true)
        continue;

      calCall = calcall;
      break;
    }

    if (calCall == true) {
      initializeTargetTime();    // Initialize "_targetTime".
      return true;
    }

    // Error process.
    //
    int i = 0;
    int k = 0;
    for (k = 0; k < _numberOfComponents; k++) {
      if (((OhElement)_componentArray[k]).doInitialWork() ==
        OperationFlag.E_UNDONE)
        ((OhElement)_componentArray[k]).
          oPrint(new PrintWriter(System.err, true), "\t");
    }
    
    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if (((OhElement)_iteratorArray[i][k]).doInitialWork() ==
          OperationFlag.E_UNDONE) {
          ((OhElement)_iteratorArray[i][k]).
            oPrint(new PrintWriter(System.err, true), "\t");
        }
      }
    }
    
    System.out.println("OhTotalSystem.initialWork()");
    
    return false;
  }



  /**
   * <japanese>
   * <p>
   * _currentTime ԂB<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[
   * ^pbNM[q aRp MĂȂ΁Avf͌vZ
   * Jn邱Ƃł܂B
   * </japanese>
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeFrom() {
    return _currentTime;
  }

  /**
   * <japanese>
   * <p>
   * _terminalTime ԂB<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[
   * ^pbNM[q aRp MĂȂ΁Avf͌vZ
   * Jn邱Ƃł܂B
   * </japanese>
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeTo() {
    return _terminalTime;
  }


  /**
   * <japanese>
   * <p>
   * V~[VڕWV~[Vs҂ƑΘb
   * 肷B
   * </japanese>
   *
   * @return a <code>long</code> value
   * @exception OhError if an error occurs
   */
  public OhTime calculateTimeStep() throws OhError {
    long timeStep;
    OhTime ts;

    while (true) {    // -------------------- while loop starts.
    
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);

      pw.print(_newestDataTimeCol +
        "# Key in time step or terminal time for \'total system\'.");
      pw.print(NEWLINE);
      pw.print("# current time (year sec frac) = "
        + _currentTime.getYear() + " "
        + _currentTime.getTsec() + " "
        + _currentTime.getFrac());
      pw.print(NEWLINE);
      pw.print(
        "# time step (day hour min sec frac) or terminal time (year sec frac)");
      pw.print(NEWLINE);
      
      StringWriter dsw = new StringWriter();
      PrintWriter  dpw = new PrintWriter(dsw);
      
      dpw.print(_currentTime.getYear() + " "
        + _currentTime.getTsec() + " "
        + _currentTime.getFrac());

      String message = sw.toString();
      String reply   = dsw.toString();

      pw.close();
      dpw.close();
      
      String textLine = OhInterface.getReply(message, reply);

      if (textLine == null)
        throw new OhError("Quit.");

      String str = textLine;
      StringTokenizer st = new StringTokenizer(str);
      int i = 0;
      while (st.hasMoreTokens()) {
        st.nextToken();
        i++;
      }

      StringTokenizer strToken = new StringTokenizer(textLine);
        
      if (i == 5) {
        int day  = Integer.parseInt(strToken.nextToken());
        int hour = Integer.parseInt(strToken.nextToken());
        long min = Long.parseLong(strToken.nextToken());
        long sec = Long.parseLong(strToken.nextToken());
        double frac = Double.parseDouble(strToken.nextToken());
        timeStep = sec + 60 * (min + 60 * (hour + 24 * day));
        ts = new OhTime(timeStep, frac);

        if (ts.toDouble() <= 0.0) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("OHyMoSj pause:" + NEWLINE);
          out.print("You inputed:   " + textLine + NEWLINE);
          out.print("But this produces non-positive time step");
          String pauseComment = buffer.toString();
          out.close();

          OhInterface.pause(pauseComment);

          continue;
        }
        break;
      }
      else if (i == 4) {
        int hour = Integer.parseInt(strToken.nextToken());
        long min = Long.parseLong(strToken.nextToken());
        long sec = Long.parseLong(strToken.nextToken());
        double frac = Double.parseDouble(strToken.nextToken());
        timeStep = sec + 60 * (min + 60 * hour);
        ts = new OhTime(timeStep, frac);
        
        if (ts.toDouble() <= 0.0) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("OHyMoSj pause:" + NEWLINE);
          out.print("You inpupted:   " + textLine + NEWLINE);
          out.print("But this produces non-positive time step");
          String pauseComment = buffer.toString();
          out.close();

          OhInterface.pause(pauseComment);

          continue;
        }

        break;
      }
      else if (i == 3) {
        int  year = Integer.parseInt(strToken.nextToken());
        long tsec = Long.parseLong(strToken.nextToken());
        double frac = Double.parseDouble(strToken.nextToken());
        OhTimePoint tp = new OhTimePoint(year, tsec, frac);
        ts = OhTimePoint.sub(tp, _currentTime);
     
        if (ts.toDouble() <= 0) {
          OhInterface.pause(
            "OHyMoSj pause:" + NEWLINE +
            "You inputed:   " + textLine + NEWLINE +
            "But this produces non-positive time step"
          );
          
          continue;
        }
   
        break;
      }
      else {
        OhInterface.pause("# Invalid input" + NEWLINE);
        continue;
      }
      
    }    // ----------------------------------- while loop ends.

    return ts;
  }
    

  /**
   * <japanese>
   * <p>
   * PXebv̌vZ
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean calculate() throws OhCannotGetDataError, OhError {
    return true;
  }

  /**
   * <japanese>
   * <p>
   * o͗p[qpāAŠnɑ΂o͂sȂB
   * ܂A\vfɌvZs߂𑗂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean work() throws OhCannotGetDataError, OhError { 
    boolean calc    = false;
    boolean flagAll = false;
    boolean flagItr = false;
    int i = 0;
    int j = 0;
    int k = 0;
    char reply;
    int[] itrCounter = new int[_numberOfIteratorSets];
    OhTime mtti;
      // "Medium Term Target Time" increment.

    try {
      
    LOOP:
      while(true) {      // ----------------------- LOOP starts.
        
        _timeStep = calculateTimeStep(); 
        
        while(true) {    // -------------- Infinite loop starts.

          StringWriter sw = new StringWriter();
          PrintWriter  pw = new PrintWriter(sw);
    
          pw.print("# _timeStep = " + _timeStep + " (sec)");
          pw.print(NEWLINE);
          pw.print(
            "# Key in \"Medium Term Target Time\" increment");
          pw.print(NEWLINE);
          pw.print("#   (sec frac) or (min sec frac) or " +
            "(hour min sec frac) " +
            "or (day hour min sec frac) ");
          pw.print(NEWLINE);
      

          StringWriter dsw = new StringWriter();
          PrintWriter  dpw = new PrintWriter(dsw);
      
          dpw.print(_timeStep);

          String message        = sw.toString();
          String defaultMessage = dsw.toString();

          pw.close();
          dpw.close();
      
          String textLine = OhInterface.getReply(message, defaultMessage);

          mtti = new OhTime(0, 0.0);

          if (textLine == null)
            throw new OhError("Quit.");
          
          String str = textLine; // For work.
          StringTokenizer st = new StringTokenizer(str);
          int a = 0;
          while (st.hasMoreTokens()) { // "n_words(textLine)"
            st.nextToken();
            a++;
          }
          
          StringTokenizer strToken = new StringTokenizer(textLine);
        
          if (a == 5) {
            int day  = Integer.parseInt(strToken.nextToken());
            int hour = Integer.parseInt(strToken.nextToken());
            long min = Long.parseLong(strToken.nextToken());
            long sec = Long.parseLong(strToken.nextToken());
            double frac = Double.parseDouble(strToken.nextToken());
            mtti = new OhTime(
              sec + 60 * (min + 60 * (hour + 24 * day)), frac);
          }
          else if (a == 4) {
            int hour = Integer.parseInt(strToken.nextToken());
            long min = Long.parseLong(strToken.nextToken());
            long sec = Long.parseLong(strToken.nextToken());
            double frac = Double.parseDouble(strToken.nextToken());
            mtti = new OhTime(sec + 60 * (min + 60 * hour), frac);
          }
          else if (a == 3) {
            long min = Long.parseLong(strToken.nextToken());
            long sec = Long.parseLong(strToken.nextToken());
            double frac = Double.parseDouble(strToken.nextToken());
            mtti = new OhTime(sec + 60 * min, frac);
          }
          else if (a == 2) {
            long sec = Long.parseLong(strToken.nextToken());
            double frac = Double.parseDouble(strToken.nextToken());
            mtti = new OhTime(sec, frac);
          }
          
          if (mtti.toDouble() > 0.0)    //; Escapes from an infinite loop.
            break;

          OhInterface.pause(
            "# Invalid input : " +
            "\"Medium Term Target Time\" increment <= 0");
          
        }    // ---------------------------- Infinite loop ends.

        _terminalTime = OhTimePoint.add(_currentTime, _timeStep);
        _mediumTermTargetTime = new OhTimePoint(_currentTime);
        

//;     System.out.println("_currentTime = " + _currentTime);                    //; 0
//;     System.out.println("_mediumTermTargetTime = " + _mediumTermTargetTime);  //; 0
//;     System.out.println("_terminalTime = " + _terminalTime);                  //; 100
//;     System.out.println("mtti = " + mtti);                                    //; 50000
        
/* test
        OhTimePoint a = new OhTimePoint(2001, 100);
        OhTimePoint b = new OhTimePoint(2000, 30);
        System.out.println("a = " + a);                    //; 2000 0
        System.out.println("b = " + b);                    //; 2000 0
        long kkk = a.subtract(b);
        System.out.println("a = " + a);                    //; 2000 70
        System.out.println("b = " + b);  //; 2000 30
        System.out.println("kkk = " + kkk);
/* test */
        
//;     System.out.println("_currentTime" + _currentTime);  //; 2000 0
//;     System.out.println("_mediumTermTargetTime = " + _mediumTermTargetTime);  //; 2000 100
//;     System.out.println("mtti" + mtti);  //; 50000

        try {
      WORK:
        while(true) {  // ------------------------- WORK starts.
          
          calc = false;
          for (i = 0; i < _numberOfIteratorSets; i++)
            itrCounter[i] = 0;

          // Increments _mediumTermTargetTime by mtti
          _mediumTermTargetTime.add(mtti);
            
          if (_currentTime.compareTo(_mediumTermTargetTime) > 0) {
            if (_currentTime.compareTo(_terminalTime) >= 0)
              break WORK;
            else
              continue WORK;
          }

          do {
            flagAll = false;

            for (k = 0; k < _numberOfComponents; k++) {
              if (((OhElement)_componentArray[k]).work() == true) {
                // Abstract method call. "OhCannotGetDataError".
                calc = flagAll = true;
                break;                // Escapes from this for loop.
              }
            }

            while (++k < _numberOfComponents)
              ((OhElement)_componentArray[k]).work();
              // Abstract method call. Throws "OhCannotGetDataError".


            // _iteratorArray
            //
            for (i = 0; i < _numberOfIteratorSets; i++) {
              // ---------------------------- for loop 1 starts.

              do {
                flagItr = false;

                for (j = 0; j < _numberOfIterators[i]; j++) {
                  if (((OhElement)_iteratorArray[i][j]).work() == true) {
                    // Abstract method call."OhCannotGetDataError"
                    calc = flagAll = flagItr = true;
                    break; // Escapes from this for loop.
                  }
                }

                while (++j < _numberOfIterators[i])
                  ((OhElement)_iteratorArray[i][j]).work();
                  // abstract method call which may throw
                  // (Ref "OhCannotGetDataError").
              } while (flagItr == true);

              if (hasReachedTargetTime(i) == true) {
                if (hasConverged(i) == true) {
                  flushData(i);               
                  renewFixedTime(i);          
                  renewTargetTime(i, calculateIterationStep(i));
                  itrCounter[0] = 0;
                }
                else {
                  rewindCurrentTime(i);

                  if (++itrCounter[i] >= 1000) {
                    throw new OhError(
                      "\tin group [" + i + "]." + NEWLINE +
                      oPrintString("\t")
                    );
                  }
                }
              }
            }    // --------------------------- for loop 1 ends.

          } while (flagAll == true);

          
          if (calc == false) {
            System.out.print(NEWLINE);
            System.out.print("# No object has worked." + NEWLINE);
            break WORK;
          }

          renewCurrentTime();
            
          if (_currentTime.compareTo(_mediumTermTargetTime) < 0) {
            System.err.println(NEWLINE);
            System.err.println(
              "# Error : Stopped before \"MediumTermTargetTime\"." +
              NEWLINE);
            break WORK;
          }
          
          if (_mediumTermTargetTime.compareTo(_terminalTime) < 0) {
            continue WORK;
          }
          
          outputData();

          break WORK;
          
        }  // --------------------------------------- WORK ends.

        } catch (OhCannotGetDataError e) {
          OhInterface.pause("# ERROR : Now in catch block. ");
          outputData();
        }


      ASK:
        while(true) {
          
          StringWriter sw = new StringWriter();
          PrintWriter  pw = new PrintWriter(sw);
    
          pw.print(NEWLINE);
          pw.print(
            "# Now : _currentTime = " + _currentTime +
            " : _terminalTime = " + _terminalTime);
          pw.print(NEWLINE);
          pw.print("# Quit? Key in [Y(y)] or [N(n)]");
          pw.print(NEWLINE);

          String message = sw.toString();

          pw.close();
      
          String textLine = OhInterface.getReply(message, "y");

          if (textLine == null)
            throw new OhError("Quit.");

          StringReader     sr = new StringReader(textLine);
          OhBufferedReader br = new OhBufferedReader(sr);

          int c = br.read();
            //; 󂫕i""ĵƂ -1 ԂB

          reply = (char)c;
          
          switch(reply) {
          case 'y':
          case 'Y':
            break LOOP;
            
          case 'n':
          case 'N':
            inputData();
            continue LOOP;
          default:
            System.out.print(
              "# Invalid input : Key in \'Y\' or \'N\'" +
              NEWLINE + NEWLINE);
            continue ASK;
          }

        }  // ----------------------------------- ASK loop ends.

      }  // ----------------------------------------- LOOP ends.

    }  // -------------------------------------------- try ends.

    catch (IOException e) {
      OhInterface.pause("# Error : Now in catch block" + NEWLINE);
    }

    return true;
  }


  // 4. Save function of the last state.

  /**
   * Write "_currentTime".
   *
   * <japanese>
   * <p>
   * _currentTime ̏ <p>
   *
   * _currentTime ݂܂B
   * KvȂApĂŠnfŕύXĂB
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveTerminalState(PrintWriter aPw) {
    aPw.print(_currentTime);
  }

  /**
   * Saves to the file of the last state.
   *
   * <japanese>
   * <p>
   * ŏIԂ̃t@Cւ̕ۑ <p>
   *
   * t@C aFimeName ijVKɃI[vāAŠn̍
   * IԂۑ܂Bt@C aFimeName łɑ݂ꍇ
   * ͏㏑܂B<p>
   *
   * ŁA
   * (Ref "OhSubSystem#terminalState(PrintWriter)")
   * ǂł܂B
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void writeTerminalState(String aFileName)
    throws OhError {
    FileWriter     fw = null;
    BufferedWriter bw = null;
    PrintWriter    pw = null;
    
    try {
      fw = new FileWriter(aFileName);
      bw = new BufferedWriter(fw);
      pw = new PrintWriter(bw);

      pw.print(
        "# terminal state of " + _objectName  + " " + NEWLINE);
      pw.print(
        "# _currentTime =    " + _currentTime + " " + NEWLINE);

      super.terminalState(pw);

      pw.close();
    }
    catch (IOException e) {
      throw new OhError(oPrintString("\t"));
    }
  }


  /**
   * Stores the data packs which remain in send ports into a
   * file withe the specified file name.  If the file with the
   * file name has been created already, it will be
   * overwritten.<p>
   *
   * <japanese>
   * <p>
   * o͒[qɎcĂf[^̕ۑ <p>
   *
   * t@C aFileName ijVKɃI[vāA
   * o͒[qɎcĂf[^t@Cɕۑ܂B
   * t@C aFilefName łɑ݂ꍇ͏㏑܂B<p>
   *
   * ŁA
   * (Ref "OhElement#saveSendPortStock(PrintWriter)")
   * (Ref "OhSubSystem#saveSendPortStock(PrintWriter)")
   * Ăł܂B
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void writeSendPortStock(String aFileName)
    throws OhError {
    FileWriter     fw = null;
    BufferedWriter bw = null;
    PrintWriter    pw = null;
  
    try {
      fw = new FileWriter(aFileName);
      bw = new BufferedWriter(fw);
      pw = new PrintWriter(bw);

      pw.print(
        "# send port data of  " + _objectName + " " + NEWLINE);
      pw.print(
        "# _currentTime =     " + _currentTime + " " +
        NEWLINE + "# " + NEWLINE);

      ((OhElement)this).saveSendPortStock(pw);

      int i = 0;
      int k = 0;

      if ((_numberOfComponents > 0) || (_numberOfIteratorSets > 0)) {
        pw.print("{" + NEWLINE);

        for (k = 0; k < _numberOfComponents; k++) {
          ((OhElement)_componentArray[k]).saveSendPortStock(pw);
        }

        for (i = 0; i < _numberOfIteratorSets; i++) {
          for (k = 0; k < _numberOfIterators[i]; k++) {
            ((OhElement)_iteratorArray[i][k]).saveSendPortStock(pw);
          }
        }

        pw.print("}" + NEWLINE);
      }

      pw.close();
      
    }
    catch (IOException e) {
      throw new OhError(oPrintString("\t"));
    }
  }
    
      
  // 5. Information display function.

  /**
   * The input of output port information.
   *
   * <japanese>
   * <p>
   * ͒[q̏o
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> o͐̃t@CXg[B
   *  EaHead -> õ͎wb_B
   * </pre>
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printInputs(PrintWriter aPw, String aHead) {
    printObjects(aPw, aHead, "Inputs :",
      _sendPortArray, _numberOfSendPorts);
  }

  /**
   * The input of output port information.
   *
   * <japanese>
   * <p>
   * ͒[q̏o
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> o͐̃t@CXg[B
   * </pre>
   *
   * õ͎wb_͂SJɂȂ܂BB
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void printInputs(PrintWriter aPw) {
    printObjects(aPw, "    ", "Inputs :",
      _sendPortArray, _numberOfSendPorts);
  }

  /**
   * The output of output port information.
   *
   * <japanese>
   * <p>
   * o͒[q̏o
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> o͐̃t@CXg[B
   *  EaHead -> õ͎wb_B
   * </pre>
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printOutputs(PrintWriter aPw, String aHead) {
    printObjects(aPw, aHead, "Outputs :",
      _receivePortArray, _numberOfReceivePorts);
  }

  /**
   * <japanese>
   * <p>
   * o͒[q̏o
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> o͐̃t@CXg[B
   * </pre>
   *
   * õ͎wb_͂SJɂȂ܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void printOutputs(PrintWriter aPw) {
    printObjects(aPw, "    ", "Outputs :",
      _receivePortArray, _numberOfReceivePorts);
  }

} // OhTotalSystem
