/**
 * @(#) TimeControl; class to express TimeControl.
 */

package ohymosj;

import java.util.*;
import java.io.*;


/**
 * TimeControl;
 *
 * TimeControl class. <p>
 *
 * Definition of TimeControl. <p>
 *
 * <dl>
 *  <dt> GO_BEYOND_TERMINAL_TIME
 *  <dd> Can go beyond the teminal time
 *  <dt> DO_NOT_GO_BEYOND_TERMINAL_TIME
 *  <dd> Cannot go beyond the terminal time
 * </dl>
 *
 * <japanese>
 * <p>
 * ԖڕW𒴂ČvZ邩ۂ\NXB<p>
 *
 * ^ɈSȒ萔̃NXłB<p>
 *
 * <ul>
 *  <li> GO_BEYOND_TERMINAL_TIME        : ԖڕW𒴂ČvZB
 *  <li> DO_NOT_GO_BEYOND_TERMINAL_TIME : ԖڕW𒴂ɌvZB
 * </ul>
 *
 * ԖڕWƂ́A
 * ڕW (Ref "OhTotalSystem#_terminalTime") ȑO
 * iʂȏꍇƂāA(Ref "OhTotalSystem#_terminalTime")
 * ɈvĂ悢j̎ŁASĂ̗vf͂̎𒴂邱
 * ڕWɌvZi߂ĂƂłB<p>
 *
 * ԖڕẂA
 * (Ref "OhTotalSystem") ̃oϐ
 * (Ref "OhTotalSystem#_mediumTermTargetTime") Ƃ
 * ^Ă܂B<p>
 *
 * (Ref "OhTotalSystem#_terminalTime") ȑOɒԖڕWݒ肵A
 * SĂ̗vf͒ԖڕW܂ŌvZΐ摖肹A
 * ̗vf̌vZ҂悤ɂ̂悤ɂĂ܂B<p>
 *
 * ܂AԖڕW̓ɂāAŠn̗vfׂĂ
 * ŌvZi߂lɂȂłȂAʂ̍팸}
 * ܂B܂AvfXgbNĂ˂΂ȂȂf[^pbN
 * ́AԖڕW玟̒ԖڕW܂ł̎ԕł悭A
 * vZԂɂĎgpʂϓ邱ƂȂȂƍl
 * ܂B
 *
 * </japanese>
 * 
 *
 * @author satoh, shiiba
 * @version $Revision: 1.8 $ $Date: 2004/01/19 07:49:25 $
 */
public final class TimeControl implements Serializable
{
  // @begin {instance variable or static variable} -------------

  private static int nextOrdinal = 0;
  private final  int ordinal     = nextOrdinal++;

  /**
   * Can go beyond the teminal time
   *
   * <japanese>
   * <p>
   * ԖڕW𒴂ČvZB
   * </japanese>
   *
   */
  public static final TimeControl GO_BEYOND_TERMINAL_TIME = new TimeControl();

  /**
   * Cannot go beyond the terminal time
   *
   * <japanese>
   * <p>
   * ԖڕW𒴂ɌvZB
   * </japanese>
   *
   */
  public static final TimeControl DO_NOT_GO_BEYOND_TERMINAL_TIME
    = new TimeControl();

  private static final TimeControl[] PRIVATE_VALUES =
    {GO_BEYOND_TERMINAL_TIME, DO_NOT_GO_BEYOND_TERMINAL_TIME};
  
  // @end {instance variable or static variable} ---------------
  
  
  // @begin {constructors} -------------------------------------
  
  /**
   * Creates a new default <code>TimeControl</code> instance.
   *
   */
  private TimeControl() {}

  // @end {constructors} ---------------------------------------

  
  // @begin {methods} ------------------------------------------

  private Object readResolve() throws ObjectStreamException {
    return PRIVATE_VALUES[ordinal];
  }

  // @end {methods} --------------------------------------------

} // TimeControl
